-- Created by Giseldah

local p = _G; --p stands for package

-- ****************************** Start CalcStat ******************************

-- Floating point numbers bring errors into the calculation, both inside the Lotro-client and in this function collection. This is why a 100% match with the stats in Lotro is impossible.
-- Anyway, to compensate for some errors, we use a calculation deviation correction value. This makes for instance 24.49999999 round to 25, as it's assumed that 24.5 was intended as outcome of a formula.
DblCalcDev = 0.00000001;

function CalcStat(SName, SLvl, SParam)

	-- process parameters and parameter defaults

	if type(SName) ~= "string" then
		return "Missing stat name";
	end
	local SN = string.match(SName:upper(),"^%s*(-?%a+%d*%a+)%s*$"); -- allow nested digits in stat name and a starting -
	if SN == nil then
		SN = string.match(SName,"^%s*(-?%a+)%s*$"); -- to allow single character stat name
		if SN == nil then
			return "Illegal stat name";
		end
	end
	SN = string.upper(SN);

	local L = 1; -- default L
	if type(SLvl) == "number" then
		L = SLvl;
	elseif type(SLvl) ~= "nil" then
		return "Illegal level";
	end
	local Lm = L-DblCalcDev;
	local Lp = L+DblCalcDev;

	local N = 1; -- default N
	local C = ""; -- default C
	if type(SParam) == "number" then
		N = SParam;
	elseif type(SParam) == "string" then
		C = SParam;
	elseif type(SParam) ~= "nil" then
		return "Illegal N or C";
	end

	-- binary search tree (generated code)

	if SN < "MINSTRELCDMIGHTTOOUTHEAL" then
		if SN < "CREEPAUDACITYCOST" then
			if SN < "BRGREVWEAKNRESIST" then
				if SN < "BEORNINGCDBASENCPR" then
					if SN < "AWARDILVLED" then
						if SN < "ARMQTYMP" then
							if SN < "ARMOURADJBYTYPE" then
								if SN < "ADJUMBARTRAITMIT" then
									if SN < "ACIDMITT" then
										if SN > "-VERSION" then
											if SN == "ACIDMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "-VERSION" then
											return "2.4f";
										else
											return 0;
										end
									elseif SN > "ACIDMITT" then
										if SN < "ADJUMBARITEMMIT" then
											if SN == "ADJUMBARITEM" then
												if Lm <= 450 then
													return 1;
												elseif Lm <= 550 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "ADJUMBARITEMMIT" then
											if SN == "ADJUMBARTRAIT" then
												if Lm <= 140 then
													return 1;
												elseif Lm <= 150 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											if Lm <= 450 then
												return 1;
											elseif Lm <= 499 then
												return 0.78;
											elseif Lm <= 550 then
												return 0.7;
											else
												return 1;
											end
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								elseif SN > "ADJUMBARTRAITMIT" then
									if SN < "ARMCATMP" then
										if SN < "AGILITYC" then
											if SN == "AGILITY" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "AGILITYC" then
											if SN == "AGILITYT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MainC",L,N);
										end
									elseif SN > "ARMCATMP" then
										if SN < "ARMOUR" then
											if SN == "ARMCATPROGB" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("ProgBArmourHeavy",N);
												elseif Lm <= 2 then
													return CalcStat("ProgBArmourMedium",N);
												else
													return CalcStat("ProgBArmourLight",N);
												end
											else
												return 0;
											end
										elseif SN > "ARMOUR" then
											if SN == "ARMOURADJ" then
												return CalcStat("ArmourAdjByType",ArmCodeIndex(C,2),L);
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("ArmourPntMP","ItemPntS","ArmourProgB","ArmourAdj",L,C,0));
										end
									else
										return CalcStat("PntMPArmour",L);
									end
								else
									if Lm <= 140 then
										return 1;
									elseif Lm <= 141 then
										return 0.78;
									else
										return 0.7;
									end
								end
							elseif SN > "ARMOURADJBYTYPE" then
								if SN < "ARMOURPENT" then
									if SN < "ARMOURC" then
										if SN < "ARMOURADJTYPEOTHER" then
											if SN == "ARMOURADJTYPECLOAK" then
												return CalcStat("AdjUmbarItemMit",L);
											else
												return 0;
											end
										elseif SN > "ARMOURADJTYPEOTHER" then
											if SN == "ARMOURADJTYPESHIELD" then
												return CalcStat("AdjUmbarItemMit",L);
											else
												return 0;
											end
										else
											return CalcStat("AdjUmbarItemMit",L);
										end
									elseif SN > "ARMOURC" then
										if SN < "ARMOURLOW" then
											if SN == "ARMOURCRAW" then
												return StatLinInter("PntMPArmourC","CreepPntS","ProgBArmourC","",L,N,99);
											else
												return 0;
											end
										elseif SN > "ARMOURLOW" then
											if SN == "ARMOURLOWPNTMP" then
												return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyLowMP",ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("ArmourLowPntMP","ItemPntS","ArmourProgB","ArmourAdj",L,C,0));
										end
									else
										return StatLinInter("PntMPArmourC","CreepPntS","ProgBArmourC","",L,N,0);
									end
								elseif SN > "ARMOURPENT" then
									if SN < "ARMQTYCOMMMP" then
										if SN < "ARMOURPROGB" then
											if SN == "ARMOURPNTMP" then
												return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyMP",ArmCodeIndex(C,3),ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
											else
												return 0;
											end
										elseif SN > "ARMOURPROGB" then
											if SN == "ARMOURT" then
												return StatLinInter("PntMPArmourT","TraitPntS","ProgBArmour","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("ArmCatProgB",ArmCodeIndex(C,1),L);
										end
									elseif SN > "ARMQTYCOMMMP" then
										if SN < "ARMQTYINCOMPMP" then
											if SN == "ARMQTYEPICMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1,1,1,1,1.002,1,1},L);
												end
											else
												return 0;
											end
										elseif SN > "ARMQTYINCOMPMP" then
											if SN == "ARMQTYLOWMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({12/33,12/33,12/33,12/33,12/33,0.996*12/33,12/33,12/33},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({28/30,28/30,0.95,0.95,0.94,0.948,0.95,0.95},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({22/30,22/30,0.8,0.8,0.76,0.804,0.8,0.7},L);
										end
									end
								else
									return StatLinInter("PntMPArmourPenT","TraitPntS","ProgBArmour","",L,N,0);
								end
							else
								if 3 <= Lp and Lm <= 3 then
									return CalcStat("ArmourAdjTypeCloak",N);
								elseif 8 <= Lp and Lm <= 8 then
									return CalcStat("ArmourAdjTypeShield",N);
								else
									return CalcStat("ArmourAdjTypeOther",N);
								end
							end
						elseif SN > "ARMQTYMP" then
							if SN < "AWARDILVLCC" then
								if SN < "AWARDILVLAC" then
									if SN < "AUTOLVLTOILVL" then
										if SN < "ARMQTYUNCOMMP" then
											if SN == "ARMQTYRAREMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({26/30,26/30,0.9,0.9,0.88,0.9,0.9,0.8},L);
												end
											else
												return 0;
											end
										elseif SN > "ARMQTYUNCOMMP" then
											if SN == "ARMTYPEMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({9,9,20,30,15,25,12,32.726},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({22/30,22/30,0.85,0.85,0.82,0.852,0.85,0.75},L);
											end
										end
									elseif SN > "AUTOLVLTOILVL" then
										if SN < "AWARDILVLAA" then
											if SN == "AWARDILVLA" then
												if Lm <= 75 then
													return L;
												elseif Lm <= 94 then
													return 5*L-304;
												elseif Lm <= 95 then
													return 173;
												elseif Lm <= 100 then
													return 5*L-304;
												elseif Lm <= 101 then
													return 197;
												elseif Lm <= 104 then
													return 4*L-206;
												elseif Lm <= 105 then
													return 218;
												elseif Lm <= 110 then
													return RoundDbl(LinFmod(1,295,303.8,106,110,L));
												elseif Lm <= 115 then
													return 304;
												elseif Lm <= 119 then
													return RoundDbl(LinFmod(1,345,357,116,119,L));
												elseif Lm <= 120 then
													return 359;
												elseif Lm <= 130 then
													return 395;
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,445.4,455.4,131,140,L));
												elseif Lm <= 150 then
													return RoundDbl(LinFmod(1,479.4,491.4,141,150,L));
												else
													return CalcStat("AwardILvlA",150);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLAA" then
											if SN == "AWARDILVLAB" then
												if 75 <= Lp and Lm <= 75 then
													return CalcStat("AwardILvlA",L)+1;
												else
													return CalcStat("AwardILvlA",L);
												end
											else
												return 0;
											end
										else
											if 75 <= Lp and Lm <= 75 then
												return CalcStat("AwardILvlA",L)+3;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									else
										return CalcStat("AwardILvlCC",L);
									end
								elseif SN > "AWARDILVLAC" then
									if SN < "AWARDILVLBOLD" then
										if SN < "AWARDILVLB" then
											if SN == "AWARDILVLAOLD" then
												return CalcStat("AwardILvlA",L);
											else
												return 0;
											end
										elseif SN > "AWARDILVLB" then
											if SN == "AWARDILVLBA" then
												return CalcStat("AwardILvlB",L);
											else
												return 0;
											end
										else
											if Lm <= 4 then
												return CalcStat("AwardILvlA",1);
											elseif Lm <= 50 then
												return CalcStat("AwardILvlA",L-4);
											elseif Lm <= 54 then
												return CalcStat("AwardILvlA",51);
											elseif Lm <= 75 then
												return CalcStat("AwardILvlA",L-4);
											elseif Lm <= 95 then
												return CalcStat("AwardILvlA",L)+4;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									elseif SN > "AWARDILVLBOLD" then
										if SN < "AWARDILVLCA" then
											if SN == "AWARDILVLC" then
												if Lm <= 75 then
													return CalcStat("AwardILvlB",L);
												else
													return CalcStat("AwardILvlA",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLCA" then
											if SN == "AWARDILVLCB" then
												if 96 <= Lp and Lm <= 99 then
													return CalcStat("AwardILvlC",L)+4;
												elseif 100 <= Lp and Lm <= 100 then
													return CalcStat("AwardILvlC",L)+6;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										else
											if 96 <= Lp and Lm <= 100 then
												return CalcStat("AwardILvlC",L)+4;
											else
												return CalcStat("AwardILvlC",L);
											end
										end
									else
										return CalcStat("AwardILvlB",L);
									end
								else
									if 75 <= Lp and Lm <= 75 then
										return CalcStat("AwardILvlA",L)+2;
									else
										return CalcStat("AwardILvlA",L);
									end
								end
							elseif SN > "AWARDILVLCC" then
								if SN < "AWARDILVLDD" then
									if SN < "AWARDILVLD" then
										if SN < "AWARDILVLCE" then
											if SN == "AWARDILVLCD" then
												if 101 <= Lp and Lm <= 105 then
													return CalcStat("AwardILvlC",L)+4;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLCE" then
											if SN == "AWARDILVLCF" then
												if Lm <= 140 then
													return CalcStat("AwardILvlC",L);
												elseif Lm <= 150 then
													return RoundDbl(LinFmod(1,500.4,517.4,141,150,L));
												else
													return CalcStat("AwardILvlCF",150);
												end
											else
												return 0;
											end
										else
											if Lm <= 140 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 150 then
												return RoundDbl(LinFmod(1,500.4,512.4,141,150,L));
											else
												return CalcStat("AwardILvlCE",150);
											end
										end
									elseif SN > "AWARDILVLD" then
										if SN < "AWARDILVLDB" then
											if SN == "AWARDILVLDA" then
												if 106 <= Lp and Lm <= 115 then
													return RoundDbl(LinFmod(1,300,336.5,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLDB" then
											if SN == "AWARDILVLDC" then
												if 106 <= Lp and Lm <= 110 then
													return RoundDbl(LinFmod(1,300,320,106,115,L));
												elseif 111 <= Lp and Lm <= 115 then
													return CalcStat("AwardILvlDC",110);
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										else
											if 106 <= Lp and Lm <= 115 then
												return RoundDbl(LinFmod(1,300,345,106,115,L));
											else
												return CalcStat("AwardILvlD",L);
											end
										end
									else
										if Lm <= 105 then
											return CalcStat("AwardILvlC",L);
										elseif Lm <= 115 then
											return 300;
										else
											return CalcStat("AwardILvlA",L);
										end
									end
								elseif SN > "AWARDILVLDD" then
									if SN < "AWARDILVLE" then
										if SN < "AWARDILVLDF" then
											if SN == "AWARDILVLDE" then
												if 106 <= Lp and Lm <= 115 then
													return RoundDbl(LinFmod(1,300.4,326.4,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLDF" then
											if SN == "AWARDILVLDG" then
												if 106 <= Lp and Lm <= 114 then
													return RoundDbl(LinFmod(1,300,336,106,115,L));
												elseif 115 <= Lp and Lm <= 115 then
													return CalcStat("AwardILvlD",115)+40;
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										else
											if 106 <= Lp and Lm <= 110 then
												return CalcStat("AwardILvlDF",111);
											elseif 111 <= Lp and Lm <= 115 then
												return RoundDbl(LinFmod(1,299.5,320.5,106,115,L));
											else
												return CalcStat("AwardILvlD",L);
											end
										end
									elseif SN > "AWARDILVLE" then
										if SN < "AWARDILVLEB" then
											if SN == "AWARDILVLEA" then
												if 116 <= Lp and Lm <= 119 then
													return RoundDbl(LinFmod(1,350,378,116,120,L));
												elseif 120 <= Lp and Lm <= 120 then
													return CalcStat("AwardILvlE",120)+26;
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLEB" then
											if SN == "AWARDILVLEC" then
												if 116 <= Lp and Lm <= 120 then
													return RoundDbl(LinFmod(1,350,382,116,120,L));
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										else
											if 116 <= Lp and Lm <= 119 then
												return RoundDbl(LinFmod(1,350,390,116,120,L));
											elseif 120 <= Lp and Lm <= 120 then
												return CalcStat("AwardILvlE",120)+38;
											else
												return CalcStat("AwardILvlE",L);
											end
										end
									else
										if Lm <= 115 then
											return CalcStat("AwardILvlC",L);
										elseif Lm <= 120 then
											return 350;
										else
											return CalcStat("AwardILvlA",L);
										end
									end
								else
									if 106 <= Lp and Lm <= 114 then
										return RoundDbl(LinFmod(1,300,327,106,115,L));
									elseif 115 <= Lp and Lm <= 115 then
										return CalcStat("AwardILvlD",115)+30;
									else
										return CalcStat("AwardILvlD",L);
									end
								end
							else
								return CalcStat("AwardILvlC",L);
							end
						else
							if Lm <= 0 then
								return 0;
							elseif Lm <= 1 then
								return CalcStat("ArmQtyCommMP",N);
							elseif Lm <= 2 then
								return CalcStat("ArmQtyUncomMP",N);
							elseif Lm <= 3 then
								return CalcStat("ArmQtyRareMP",N);
							elseif Lm <= 4 then
								return CalcStat("ArmQtyIncompMP",N);
							else
								return CalcStat("ArmQtyEpicMP",N);
							end
						end
					elseif SN > "AWARDILVLED" then
						if SN < "BASEMAIN" then
							if SN < "AWARDILVLJ" then
								if SN < "AWARDILVLGB" then
									if SN < "AWARDILVLFA" then
										if SN > "AWARDILVLEE" then
											if SN == "AWARDILVLF" then
												if Lm <= 40 then
													return LinFmod(1,1,40,1,40,L);
												elseif Lm <= 41 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 49 then
													return LinFmod(1,41,48,42,49,L);
												elseif Lm <= 50 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 55 then
													return LinFmod(1,51,55,51,55,L);
												elseif Lm <= 56 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 59 then
													return LinFmod(1,56,58,57,59,L);
												elseif Lm <= 60 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 64 then
													return RoundDbl(LinFmod(1,60.6,63,61,64,L));
												elseif Lm <= 65 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 70 then
													return LinFmod(1,66,70,66,70,L);
												elseif Lm <= 71 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 74 then
													return LinFmod(1,71,73,72,74,L);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlF",L-1);
												elseif Lm <= 85 then
													return RoundDbl(LinFmod(1,80.4,123.4,76,85,L));
												elseif Lm <= 94 then
													return RoundDbl(LinFmod(1,130,169,86,94,L));
												elseif Lm <= 100 then
													return RoundDbl(LinFmod(1,175,198.4,95,100,L));
												elseif Lm <= 104 then
													return LinFmod(1,201,213,101,104,L);
												elseif Lm <= 105 then
													return 220;
												elseif Lm <= 114 then
													return RoundDbl(LinFmod(1,300,323,106,114,L));
												elseif Lm <= 115 then
													return 324;
												elseif Lm <= 119 then
													return RoundDbl(LinFmod(1,349.6,368.4,116,120,L));
												elseif Lm <= 120 then
													return 380;
												elseif Lm <= 125 then
													return RoundDbl(LinFmod(1,400.4,416.4,121,125,L));
												elseif Lm <= 129 then
													return RoundDbl(LinFmod(1,418.4,430.4,126,129,L));
												elseif Lm <= 130 then
													return 432;
												elseif Lm <= 135 then
													return RoundDbl(LinFmod(1,450.4,463.8,131,135,L));
												elseif Lm <= 138 then
													return RoundDbl(LinFmod(1,466,472.5,136,138,L));
												else
													return 475;
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLEE" then
											if 116 <= Lp and Lm <= 119 then
												return RoundDbl(LinFmod(1,350,366,116,120,L));
											elseif 120 <= Lp and Lm <= 120 then
												return CalcStat("AwardILvlE",120)+14;
											else
												return CalcStat("AwardILvlE",L);
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLFA" then
										if SN < "AWARDILVLG" then
											if SN == "AWARDILVLFB" then
												if Lm <= 140 then
													return CalcStat("AwardILvlF",L);
												elseif Lm <= 150 then
													return RoundDbl(LinFmod(1,500.4,517.4,141,150,L));
												else
													return CalcStat("AwardILvlFB",150);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLG" then
											if SN == "AWARDILVLGA" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,412.4,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 120 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 130 then
												return 400;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									else
										if Lm <= 9 then
											return RoundDbl(LinFmod(1,0.5,4.5,1,9,L));
										elseif Lm <= 19 then
											return LinFmod(1,6,15,10,19,L);
										elseif Lm <= 23 then
											return RoundDbl(LinFmod(1,15.5,17,20,23,L));
										elseif Lm <= 49 then
											return LinFmod(1,18,43,24,49,L);
										elseif Lm <= 65 then
											return LinFmod(1,43,58,50,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,61,70,66,75,L);
										elseif Lm <= 140 then
											return CalcStat("AwardILvlF",L);
										else
											return 500;
										end
									end
								elseif SN > "AWARDILVLGB" then
									if SN < "AWARDILVLGF" then
										if SN < "AWARDILVLGD" then
											if SN == "AWARDILVLGC" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,408.3,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLGD" then
											if SN == "AWARDILVLGE" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,434.4,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										else
											if 121 <= Lp and Lm <= 130 then
												return RoundDbl(LinFmod(1,400.4,426.4,121,130,L));
											else
												return CalcStat("AwardILvlG",L);
											end
										end
									elseif SN > "AWARDILVLGF" then
										if SN < "AWARDILVLI" then
											if SN == "AWARDILVLH" then
												if Lm <= 4 then
													return CalcStat("AwardILvlA",1);
												elseif Lm <= 50 then
													return CalcStat("AwardILvlA",L-4);
												elseif Lm <= 54 then
													return CalcStat("AwardILvlA",51);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlA",L-4);
												else
													return L-4;
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLI" then
											if SN == "AWARDILVLIA" then
												if Lm <= 44 then
													return CalcStat("AwardILvlI",L);
												elseif Lm <= 55 then
													return RoundDblDown((L-45)/6)+51;
												elseif Lm <= 90 then
													return CalcStat("AwardILvlI",L);
												elseif Lm <= 104 then
													return RoundDblDown((L-91)/5)*25+165;
												elseif Lm <= 105 then
													return CalcStat("LvlToILvl",L);
												elseif Lm <= 110 then
													return CalcStat("LvlToILvl",106)+15;
												elseif Lm <= 115 then
													return CalcStat("LvlToILvl",115)-20;
												elseif Lm <= 120 then
													return CalcStat("LvlToILvl",116)+15;
												elseif Lm <= 125 then
													return CalcStat("LvlToILvl",121)+15;
												elseif Lm <= 130 then
													return CalcStat("LvlToILvl",130)-20;
												elseif Lm <= 135 then
													return CalcStat("LvlToILvl",131)+15;
												elseif Lm <= 140 then
													return CalcStat("LvlToILvl",140)-4;
												elseif Lm <= 150 then
													return CalcStat("LvlToILvl",141)+15;
												else
													return CalcStat("AwardILvlIA",150);
												end
											else
												return 0;
											end
										else
											if Lm <= 44 then
												return 1;
											elseif Lm <= 55 then
												return 52;
											elseif Lm <= 75 then
												return RoundDblDown((L-56)/5)*5+60;
											elseif Lm <= 85 then
												return RoundDblDown((L-76)/5)*29+100;
											elseif Lm <= 95 then
												return RoundDblDown((L-86)/5)*20+155;
											elseif Lm <= 100 then
												return RoundDblDown((L-96)/4)*10+190;
											elseif Lm <= 105 then
												return RoundDblDown((L-101)/4)*35+215;
											elseif Lm <= 110 then
												return CalcStat("LvlToILvl",106)+15;
											elseif Lm <= 115 then
												return CalcStat("LvlToILvl",115);
											elseif Lm <= 119 then
												return CalcStat("LvlToILvl",116)+15;
											elseif Lm <= 120 then
												return CalcStat("LvlToILvl",120);
											elseif Lm <= 125 then
												return CalcStat("LvlToILvl",121)+15;
											elseif Lm <= 130 then
												return CalcStat("LvlToILvl",130);
											elseif Lm <= 135 then
												return CalcStat("LvlToILvl",131)+15;
											elseif Lm <= 140 then
												return CalcStat("LvlToILvl",140);
											elseif Lm <= 145 then
												return CalcStat("LvlToILvl",141)+15;
											elseif Lm <= 150 then
												return CalcStat("LvlToILvl",150);
											else
												return CalcStat("AwardILvlI",150);
											end
										end
									else
										if 121 <= Lp and Lm <= 130 then
											return RoundDbl(LinFmod(1,400.4,440.4,121,130,L));
										else
											return CalcStat("AwardILvlG",L);
										end
									end
								else
									if 121 <= Lp and Lm <= 130 then
										return RoundDbl(LinFmod(1,400.4,418.4,121,130,L));
									else
										return CalcStat("AwardILvlG",L);
									end
								end
							elseif SN > "AWARDILVLJ" then
								if SN < "AXEARMOURREND" then
									if SN < "AWARDILVLJD" then
										if SN < "AWARDILVLJB" then
											if SN == "AWARDILVLJA" then
												if 131 <= Lp and Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,465.4,131,140,L));
												else
													return CalcStat("AwardILvlJ",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLJB" then
											if SN == "AWARDILVLJC" then
												if 131 <= Lp and Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,475.4,131,140,L));
												else
													return CalcStat("AwardILvlJ",L);
												end
											else
												return 0;
											end
										else
											if 131 <= Lp and Lm <= 140 then
												return RoundDbl(LinFmod(1,450.4,470.4,131,140,L));
											else
												return CalcStat("AwardILvlJ",L);
											end
										end
									elseif SN > "AWARDILVLJD" then
										if SN < "AWARDLVLCAP" then
											if SN == "AWARDILVLK" then
												return CalcStat("CombatBaseTacHPSLvlToILvl",L);
											else
												return 0;
											end
										elseif SN > "AWARDLVLCAP" then
											if SN == "AWARDLVLTOILVL" then
												return CalcStat("AwardILvlC",L);
											else
												return 0;
											end
										else
											return 150;
										end
									else
										if 131 <= Lp and Lm <= 140 then
											return RoundDbl(LinFmod(1,450.4,480.4,131,140,L));
										else
											return CalcStat("AwardILvlJ",L);
										end
									end
								elseif SN > "AXEARMOURREND" then
									if SN < "BALANCEOFMANPARRY" then
										if SN < "BALANCEOFMANBLOCK" then
											if SN == "AXEARMRENDPOS" then
												if Lm <= 7 then
													return RoundDbl(LinFmod(1,0.5,3.25,1,7,L));
												elseif Lm <= 14 then
													return RoundDbl(LinFmod(1,3.75,8.5,8,14,L));
												elseif Lm <= 30 then
													return RoundDbl(LinFmod(1,9,24.7,15,30,L));
												elseif Lm <= 39 then
													return RoundDbl(LinFmod(1,26.5,37,31,39,L));
												elseif Lm <= 50 then
													return RoundDbl(LinFmod(1,38.5,53,40,50,L));
												elseif Lm <= 52 then
													return 53;
												elseif Lm <= 60 then
													return RoundDbl(LinFmod(1,53.2,64.8,53,60,L));
												elseif Lm <= 77 then
													return RoundDbl(LinFmod(1,66.15,94.8,61,77,L));
												elseif Lm <= 113 then
													return RoundDbl(RoundDbl(L/3-0.5)*1.95+44.1);
												elseif Lm <= 141 then
													return 116;
												elseif Lm <= 181 then
													return RoundDbl(RoundDbl(L/4)*2+46);
												elseif Lm <= 217 then
													return RoundDbl(RoundDbl(L/4)*2+45);
												elseif Lm <= 221 then
													return 153;
												elseif Lm <= 299 then
													return 155;
												elseif Lm <= 308 then
													return RoundDbl(RoundDbl(L/2.6-0.4)*2-73);
												elseif Lm <= 317 then
													return RoundDbl(RoundDbl(L/2.74-0.75)*2-60);
												elseif Lm <= 325 then
													return 172;
												elseif Lm <= 326 then
													return 174;
												elseif Lm <= 599 then
													return ExpFmod(CalcStat("AxeArmRendPos",326),327,1,L,1);
												else
													return CalcStat("AxeArmRendPos",599);
												end
											else
												return 0;
											end
										elseif SN > "BALANCEOFMANBLOCK" then
											if SN == "BALANCEOFMANEVADE" then
												return CalcStat("EvadeT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,0.8);
										end
									elseif SN > "BALANCEOFMANPARRY" then
										if SN < "BASEARMOUR" then
											if SN == "BASEAGILITY" then
												return CalcStat("BaseMain",L,N);
											else
												return 0;
											end
										elseif SN > "BASEARMOUR" then
											if SN == "BASEFATE" then
												if Lm <= 50 then
													return RoundDbl(CalcStat("ProgBEnergy",L)*3.5);
												elseif Lm <= 95 then
													return RoundDbl(CalcStat("ProgBEnergy",L)*3.5);
												else
													return RoundDbl(CalcStat("BasePower",L)*0.7);
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return RoundDblUp(LinFmod(1,1,10,1,50,L)*N);
											elseif Lm <= 60 then
												return RoundDblUp(LinFmod(1,10,15,50,60,L)*N);
											elseif Lm <= 65 then
												return RoundDblUp(LinFmod(1,15,20,60,65,L)*N);
											elseif Lm <= 75 then
												return RoundDblUp(LinFmod(1,20,30,65,75,L)*N);
											elseif Lm <= 85 then
												return RoundDblUp(LinFmod(1,30,45,75,85,L)*N);
											elseif Lm <= 95 then
												return RoundDblUp(LinFmod(1,45,65,85,95,L)*N);
											elseif Lm <= 100 then
												return RoundDblUp(LinFmod(1,65,100,95,100,L)*N);
											elseif Lm <= 105 then
												return RoundDblUp(LinFmod(1,100,130,100,105,L)*N);
											elseif Lm <= 115 then
												return RoundDblUp(LinFmod(1,143,200,106,115,L,0)*N);
											elseif Lm <= 120 then
												return RoundDblUp(LinFmod(1,230,250,116,120,L,0)*N);
											elseif Lm <= 130 then
												return RoundDblUp(LinFmod(1,288,380,121,130,L,0)*N);
											elseif Lm <= 140 then
												return RoundDblUp(LinFmod(1,440,750,131,140,L,0)*N);
											else
												return CalcStat("BaseArmour",140,N);
											end
										end
									else
										return CalcStat("ParryT",L,0.8);
									end
								else
									return -CalcStat("AxeArmRendPos",L);
								end
							else
								if Lm <= 130 then
									return CalcStat("AwardILvlC",L);
								elseif Lm <= 140 then
									return RoundDbl(LinFmod(1,450.4,460.4,131,140,L));
								else
									return CalcStat("AwardILvlA",L);
								end
							end
						elseif SN > "BASEMAIN" then
							if SN < "BEORNINGCDAGILITYTOCRITHIT" then
								if SN < "BATTLELOREPHYMAS" then
									if SN < "BASEPOWER" then
										if SN < "BASEMIGHT" then
											if SN == "BASEMAINPROG" then
												if Lm <= 95 then
													return CalcStat("StdProg",L,1.0);
												elseif Lm <= 140 then
													return RoundDbl(CalcStat("StdProg",L,1.0));
												else
													return RoundDbl(LinFmod(1,780,1360,141,150,L));
												end
											else
												return 0;
											end
										elseif SN > "BASEMIGHT" then
											if SN == "BASEMORALE" then
												if Lm <= 50 then
													return RoundDblDown(CalcStat("ProgBHealth",L)*5);
												elseif Lm <= 95 then
													return RoundDbl(CalcStat("ProgBHealth",L)*5);
												else
													return RoundDbl(CalcStat("ProgBHealth",L)*5,-1);
												end
											else
												return 0;
											end
										else
											return CalcStat("BaseMain",L,N);
										end
									elseif SN > "BASEPOWER" then
										if SN < "BASEWILL" then
											if SN == "BASEVITALITY" then
												if Lm <= 50 then
													return RoundDbl(CalcStat("ProgBHealth",L)*0.75);
												elseif Lm <= 95 then
													return RoundDbl(CalcStat("ProgBHealth",L)*0.75);
												else
													return RoundDbl(CalcStat("BaseMorale",L)*0.15);
												end
											else
												return 0;
											end
										elseif SN > "BASEWILL" then
											if SN == "BATTLELOREMAS" then
												if Lm <= 105 then
													return CalcStat("Mastery",L,2.5);
												elseif Lm <= 119 then
													return CalcStat("MasteryT",L,1.2);
												elseif Lm <= 120 then
													return CalcStat("MasteryT",L,1.6);
												elseif Lm <= 129 then
													return CalcStat("MasteryT",L,1.2);
												elseif Lm <= 130 then
													return CalcStat("MasteryT",L,1.6);
												elseif Lm <= 139 then
													return CalcStat("MasteryT",L,1.2);
												else
													return CalcStat("MasteryT",L,1.6);
												end
											else
												return 0;
											end
										else
											return CalcStat("BaseMain",L,N);
										end
									else
										if Lm <= 50 then
											return RoundDblDown(CalcStat("ProgBEnergy",L)*5);
										elseif Lm <= 95 then
											return RoundDblDown(CalcStat("ProgBEnergy",L)*5);
										else
											return RoundDbl(CalcStat("ProgBEnergy",L)*5);
										end
									end
								elseif SN > "BATTLELOREPHYMAS" then
									if SN < "BEOFATE" then
										if SN < "BEOBEARFORMCRITDEF" then
											if SN == "BATTLELORETACMAS" then
												return CalcStat("BattleLoreMas",L);
											else
												return 0;
											end
										elseif SN > "BEOBEARFORMCRITDEF" then
											if SN == "BEOEMISSARYFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("CritDefT",L,0.4);
										end
									elseif SN > "BEOFATE" then
										if SN < "BEOFEWINNUMBERFATE" then
											if SN == "BEOFERALPRESFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										elseif SN > "BEOFEWINNUMBERFATE" then
											if SN == "BEOMIGHTOFTHEWILDMIGHT" then
												return CalcStat("MightT",L,1.0);
											else
												return 0;
											end
										else
											return -CalcStat("FateT",L,0.4);
										end
									else
										return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
									end
								else
									return CalcStat("BattleLoreMas",L);
								end
							elseif SN > "BEORNINGCDAGILITYTOCRITHIT" then
								if SN < "BEORNINGCDARMOURTYPE" then
									if SN < "BEORNINGCDAGILITYTOPHYMAS" then
										if SN < "BEORNINGCDAGILITYTOFINESSE" then
											if SN == "BEORNINGCDAGILITYTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDAGILITYTOFINESSE" then
											if SN == "BEORNINGCDAGILITYTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGCDAGILITYTOPHYMAS" then
										if SN < "BEORNINGCDARMOURTONONPHYMIT" then
											if SN == "BEORNINGCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDARMOURTONONPHYMIT" then
											if SN == "BEORNINGCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 2;
									end
								elseif SN > "BEORNINGCDARMOURTYPE" then
									if SN < "BEORNINGCDBASEICPR" then
										if SN < "BEORNINGCDBASEFATE" then
											if SN == "BEORNINGCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEFATE" then
											if SN == "BEORNINGCDBASEICMR" then
												return CalcStat("ClassBaseICMRM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									elseif SN > "BEORNINGCDBASEICPR" then
										if SN < "BEORNINGCDBASEMORALE" then
											if SN == "BEORNINGCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEMORALE" then
											if SN == "BEORNINGCDBASENCMR" then
												return CalcStat("ClassBaseNCMRM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								else
									return 3;
								end
							else
								return 2;
							end
						else
							if Lm <= 50 then
								return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
							elseif Lm <= 95 then
								return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
							else
								return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
							end
						end
					else
						if 116 <= Lp and Lm <= 120 then
							return RoundDbl(LinFmod(1,350,370,116,120,L));
						else
							return CalcStat("AwardILvlE",L);
						end
					end
				elseif SN > "BEORNINGCDBASENCPR" then
					if SN < "BPEPRATPCAPR" then
						if SN < "BEORNINGRDPSVONENAME" then
							if SN < "BEORNINGCDMIGHTTOPHYMAS" then
								if SN < "BEORNINGCDCANBLOCK" then
									if SN < "BEORNINGCDBASEWILL" then
										if SN > "BEORNINGCDBASEPOWER" then
											if SN == "BEORNINGCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN == "BEORNINGCDBASEPOWER" then
											return 10;
										else
											return 0;
										end
									elseif SN > "BEORNINGCDBASEWILL" then
										if SN < "BEORNINGCDCALCTYPENONPHYMIT" then
											if SN == "BEORNINGCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDCALCTYPENONPHYMIT" then
											if SN == "BEORNINGCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return CalcStat("ClassBaseWillM",L);
									end
								elseif SN > "BEORNINGCDCANBLOCK" then
									if SN < "BEORNINGCDMIGHTTOCRITHIT" then
										if SN < "BEORNINGCDFATETOMORALE" then
											if SN == "BEORNINGCDFATETOICMR" then
												return 0.04;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDFATETOMORALE" then
											if SN == "BEORNINGCDFATETONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 2.5;
										end
									elseif SN > "BEORNINGCDMIGHTTOCRITHIT" then
										if SN < "BEORNINGCDMIGHTTOOUTHEAL" then
											if SN == "BEORNINGCDMIGHTTOEVADE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDMIGHTTOOUTHEAL" then
											if SN == "BEORNINGCDMIGHTTOPARRY" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 1;
									end
								else
									if Lm <= 5 then
										return 0;
									else
										return 1;
									end
								end
							elseif SN > "BEORNINGCDMIGHTTOPHYMAS" then
								if SN < "BEORNINGCDVITALITYTONCMR" then
									if SN < "BEORNINGCDPHYMITTONONPHYMIT" then
										if SN < "BEORNINGCDMIGHTTOTACMIT" then
											if SN == "BEORNINGCDMIGHTTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDMIGHTTOTACMIT" then
											if SN == "BEORNINGCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGCDPHYMITTONONPHYMIT" then
										if SN < "BEORNINGCDVITALITYTOICMR" then
											if SN == "BEORNINGCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDVITALITYTOICMR" then
											if SN == "BEORNINGCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 0.012;
										end
									else
										return 1;
									end
								elseif SN > "BEORNINGCDVITALITYTONCMR" then
									if SN < "BEORNINGCDWILLTOPHYMIT" then
										if SN < "BEORNINGCDWILLTOOUTHEAL" then
											if SN == "BEORNINGCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDWILLTOOUTHEAL" then
											if SN == "BEORNINGCDWILLTOPHYMAS" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGCDWILLTOPHYMIT" then
										if SN < "BEORNINGCDWILLTOTACMIT" then
											if SN == "BEORNINGCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDWILLTOTACMIT" then
											if SN == "BEORNINGRDPSVONEFATE" then
												return CalcStat("BeoEmissaryFate",L);
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1.5;
									end
								else
									return 0.12;
								end
							else
								return 3;
							end
						elseif SN > "BEORNINGRDPSVONENAME" then
							if SN < "BLOCKPRATP" then
								if SN < "BLACKARROWCDCALCTYPECOMPHYMIT" then
									if SN < "BEORNINGRDTRAITVITALITY" then
										if SN < "BEORNINGRDTRAITFATE" then
											if SN == "BEORNINGRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										elseif SN > "BEORNINGRDTRAITFATE" then
											if SN == "BEORNINGRDTRAITMIGHT" then
												return CalcStat("BeoMightoftheWildMight",L);
											else
												return 0;
											end
										else
											return CalcStat("BeoFewinNumberFate",L);
										end
									elseif SN > "BEORNINGRDTRAITVITALITY" then
										if SN < "BEOVITALITYINCREASE" then
											if SN == "BEOTHICKHIDEVITALITY" then
												return CalcStat("VitalityT",L,1.0);
											else
												return 0;
											end
										elseif SN > "BEOVITALITYINCREASE" then
											if SN == "BLACKARROWCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return CalcStat("BeoThickHideVitality",L);
									end
								elseif SN > "BLACKARROWCDCALCTYPECOMPHYMIT" then
									if SN < "BLOCK" then
										if SN < "BLACKARROWCDCALCTYPETACMIT" then
											if SN == "BLACKARROWCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "BLACKARROWCDCALCTYPETACMIT" then
											if SN == "BLACKARROWCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "BLOCK" then
										if SN < "BLOCKPBONUS" then
											if SN == "BLOCKC" then
												return CalcStat("BPEC",L,N);
											else
												return 0;
											end
										elseif SN > "BLOCKPBONUS" then
											if SN == "BLOCKPPRAT" then
												return CalcStat("BPEPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPBonus",L);
										end
									else
										return CalcStat("BPE",L,N);
									end
								else
									return 13;
								end
							elseif SN > "BLOCKPRATP" then
								if SN < "BPEC" then
									if SN < "BLOCKPRATPCAP" then
										if SN < "BLOCKPRATPB" then
											if SN == "BLOCKPRATPA" then
												return CalcStat("BPEPRatPA",L);
											else
												return 0;
											end
										elseif SN > "BLOCKPRATPB" then
											if SN == "BLOCKPRATPC" then
												return CalcStat("BPEPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPB",L);
										end
									elseif SN > "BLOCKPRATPCAP" then
										if SN < "BLOCKT" then
											if SN == "BLOCKPRATPCAPR" then
												return CalcStat("BPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "BLOCKT" then
											if SN == "BPE" then
												return StatLinInter("PntMPBPE","ItemPntS","ProgBBPE","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("BPET",L,N);
										end
									else
										return CalcStat("BPEPRatPCap",L);
									end
								elseif SN > "BPEC" then
									if SN < "BPEPRATPA" then
										if SN < "BPEPPRAT" then
											if SN == "BPEPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "BPEPPRAT" then
											if SN == "BPEPRATP" then
												return CalcPercAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCapR",L),N);
										end
									elseif SN > "BPEPRATPA" then
										if SN < "BPEPRATPC" then
											if SN == "BPEPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "BPEPRATPC" then
											if SN == "BPEPRATPCAP" then
												return 13;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 39;
									end
								else
									return StatLinInter("PntMPBPEC","CreepPntS","ProgBBPEC","",L,N,0);
								end
							else
								return CalcStat("BPEPRatP",L,N);
							end
						else
							return "Emissary";
						end
					elseif SN > "BPEPRATPCAPR" then
						if SN < "BRAWLERCDBASEPOWER" then
							if SN < "BRAWLERCDAGILITYTOEVADE" then
								if SN < "BRATMITLIGHT" then
									if SN < "BRATDEVHIT" then
										if SN > "BPET" then
											if SN == "BRATCRITMAGN" then
												return CalcStat("StdProg",L,600);
											else
												return 0;
											end
										elseif SN == "BPET" then
											return StatLinInter("PntMPBPE","TraitPntS","ProgBBPE","AdjUmbarTrait",L,N,0);
										else
											return 0;
										end
									elseif SN > "BRATDEVHIT" then
										if SN < "BRATMITHEAVY" then
											if SN == "BRATEXTRA" then
												return CalcStat("StdProg",L,300);
											else
												return 0;
											end
										elseif SN > "BRATMITHEAVY" then
											if SN == "BRATMITIGATIONS" then
												if Lm <= 50 then
													return LinFmod(1,(N*CalcStat("BRatStandard",1))*7/6-50.4,N*CalcStat("BRatStandard",50),1,50,L,"P");
												else
													return StatLinInter("","StdPntS","BRatStandard","",L,N,3);
												end
											else
												return 0;
											end
										else
											return CalcStat("BRatStandard",L);
										end
									else
										return CalcStat("StdProg",L,400);
									end
								elseif SN > "BRATMITLIGHT" then
									if SN < "BRATPROGB" then
										if SN < "BRATOUTHEAL" then
											if SN == "BRATMITMEDIUM" then
												return CalcStat("BRatMitigations",L,0.833);
											else
												return 0;
											end
										elseif SN > "BRATOUTHEAL" then
											if SN == "BRATPARTBPE" then
												return CalcStat("StdProg",L,350);
											else
												return 0;
											end
										else
											return CalcStat("StdProg",L,450);
										end
									elseif SN > "BRATPROGB" then
										if SN < "BRATSTANDARD" then
											if SN == "BRATROUNDED" then
												if Lm <= 50 then
													return RoundDbl(CalcStat(C,L));
												elseif Lm <= 105 then
													return RoundDbl(CalcStat(C,L),-1);
												elseif Lm <= 115 then
													return RoundDbl(CalcStat(C,L),-2);
												elseif Lm <= 130 then
													return RoundDbl(CalcStat(C,L),-1);
												elseif Lm <= 150 then
													return RoundDbl(CalcStat(C,L),-2);
												else
													return RoundDbl(CalcStat(C,L));
												end
											else
												return 0;
											end
										elseif SN > "BRATSTANDARD" then
											if SN == "BRAWLERCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("StdProg",L,200);
										end
									else
										if Lm <= 50 then
											return RoundDbl(CalcStat(C,L));
										else
											return CalcStat(C,L);
										end
									end
								else
									return CalcStat("BRatMitigations",L,0.666);
								end
							elseif SN > "BRAWLERCDAGILITYTOEVADE" then
								if SN < "BRAWLERCDBASEAGILITY" then
									if SN < "BRAWLERCDARMOURTOCOMPHYMIT" then
										if SN < "BRAWLERCDAGILITYTOOUTHEAL" then
											if SN == "BRAWLERCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDAGILITYTOOUTHEAL" then
											if SN == "BRAWLERCDAGILITYTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "BRAWLERCDARMOURTOCOMPHYMIT" then
										if SN < "BRAWLERCDARMOURTOTACMIT" then
											if SN == "BRAWLERCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDARMOURTOTACMIT" then
											if SN == "BRAWLERCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "BRAWLERCDBASEAGILITY" then
									if SN < "BRAWLERCDBASEMIGHT" then
										if SN < "BRAWLERCDBASEICMR" then
											if SN == "BRAWLERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEICMR" then
											if SN == "BRAWLERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRL",L);
										end
									elseif SN > "BRAWLERCDBASEMIGHT" then
										if SN < "BRAWLERCDBASENCMR" then
											if SN == "BRAWLERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASENCMR" then
											if SN == "BRAWLERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									else
										return CalcStat("ClassBaseMightH",L);
									end
								else
									return CalcStat("ClassBaseAgilityM",L);
								end
							else
								return 2;
							end
						elseif SN > "BRAWLERCDBASEPOWER" then
							if SN < "BRAWLERCDPHYMITTOCOMPHYMIT" then
								if SN < "BRAWLERCDHASPOWER" then
									if SN < "BRAWLERCDCALCTYPENONPHYMIT" then
										if SN < "BRAWLERCDBASEWILL" then
											if SN == "BRAWLERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEWILL" then
											if SN == "BRAWLERCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillL",L);
										end
									elseif SN > "BRAWLERCDCALCTYPENONPHYMIT" then
										if SN < "BRAWLERCDFATETONCPR" then
											if SN == "BRAWLERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDFATETONCPR" then
											if SN == "BRAWLERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.07;
										end
									else
										return 14;
									end
								elseif SN > "BRAWLERCDHASPOWER" then
									if SN < "BRAWLERCDMIGHTTOPARRY" then
										if SN < "BRAWLERCDMIGHTTOEVADE" then
											if SN == "BRAWLERCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDMIGHTTOEVADE" then
											if SN == "BRAWLERCDMIGHTTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BRAWLERCDMIGHTTOPARRY" then
										if SN < "BRAWLERCDMIGHTTOPHYMIT" then
											if SN == "BRAWLERCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDMIGHTTOPHYMIT" then
											if SN == "BRAWLERCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "BRAWLERCDPHYMITTOCOMPHYMIT" then
								if SN < "BRAWLERCDWILLTOPHYMAS" then
									if SN < "BRAWLERCDVITALITYTOMORALE" then
										if SN < "BRAWLERCDTACMASTOOUTHEAL" then
											if SN == "BRAWLERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDTACMASTOOUTHEAL" then
											if SN == "BRAWLERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BRAWLERCDVITALITYTOMORALE" then
										if SN < "BRAWLERCDWILLTOFINESSE" then
											if SN == "BRAWLERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDWILLTOFINESSE" then
											if SN == "BRAWLERCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 4.5;
									end
								elseif SN > "BRAWLERCDWILLTOPHYMAS" then
									if SN < "BRGALLINFINESSE" then
										if SN < "BRAWLERCDWILLTORESIST" then
											if SN == "BRAWLERCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDWILLTORESIST" then
											if SN == "BRAWLERCDWILLTOTACMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BRGALLINFINESSE" then
										if SN < "BRGREVWEAKNCRITDEF" then
											if SN == "BRGALLINONEXPFINESSE" then
												return -CalcStat("Finesse",L,2);
											else
												return 0;
											end
										elseif SN > "BRGREVWEAKNCRITDEF" then
											if SN == "BRGREVWEAKNFINESSE" then
												return -CalcStat("FinesseT",L,0.4);
											else
												return 0;
											end
										else
											return -CalcStat("CritDefT",L,0.4);
										end
									else
										return CalcStat("Finesse",L,2);
									end
								else
									return 1;
								end
							else
								return 1;
							end
						else
							return CalcStat("ClassBasePower",L);
						end
					else
						return CalcStat("BPEPRatPB",L)*CalcStat("BPEPRatPC",L);
					end
				else
					return CalcStat("ClassBaseNCPR",L);
				end
			elseif SN > "BRGREVWEAKNRESIST" then
				if SN < "CHAMPIONCDBASEVITALITY" then
					if SN < "CAPTAINCDAGILITYTOPARRY" then
						if SN < "BURGLARCDBASEMORALE" then
							if SN < "BURGLARCDAGILITYTOCRITHIT" then
								if SN < "BRWMAELSTROMMIGHT" then
									if SN < "BRWAGGPOSTUREPHYMIT" then
										if SN > "BRGTRICKCNTDEFBPE" then
											if SN == "BRGTRICKCNTDEFCRITDEF" then
												return -CalcStat("CritDefT",L,0.8);
											else
												return 0;
											end
										elseif SN == "BRGTRICKCNTDEFBPE" then
											return -CalcStat("BPET",L,1.2);
										else
											return 0;
										end
									elseif SN > "BRWAGGPOSTUREPHYMIT" then
										if SN < "BRWINNSTRCLEVTECHFINESSE" then
											if SN == "BRWDEFPOSTUREPHYMAS" then
												return -CalcStat("PhyMasT",L,4);
											else
												return 0;
											end
										elseif SN > "BRWINNSTRCLEVTECHFINESSE" then
											if SN == "BRWINNSTRPRECISIONCRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait13510Choice",N)*0.2);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("Trait13510Choice",N)*0.2);
										end
									else
										return -CalcStat("PhyMitT",L,3);
									end
								elseif SN > "BRWMAELSTROMMIGHT" then
									if SN < "BRWSHAREISBALANCEFINESSE" then
										if SN < "BRWRETINTENSITYCRITHIT" then
											if SN == "BRWMIGHTINCREASE" then
												return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "BRWRETINTENSITYCRITHIT" then
											if SN == "BRWRETPRECISIONFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait234Choice",N));
											else
												return 0;
											end
										else
											return CalcStat("CritHitT",L,CalcStat("Trait234Choice",N));
										end
									elseif SN > "BRWSHAREISBALANCEFINESSE" then
										if SN < "BRWTACMIT" then
											if SN == "BRWSHAREISHEAVYCRITHIT" then
												return CalcStat("BrwInnStrPrecisionCritHit",L,3);
											else
												return 0;
											end
										elseif SN > "BRWTACMIT" then
											if SN == "BRWVITALITYINCREASE" then
												return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.4);
										end
									else
										return CalcStat("BrwInnStrClevTechFinesse",L,3);
									end
								else
									return CalcStat("MightT",L,2);
								end
							elseif SN > "BURGLARCDAGILITYTOCRITHIT" then
								if SN < "BURGLARCDARMOURTONONPHYMIT" then
									if SN < "BURGLARCDAGILITYTOPHYMAS" then
										if SN < "BURGLARCDAGILITYTOOUTHEAL" then
											if SN == "BURGLARCDAGILITYTOEVADE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BURGLARCDAGILITYTOOUTHEAL" then
											if SN == "BURGLARCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "BURGLARCDAGILITYTOPHYMAS" then
										if SN < "BURGLARCDAGILITYTOTACMIT" then
											if SN == "BURGLARCDAGILITYTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDAGILITYTOTACMIT" then
											if SN == "BURGLARCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 3;
									end
								elseif SN > "BURGLARCDARMOURTONONPHYMIT" then
									if SN < "BURGLARCDBASEFATE" then
										if SN < "BURGLARCDARMOURTYPE" then
											if SN == "BURGLARCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "BURGLARCDARMOURTYPE" then
											if SN == "BURGLARCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityH",L);
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "BURGLARCDBASEFATE" then
										if SN < "BURGLARCDBASEICPR" then
											if SN == "BURGLARCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASEICPR" then
											if SN == "BURGLARCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									else
										return CalcStat("ClassBaseFate",L);
									end
								else
									return 0.2;
								end
							else
								return 1;
							end
						elseif SN > "BURGLARCDBASEMORALE" then
							if SN < "BURGLARCDMIGHTTOPHYMAS" then
								if SN < "BURGLARCDCALCTYPETACMIT" then
									if SN < "BURGLARCDBASEVITALITY" then
										if SN < "BURGLARCDBASENCPR" then
											if SN == "BURGLARCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASENCPR" then
											if SN == "BURGLARCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									elseif SN > "BURGLARCDBASEVITALITY" then
										if SN < "BURGLARCDCALCTYPECOMPHYMIT" then
											if SN == "BURGLARCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDCALCTYPECOMPHYMIT" then
											if SN == "BURGLARCDCALCTYPENONPHYMIT" then
												return 13;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ClassBaseVitality",L);
									end
								elseif SN > "BURGLARCDCALCTYPETACMIT" then
									if SN < "BURGLARCDMIGHTTOCRITHIT" then
										if SN < "BURGLARCDFATETOPOWER" then
											if SN == "BURGLARCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "BURGLARCDFATETOPOWER" then
											if SN == "BURGLARCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BURGLARCDMIGHTTOCRITHIT" then
										if SN < "BURGLARCDMIGHTTOFINESSE" then
											if SN == "BURGLARCDMIGHTTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDMIGHTTOFINESSE" then
											if SN == "BURGLARCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1.5;
									end
								else
									return 26;
								end
							elseif SN > "BURGLARCDMIGHTTOPHYMAS" then
								if SN < "BURGLARCDWILLTOFINESSE" then
									if SN < "BURGLARCDVITALITYTOICMR" then
										if SN < "BURGLARCDPHYMITTONONPHYMIT" then
											if SN == "BURGLARCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDPHYMITTONONPHYMIT" then
											if SN == "BURGLARCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BURGLARCDVITALITYTOICMR" then
										if SN < "BURGLARCDVITALITYTONCMR" then
											if SN == "BURGLARCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "BURGLARCDVITALITYTONCMR" then
											if SN == "BURGLARCDWILLTOCRITHIT" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 0.012;
									end
								elseif SN > "BURGLARCDWILLTOFINESSE" then
									if SN < "BURGLARCDWILLTORESIST" then
										if SN < "BURGLARCDWILLTOPHYMAS" then
											if SN == "BURGLARCDWILLTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BURGLARCDWILLTOPHYMAS" then
											if SN == "BURGLARCDWILLTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "BURGLARCDWILLTORESIST" then
										if SN < "CAPTAINCDAGILITYTOCRITHIT" then
											if SN == "C" then
												return C;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDAGILITYTOCRITHIT" then
											if SN == "CAPTAINCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								else
									return 1.5;
								end
							else
								return 2;
							end
						else
							return CalcStat("ClassBaseMorale",L);
						end
					elseif SN > "CAPTAINCDAGILITYTOPARRY" then
						if SN < "CAPTAINCDMIGHTTOTACMIT" then
							if SN < "CAPTAINCDBASEPOWER" then
								if SN < "CAPTAINCDBASEAGILITY" then
									if SN < "CAPTAINCDARMOURTOCOMPHYMIT" then
										if SN > "CAPTAINCDAGILITYTOPHYMAS" then
											if SN == "CAPTAINCDAGILITYTOTACMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN == "CAPTAINCDAGILITYTOPHYMAS" then
											return 2;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDARMOURTOCOMPHYMIT" then
										if SN < "CAPTAINCDARMOURTOTACMIT" then
											if SN == "CAPTAINCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDARMOURTOTACMIT" then
											if SN == "CAPTAINCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "CAPTAINCDBASEAGILITY" then
									if SN < "CAPTAINCDBASEMIGHT" then
										if SN < "CAPTAINCDBASEICMR" then
											if SN == "CAPTAINCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEICMR" then
											if SN == "CAPTAINCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRM",L);
										end
									elseif SN > "CAPTAINCDBASEMIGHT" then
										if SN < "CAPTAINCDBASENCMR" then
											if SN == "CAPTAINCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASENCMR" then
											if SN == "CAPTAINCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRM",L);
										end
									else
										return CalcStat("ClassBaseMightH",L);
									end
								else
									return CalcStat("ClassBaseAgilityL",L);
								end
							elseif SN > "CAPTAINCDBASEPOWER" then
								if SN < "CAPTAINCDFATETOPOWER" then
									if SN < "CAPTAINCDCALCTYPENONPHYMIT" then
										if SN < "CAPTAINCDBASEWILL" then
											if SN == "CAPTAINCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEWILL" then
											if SN == "CAPTAINCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillM",L);
										end
									elseif SN > "CAPTAINCDCALCTYPENONPHYMIT" then
										if SN < "CAPTAINCDCANBLOCK" then
											if SN == "CAPTAINCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDCANBLOCK" then
											if SN == "CAPTAINCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											if Lm <= 14 then
												return 0;
											else
												return 1;
											end
										end
									else
										return 14;
									end
								elseif SN > "CAPTAINCDFATETOPOWER" then
									if SN < "CAPTAINCDMIGHTTOPARRY" then
										if SN < "CAPTAINCDMIGHTTOBLOCK" then
											if SN == "CAPTAINCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDMIGHTTOBLOCK" then
											if SN == "CAPTAINCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "CAPTAINCDMIGHTTOPARRY" then
										if SN < "CAPTAINCDMIGHTTOPHYMIT" then
											if SN == "CAPTAINCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDMIGHTTOPHYMIT" then
											if SN == "CAPTAINCDMIGHTTOTACMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return CalcStat("ClassBasePower",L);
							end
						elseif SN > "CAPTAINCDMIGHTTOTACMIT" then
							if SN < "CHAMPIONCDAGILITYTOOUTHEAL" then
								if SN < "CAPTAINCDWILLTOPHYMAS" then
									if SN < "CAPTAINCDVITALITYTOICMR" then
										if SN < "CAPTAINCDPHYMITTONONPHYMIT" then
											if SN == "CAPTAINCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDPHYMITTONONPHYMIT" then
											if SN == "CAPTAINCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CAPTAINCDVITALITYTOICMR" then
										if SN < "CAPTAINCDVITALITYTONCMR" then
											if SN == "CAPTAINCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDVITALITYTONCMR" then
											if SN == "CAPTAINCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 0.012;
									end
								elseif SN > "CAPTAINCDWILLTOPHYMAS" then
									if SN < "CAPTAINCDWILLTOTACMIT" then
										if SN < "CAPTAINCDWILLTORESIST" then
											if SN == "CAPTAINCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDWILLTORESIST" then
											if SN == "CAPTAINCDWILLTOTACMAS" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CAPTAINCDWILLTOTACMIT" then
										if SN < "CHAMPIONCDAGILITYTOCRITHIT" then
											if SN == "CATMINTBEARARMOUR" then
												return CalcStat("LMCatmintBearArmour",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDAGILITYTOCRITHIT" then
											if SN == "CHAMPIONCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1.5;
									end
								else
									return 1;
								end
							elseif SN > "CHAMPIONCDAGILITYTOOUTHEAL" then
								if SN < "CHAMPIONCDBASEFATE" then
									if SN < "CHAMPIONCDARMOURTONONPHYMIT" then
										if SN < "CHAMPIONCDAGILITYTOPHYMAS" then
											if SN == "CHAMPIONCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDAGILITYTOPHYMAS" then
											if SN == "CHAMPIONCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "CHAMPIONCDARMOURTONONPHYMIT" then
										if SN < "CHAMPIONCDARMOURTYPE" then
											if SN == "CHAMPIONCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDARMOURTYPE" then
											if SN == "CHAMPIONCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 0.2;
									end
								elseif SN > "CHAMPIONCDBASEFATE" then
									if SN < "CHAMPIONCDBASEMORALE" then
										if SN < "CHAMPIONCDBASEICPR" then
											if SN == "CHAMPIONCDBASEICMR" then
												return CalcStat("ClassBaseICMRH",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASEICPR" then
											if SN == "CHAMPIONCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									elseif SN > "CHAMPIONCDBASEMORALE" then
										if SN < "CHAMPIONCDBASENCPR" then
											if SN == "CHAMPIONCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASENCPR" then
											if SN == "CHAMPIONCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseFate",L);
								end
							else
								return 2;
							end
						else
							return 1;
						end
					else
						return 1;
					end
				elseif SN > "CHAMPIONCDBASEVITALITY" then
					if SN < "CLASSBASEVITALITY" then
						if SN < "CHICKENCDHASPOWER" then
							if SN < "CHAMPIONCDPHYMITTOCOMPHYMIT" then
								if SN < "CHAMPIONCDFATETOPOWER" then
									if SN < "CHAMPIONCDCALCTYPENONPHYMIT" then
										if SN > "CHAMPIONCDBASEWILL" then
											if SN == "CHAMPIONCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDBASEWILL" then
											return CalcStat("ClassBaseWillL",L);
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDCALCTYPENONPHYMIT" then
										if SN < "CHAMPIONCDCANBLOCK" then
											if SN == "CHAMPIONCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDCANBLOCK" then
											if SN == "CHAMPIONCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											if Lm <= 5 then
												return 0;
											else
												return 1;
											end
										end
									else
										return 14;
									end
								elseif SN > "CHAMPIONCDFATETOPOWER" then
									if SN < "CHAMPIONCDMIGHTTOPARRY" then
										if SN < "CHAMPIONCDMIGHTTOCRITHIT" then
											if SN == "CHAMPIONCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDMIGHTTOCRITHIT" then
											if SN == "CHAMPIONCDMIGHTTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CHAMPIONCDMIGHTTOPARRY" then
										if SN < "CHAMPIONCDMIGHTTOPHYMIT" then
											if SN == "CHAMPIONCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDMIGHTTOPHYMIT" then
											if SN == "CHAMPIONCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 3;
									end
								else
									return 1;
								end
							elseif SN > "CHAMPIONCDPHYMITTOCOMPHYMIT" then
								if SN < "CHAMPIONCDWILLTOPHYMAS" then
									if SN < "CHAMPIONCDVITALITYTOMORALE" then
										if SN < "CHAMPIONCDTACMASTOOUTHEAL" then
											if SN == "CHAMPIONCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDTACMASTOOUTHEAL" then
											if SN == "CHAMPIONCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CHAMPIONCDVITALITYTOMORALE" then
										if SN < "CHAMPIONCDWILLTOFINESSE" then
											if SN == "CHAMPIONCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDWILLTOFINESSE" then
											if SN == "CHAMPIONCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 4.5;
									end
								elseif SN > "CHAMPIONCDWILLTOPHYMAS" then
									if SN < "CHICKENCANBLOCK" then
										if SN < "CHAMPIONCDWILLTORESIST" then
											if SN == "CHAMPIONCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDWILLTORESIST" then
											if SN == "CHAMPIONCDWILLTOTACMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CHICKENCANBLOCK" then
										if SN < "CHICKENCDCALCTYPENONPHYMIT" then
											if SN == "CHICKENCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "CHICKENCDCALCTYPENONPHYMIT" then
											if SN == "CHICKENCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "CHICKENCDHASPOWER" then
							if SN < "CLASSBASEICMRH" then
								if SN < "CHPFLURRYINCRCRITHIT" then
									if SN < "CHISELCRITHITLOLD" then
										if SN < "CHISELCRITHITHOLD" then
											if SN == "CHISELCRITHITH" then
												if Lm <= 140 then
													return CalcStat("U371LegacyStatFix",L,"ChiselCritHitHOld");
												else
													return CalcStat("ChiselCritHitH",140);
												end
											else
												return 0;
											end
										elseif SN > "CHISELCRITHITHOLD" then
											if SN == "CHISELCRITHITL" then
												if Lm <= 140 then
													return CalcStat("U371LegacyStatFix",L,"ChiselCritHitLOld");
												else
													return CalcStat("ChiselCritHitL",140);
												end
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return RoundDbl(24.24*L);
											else
												return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitHOld",105));
											end
										end
									elseif SN > "CHISELCRITHITLOLD" then
										if SN < "CHPCONTRBURNICPR" then
											if SN == "CHP2HWPNBLOCK" then
												return CalcStat("BlockT",L,4);
											else
												return 0;
											end
										elseif SN > "CHPCONTRBURNICPR" then
											if SN == "CHPFINESSEINCREASE" then
												return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("ICPRT",L,0.4);
										end
									else
										if Lm <= 105 then
											return RoundDbl(16.16*L);
										else
											return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitLOld",105));
										end
									end
								elseif SN > "CHPFLURRYINCRCRITHIT" then
									if SN < "CLASSBASEAGILITYH" then
										if SN < "CHPSTALWBLADEVITALITY" then
											if SN == "CHPMIGHTINCREASE" then
												return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "CHPSTALWBLADEVITALITY" then
											if SN == "CHPUNBREAKTACMIT" then
												return CalcStat("TacMitT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									elseif SN > "CLASSBASEAGILITYH" then
										if SN < "CLASSBASEAGILITYM" then
											if SN == "CLASSBASEAGILITYL" then
												return CalcStat("BaseAgility",L,0.5);
											else
												return 0;
											end
										elseif SN > "CLASSBASEAGILITYM" then
											if SN == "CLASSBASEFATE" then
												return CalcStat("BaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("BaseAgility",L,1.0);
										end
									else
										return CalcStat("BaseAgility",L,1.5);
									end
								else
									return CalcStat("CritHitT",L,2.4);
								end
							elseif SN > "CLASSBASEICMRH" then
								if SN < "CLASSBASEMORALE" then
									if SN < "CLASSBASEICPRADJ" then
										if SN < "CLASSBASEICMRM" then
											if SN == "CLASSBASEICMRL" then
												return 0.15;
											else
												return 0;
											end
										elseif SN > "CLASSBASEICMRM" then
											if SN == "CLASSBASEICPR" then
												return StatLinInter("PntMPClassBaseICPR","ClassBasePowerRegenPntS","ProgBICPR","ClassBaseICPRAdj",L,N,99);
											else
												return 0;
											end
										else
											return 0.175;
										end
									elseif SN > "CLASSBASEICPRADJ" then
										if SN < "CLASSBASEMIGHTL" then
											if SN == "CLASSBASEMIGHTH" then
												return CalcStat("BaseMight",L,1.5);
											else
												return 0;
											end
										elseif SN > "CLASSBASEMIGHTL" then
											if SN == "CLASSBASEMIGHTM" then
												return CalcStat("BaseMight",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("BaseMight",L,0.5);
										end
									else
										if Lm <= 1 then
											return 1.5*N;
										elseif Lm <= 20 then
											return 1.1*N;
										else
											return N;
										end
									end
								elseif SN > "CLASSBASEMORALE" then
									if SN < "CLASSBASENCPR" then
										if SN < "CLASSBASENCMRL" then
											if SN == "CLASSBASENCMRH" then
												return 2;
											else
												return 0;
											end
										elseif SN > "CLASSBASENCMRL" then
											if SN == "CLASSBASENCMRM" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CLASSBASENCPR" then
										if SN < "CLASSBASEPOWER" then
											if SN == "CLASSBASENCPRADJ" then
												if Lm <= 1 then
													return 1.5*N;
												elseif Lm <= 20 then
													return 1.1*N;
												else
													return N;
												end
											else
												return 0;
											end
										elseif SN > "CLASSBASEPOWER" then
											if SN == "CLASSBASEPOWERREGENPNTS" then
												return {{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150},{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150}};
											else
												return 0;
											end
										else
											return CalcStat("BasePower",L);
										end
									else
										return StatLinInter("PntMPClassBaseNCPR","ClassBasePowerRegenPntS","ProgBNCPR","ClassBaseNCPRAdj",L,N,99);
									end
								else
									return CalcStat("BaseMorale",L);
								end
							else
								return 0.2;
							end
						else
							return 1;
						end
					elseif SN > "CLASSBASEVITALITY" then
						if SN < "COMBATDAMAGEMODNERFED" then
							if SN < "COMBATBASETACDPSBYLEVELMAX" then
								if SN < "COMBATBASEH" then
									if SN < "CLASSBASEWILLM" then
										if SN > "CLASSBASEWILLH" then
											if SN == "CLASSBASEWILLL" then
												return CalcStat("BaseWill",L,0.5);
											else
												return 0;
											end
										elseif SN == "CLASSBASEWILLH" then
											return CalcStat("BaseWill",L,1.5);
										else
											return 0;
										end
									elseif SN > "CLASSBASEWILLM" then
										if SN < "COMBATBASE" then
											if SN == "CLOTHARMOUR" then
												if Lm <= 50 then
													return L;
												else
													return 50;
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASE" then
											if SN == "COMBATBASECAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseH",N);
												else
													return CalcStat("CombatBaseL",N);
												end
											else
												return 0;
											end
										else
											return CalcStat("CombatBaseCat",WpnCodeIndex(C,1),L)*CalcStat("CombatBaseTypeMP",WpnCodeIndex(C,2))*CalcStat("CombatBaseQtyMPCat",WpnCodeIndex(C,1),WpnCodeIndex(C,3));
										end
									else
										return CalcStat("BaseWill",L,1.0);
									end
								elseif SN > "COMBATBASEH" then
									if SN < "COMBATBASEPHYDPS" then
										if SN < "COMBATBASEL" then
											if SN == "COMBATBASEHQTYMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.02,1.04,1.08,1.12},L);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASEL" then
											if SN == "COMBATBASELQTYMP" then
												return CalcStat("CombatBaseHQtyMP",L);
											else
												return 0;
											end
										else
											return 0.9*CalcStat("CombatBasePhyDPS",L)*1.08;
										end
									elseif SN > "COMBATBASEPHYDPS" then
										if SN < "COMBATBASETACDPS" then
											if SN == "COMBATBASEQTYMPCAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseHQtyMP",N);
												else
													return CalcStat("CombatBaseLQtyMP",N);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASETACDPS" then
											if SN == "COMBATBASETACDPSBYLEVEL" then
												if Lm <= 50 then
													return LinFmod(1,2,CalcStat("CombatBaseTacDPSByLevelMax",L),1,50,L);
												else
													return CalcStat("CombatBaseTacDPSByLevelMax",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 47 then
												return LinFmod(1,0.5,0.5,1,47,L);
											elseif Lm <= 50 then
												return LinFmod(1,0.5,0.8,47,50,L);
											elseif Lm <= 51 then
												return LinFmod(1,1,1,51,51,L);
											elseif Lm <= 60 then
												return LinFmod(1,1.32,6.6,52,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,6.6,9.1,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,9.1,19,65,75,L);
											elseif Lm <= 125 then
												return LinFmod(1,19,32,75,125,L);
											elseif Lm <= 175 then
												return LinFmod(1,32,49,125,175,L);
											elseif Lm <= 200 then
												return LinFmod(1,49,66,175,200,L);
											elseif Lm <= 222 then
												return LinFmod(1,66,88,200,222,L);
											elseif Lm <= 299 then
												return LinFmod(1,88,88,222,299,L);
											elseif Lm <= 349 then
												return LinFmod(1,99,124,300,349,L);
											elseif Lm <= 399 then
												return LinFmod(1,138,160,350,399,L);
											elseif Lm <= 449 then
												return LinFmod(1,178,219,400,449,L);
											elseif Lm <= 499 then
												return LinFmod(1,243,340,450,499,L);
											elseif Lm <= 549 then
												return LinFmod(1,380,520,500,549,L);
											elseif Lm <= 599 then
												return LinFmod(1,570,790,550,599,L);
											elseif Lm <= 649 then
												return RoundDblDown(RoundDblUp((L-49)/5.5)*36.7-2831,-1);
											else
												return CalcStat("CombatBaseTacDPS",649);
											end
										end
									else
										if Lm <= 50 then
											return LinFmod(1,2,20,1,50,L);
										elseif Lm <= 60 then
											return LinFmod(1,20,26.6,50,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,26.6,29.1,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,29.1,39,65,75,L);
										elseif Lm <= 125 then
											return LinFmod(1,39,52,75,125,L);
										elseif Lm <= 175 then
											return LinFmod(1,52,69,125,175,L);
										elseif Lm <= 200 then
											return LinFmod(1,69,86,175,200,L);
										elseif Lm <= 222 then
											return LinFmod(1,86,108,200,222,L);
										elseif Lm <= 300 then
											return LinFmod(1,108,108,222,300,L);
										elseif Lm <= 350 then
											return LinFmod(1,119,144,301,350,L);
										elseif Lm <= 400 then
											return LinFmod(1,158,180,351,400,L);
										elseif Lm <= 450 then
											return LinFmod(1,198,239,401,450,L);
										elseif Lm <= 500 then
											return LinFmod(1,263,360,451,500,L);
										elseif Lm <= 550 then
											return LinFmod(1,400,540,501,550,L);
										elseif Lm <= 600 then
											return LinFmod(1,590,810,551,600,L);
										elseif Lm <= 650 then
											return RoundDblDown(RoundDblUp((L-50)/5.5)*36.7-2811,-1);
										else
											return CalcStat("CombatBasePhyDPS",650);
										end
									end
								else
									return 1.0*CalcStat("CombatBasePhyDPS",L)*1.08;
								end
							elseif SN > "COMBATBASETACDPSBYLEVELMAX" then
								if SN < "COMBATDAMAGEMOD" then
									if SN < "COMBATBASETACHPSCURVES" then
										if SN < "COMBATBASETACHPSBYLEVEL" then
											if SN == "COMBATBASETACHPS" then
												if Lm <= 46 then
													return 0;
												elseif Lm <= 49 then
													return CalcStat("CombatBaseTacHPSCurves",L)-CalcStat("CombatBaseTacHPSByLevelMax",L);
												elseif Lm <= 55 then
													return 0.5;
												elseif Lm <= 599 then
													return CalcStat("CommonTacHPS",L)-CalcStat("CombatBaseTacHPSByLevelMax",L);
												elseif Lm <= 649 then
													return (RoundDblUp((4*L+2)/22)*678-RoundDblUp((L+1)/22))*0.175-11734.1-CalcStat("CombatBaseTacHPSByLevelMax",L);
												else
													return CalcStat("CombatBaseTacHPS",649);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASETACHPSBYLEVEL" then
											if SN == "COMBATBASETACHPSBYLEVELMAX" then
												return 5.659142856;
											else
												return 0;
											end
										else
											if Lm <= 46 then
												return CalcStat("CombatBaseTacHPSCurves",L);
											else
												return CalcStat("CombatBaseTacHPSByLevelMax",L);
											end
										end
									elseif SN > "COMBATBASETACHPSCURVES" then
										if SN < "COMBATBASETACHPSNOCLASS" then
											if SN == "COMBATBASETACHPSLVLTOILVL" then
												if Lm <= 75 then
													return LinFmod(1,1,75,1,75,L);
												elseif Lm <= 100 then
													return LinFmod(1,75,200,75,100,L);
												elseif Lm <= 104 then
													return RoundDbl(LinFmod(1,201.3,214.3,101,104,L));
												elseif Lm <= 105 then
													return 222;
												else
													return RoundDbl(CalcStat("LvlToILvl",L));
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASETACHPSNOCLASS" then
											if SN == "COMBATBASETYPEMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.4,1.4},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("CombatBaseTacHPSByLevel",L)+CalcStat("CombatBaseTacHPS",CalcStat("CombatBaseTacHPSLvlToILvl",L));
										end
									else
										if Lm <= 25 then
											return -0.0033215*L*L+0.2470730225*L+1.7562484775;
										else
											return -0.003714298*L*L+0.254286886*L+1.82140064;
										end
									end
								elseif SN > "COMBATDAMAGEMOD" then
									if SN < "COMBATDAMAGEMODHEALTHLOW" then
										if SN < "COMBATDAMAGEMODHEALTH" then
											if SN == "COMBATDAMAGEMODENERGY" then
												return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBEnergy","",L,N,2);
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODHEALTH" then
											if SN == "COMBATDAMAGEMODHEALTHADJ" then
												if Lm <= 141 then
													return 1;
												else
													return 0.63;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBHealth","CombatDamageModHealthAdj",L,N,2);
										end
									elseif SN > "COMBATDAMAGEMODHEALTHLOW" then
										if SN < "COMBATDAMAGEMODHEALTHMEDIUM" then
											if SN == "COMBATDAMAGEMODHEALTHLOWADJ" then
												if Lm <= 1 then
													return 0.144*N;
												elseif Lm <= 25 then
													return 0.324*N;
												elseif Lm <= 50 then
													return 0.576*N;
												else
													return 0.9*N;
												end
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODHEALTHMEDIUM" then
											if SN == "COMBATDAMAGEMODHEALTHMEDIUMADJ" then
												if Lm <= 1 then
													return 0.36*N;
												elseif Lm <= 25 then
													return 0.54*N;
												elseif Lm <= 50 then
													return 0.72*N;
												else
													return 0.9*N;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBHealth","CombatDamageModHealthMediumAdj",L,N,99);
										end
									else
										return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBHealth","CombatDamageModHealthLowAdj",L,N,99);
									end
								else
									return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","",L,N,2);
								end
							else
								return 20;
							end
						elseif SN > "COMBATDAMAGEMODNERFED" then
							if SN < "CPTCOVMAIN" then
								if SN < "COMBATINHEAL" then
									if SN < "COMBATDAMAGEMODPETSADJ" then
										if SN < "COMBATDAMAGEMODNPCS" then
											if SN == "COMBATDAMAGEMODNERFEDADJ" then
												if Lm <= 141 then
													return 1;
												else
													return 0.82;
												end
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODNPCS" then
											if SN == "COMBATDAMAGEMODPETS" then
												return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,99);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","",L,N,99);
										end
									elseif SN > "COMBATDAMAGEMODPETSADJ" then
										if SN < "COMBATDAMAGEMODPLAYERS" then
											if SN == "COMBATDAMAGEMODPETSRND" then
												return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,2);
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODPLAYERS" then
											if SN == "COMBATDAMAGEMODPLAYERSADJ" then
												if Lm <= 1 then
													return 0.8*N;
												elseif Lm <= 25 then
													return 0.9*N;
												else
													return N;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","CombatDamageModPlayersAdj",L,N,99);
										end
									else
										if Lm <= 1 then
											return N;
										elseif Lm <= 25 then
											return 0.4*N;
										elseif Lm <= 50 then
											return 0.5*N;
										elseif Lm <= 60 then
											return 0.6*N;
										elseif Lm <= 65 then
											return 0.7*N;
										elseif Lm <= 75 then
											return 0.8*N;
										elseif Lm <= 85 then
											return 0.85*N;
										elseif Lm <= 95 then
											return 0.9*N;
										elseif Lm <= 100 then
											return 0.95*N;
										else
											return N;
										end
									end
								elseif SN > "COMBATINHEAL" then
									if SN < "CPTBLADECRITDEF" then
										if SN < "COMMONTACHPSITEMPNTS" then
											if SN == "COMMONTACHPS" then
												return StatLinInter("PntMPTacHPS","CommonTacHPSItemPntS","ProgBTacHPS","",L,N,99);
											else
												return 0;
											end
										elseif SN > "COMMONTACHPSITEMPNTS" then
											if SN == "CONSTSTATC" then
												return CalcStat(C,1,L);
											else
												return 0;
											end
										else
											return {{1,50,60,65,75,125,175,200,222,299,300,349,350,399,400,449,450,499,500,549,550,599},{1,50,60,65,75,85,95,100,105,105,106,115,116,120,121,130,131,140,141,150,151,160}};
										end
									elseif SN > "CPTBLADECRITDEF" then
										if SN < "CPTBLADETACMAS" then
											if SN == "CPTBLADEPHYMAS" then
												return CalcStat("PhyMasT",L,1.2);
											else
												return 0;
											end
										elseif SN > "CPTBLADETACMAS" then
											if SN == "CPTCOVFATE" then
												return CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("TacMasT",L,0.8);
										end
									else
										return CalcStat("CritDefT",L,0.8);
									end
								else
									return CalcStat("InHeal",L,2.0);
								end
							elseif SN > "CPTCOVMAIN" then
								if SN < "CPTSHIELDPHYMAS" then
									if SN < "CPTIDOMEFATE" then
										if SN < "CPTCOVVITALITY" then
											if SN == "CPTCOVPHYMIT" then
												return CalcStat("PhyMitT",L,2.4);
											else
												return 0;
											end
										elseif SN > "CPTCOVVITALITY" then
											if SN == "CPTCRITDEF" then
												return CalcStat("CritDef",L,0.6);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,0.4);
										end
									elseif SN > "CPTIDOMEFATE" then
										if SN < "CPTIDOMEVITALITY" then
											if SN == "CPTIDOMEMAIN" then
												return CalcStat("MainT",L,0.4);
											else
												return 0;
											end
										elseif SN > "CPTIDOMEVITALITY" then
											if SN == "CPTSHIELDCRITDEF" then
												return CalcStat("CritDefT",L,1.2);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,0.4);
										end
									else
										return CalcStat("FateT",L,0.4);
									end
								elseif SN > "CPTSHIELDPHYMAS" then
									if SN < "CPTSONGTACMAS" then
										if SN < "CPTSONGCRITDEF" then
											if SN == "CPTSHIELDTACMAS" then
												return CalcStat("TacMasT",L,0.8);
											else
												return 0;
											end
										elseif SN > "CPTSONGCRITDEF" then
											if SN == "CPTSONGPHYMAS" then
												return CalcStat("PhyMasT",L,0.8);
											else
												return 0;
											end
										else
											return CalcStat("CritDefT",L,0.8);
										end
									elseif SN > "CPTSONGTACMAS" then
										if SN < "CPTSTANDALONETACMAS" then
											if SN == "CPTSTANDALONEPHYMAS" then
												return CalcStat("PhyMasT",L,1.2);
											else
												return 0;
											end
										elseif SN > "CPTSTANDALONETACMAS" then
											if SN == "CREEPAUDACITYCCDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return 0.5;
												else
													return 0.4;
												end
											else
												return 0;
											end
										else
											return CalcStat("TacMasT",L,0.8);
										end
									else
										return CalcStat("TacMasT",L,1.2);
									end
								else
									return CalcStat("PhyMasT",L,0.8);
								end
							else
								return CalcStat("MainT",L,0.4);
							end
						else
							return StatLinInter("PntMPCombatDamageMod","TraitPntSVital","ProgBCombatDamageMod","CombatDamageModNerfedAdj",L,N,2);
						end
					else
						return CalcStat("BaseVitality",L);
					end
				else
					return CalcStat("ClassBaseVitality",L);
				end
			else
				return -CalcStat("ResistT",L,0.4);
			end
		elseif SN > "CREEPAUDACITYCOST" then
			if SN < "HOBHOBBITTOUGHNVITALITY" then
				if SN < "FINESSEC" then
					if SN < "DEVHITPRATPA" then
						if SN < "CRITDEFPRATPCAPR" then
							if SN < "CREEPBATPROMOUTHEALP" then
								if SN < "CREEPAUDACITYRNGREDP" then
									if SN < "CREEPAUDACITYMELDMGP" then
										if SN > "CREEPAUDACITYCOSTBASE" then
											if SN == "CREEPAUDACITYDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 10 then
													return LinFmod(1,1,1.2,1,10,L);
												elseif Lm <= 36 then
													return LinFmod(1,1.2,1.25,10,36,L);
												elseif Lm <= 41 then
													return LinFmod(1,1.25,1.3,36,41,L);
												elseif Lm <= 60 then
													return LinFmod(1,1.3,1.25,41,60,L);
												else
													return CalcStat("CreepAudacityDmgP",60);
												end
											else
												return 0;
											end
										elseif SN == "CREEPAUDACITYCOSTBASE" then
											if Lm <= 0 then
												return 0;
											elseif Lm <= 2 then
												return 2*L;
											elseif Lm <= 9 then
												return 2*L-1;
											elseif Lm <= 15 then
												return 3*L;
											elseif Lm <= 25 then
												return 6*L;
											elseif Lm <= 29 then
												return 9*L;
											elseif Lm <= 36 then
												return 300;
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "CREEPAUDACITYMELDMGP" then
										if SN < "CREEPAUDACITYREDP" then
											if SN == "CREEPAUDACITYMELREDP" then
												return CalcStat("CreepAudacityRedP",L);
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYREDP" then
											if SN == "CREEPAUDACITYRNGDMGP" then
												return CalcStat("CreepAudacityDmgP",L);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 10 then
												return LinFmod(1,0.55,0.55,1,10,L);
											elseif Lm <= 36 then
												return LinFmod(1,0.55,0.55,10,36,L);
											elseif Lm <= 41 then
												return LinFmod(1,0.55,0.55,36,41,L);
											elseif Lm <= 60 then
												return LinFmod(1,0.55,0.55,41,60,L);
											else
												return CalcStat("CreepAudacityRedP",60);
											end
										end
									else
										return CalcStat("CreepAudacityDmgP",L);
									end
								elseif SN > "CREEPAUDACITYRNGREDP" then
									if SN < "CREEPBATPROMFINESSE" then
										if SN < "CREEPAUDACITYTACREDP" then
											if SN == "CREEPAUDACITYTACDMGP" then
												return CalcStat("CreepAudacityDmgP",L);
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYTACREDP" then
											if SN == "CREEPBATPROMDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 5 then
													return LinFmod(1,0.005,0.02,1,5,L);
												elseif Lm <= 10 then
													return LinFmod(1,0.02,0.045,5,10,L);
												elseif Lm <= 15 then
													return LinFmod(1,0.045,0.075,10,15,L);
												else
													return CalcStat("CreepBatPromDmgP",15);
												end
											else
												return 0;
											end
										else
											return CalcStat("CreepAudacityRedP",L);
										end
									elseif SN > "CREEPBATPROMFINESSE" then
										if SN < "CREEPBATPROMINHEALP" then
											if SN == "CREEPBATPROMHEALTHP" then
												return CalcStat("CreepBatPromVitalP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMINHEALP" then
											if SN == "CREEPBATPROMMELDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return 0;
									end
								else
									return CalcStat("CreepAudacityRedP",L);
								end
							elseif SN > "CREEPBATPROMOUTHEALP" then
								if SN < "CRITDEFC" then
									if SN < "CREEPBATPROMVITALP" then
										if SN < "CREEPBATPROMRNGDMGP" then
											if SN == "CREEPBATPROMPOWERP" then
												return CalcStat("CreepBatPromVitalP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMRNGDMGP" then
											if SN == "CREEPBATPROMTACDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("CreepBatPromDmgP",L);
										end
									elseif SN > "CREEPBATPROMVITALP" then
										if SN < "CREEPPROGB" then
											if SN == "CREEPPNTS" then
												return {{1,150},{1,150}};
											else
												return 0;
											end
										elseif SN > "CREEPPROGB" then
											if SN == "CRITDEF" then
												return StatLinInter("PntMPCritDef","ItemPntS","ProgBCritDef","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										else
											return LinFmod(1,0.01,1,1,CalcStat("LevelCap",L),L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 5 then
											return LinFmod(1,1.02,1.1,1,5,L);
										elseif Lm <= 10 then
											return LinFmod(1,1.1,1.15,5,10,L);
										elseif Lm <= 15 then
											return LinFmod(1,1.15,1.2,10,15,L);
										else
											return CalcStat("CreepBatPromVitalP",15);
										end
									end
								elseif SN > "CRITDEFC" then
									if SN < "CRITDEFPRATPA" then
										if SN < "CRITDEFPPRAT" then
											if SN == "CRITDEFPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "CRITDEFPPRAT" then
											if SN == "CRITDEFPRATP" then
												return CalcPercAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCapR",L),N);
										end
									elseif SN > "CRITDEFPRATPA" then
										if SN < "CRITDEFPRATPC" then
											if SN == "CRITDEFPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "CRITDEFPRATPC" then
											if SN == "CRITDEFPRATPCAP" then
												return 80;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 240;
									end
								else
									return StatLinInter("PntMPCritDefC","CreepPntS","ProgBCritDefC","",L,N,0);
								end
							else
								if Lm <= 0 then
									return 0;
								elseif Lm <= 5 then
									return LinFmod(1,0.02,0.1,1,5,L);
								elseif Lm <= 10 then
									return LinFmod(1,0.1,0.15,5,10,L);
								elseif Lm <= 15 then
									return LinFmod(1,0.15,0.2,10,15,L);
								else
									return CalcStat("CreepBatPromOutHealP",15);
								end
							end
						elseif SN > "CRITDEFPRATPCAPR" then
							if SN < "CRITMAGNPRATP" then
								if SN < "CRITHITPRATPA" then
									if SN < "CRITHITOLD" then
										if SN < "CRITHIT" then
											if SN == "CRITDEFT" then
												return StatLinInter("PntMPCritDef","TraitPntS","ProgBCritDef","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										elseif SN > "CRITHIT" then
											if SN == "CRITHITC" then
												return StatLinInter("PntMPCritHitC","CreepPntS","ProgBCritHitC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCritHit","ItemPntS","ProgBCritHit","AdjUmbarItem",L,N,0);
										end
									elseif SN > "CRITHITOLD" then
										if SN < "CRITHITPPRAT" then
											if SN == "CRITHITPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "CRITHITPPRAT" then
											if SN == "CRITHITPRATP" then
												return CalcPercAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCapR",L),N);
										end
									else
										return CalcStat("CritHit",L,N);
									end
								elseif SN > "CRITHITPRATPA" then
									if SN < "CRITHITPRATPCAPR" then
										if SN < "CRITHITPRATPC" then
											if SN == "CRITHITPRATPB" then
												return CalcStat("BRatRounded",L,"BRatExtra");
											else
												return 0;
											end
										elseif SN > "CRITHITPRATPC" then
											if SN == "CRITHITPRATPCAP" then
												return 25;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "CRITHITPRATPCAPR" then
										if SN < "CRITMAGNPBONUS" then
											if SN == "CRITHITT" then
												return StatLinInter("PntMPCritHit","TraitPntS","ProgBCritHit","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										elseif SN > "CRITMAGNPBONUS" then
											if SN == "CRITMAGNPPRAT" then
												return CalcRatAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("CritHitPRatPB",L)*CalcStat("CritHitPRatPC",L);
									end
								else
									return 75;
								end
							elseif SN > "CRITMAGNPRATP" then
								if SN < "DEFILERCANBLOCK" then
									if SN < "CRITMAGNPRATPCAP" then
										if SN < "CRITMAGNPRATPB" then
											if SN == "CRITMAGNPRATPA" then
												return 225;
											else
												return 0;
											end
										elseif SN > "CRITMAGNPRATPB" then
											if SN == "CRITMAGNPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatCritMagn");
										end
									elseif SN > "CRITMAGNPRATPCAP" then
										if SN < "CRYRESIST" then
											if SN == "CRITMAGNPRATPCAPR" then
												return CalcStat("CritMagnPRatPB",L)*CalcStat("CritMagnPRatPC",L);
											else
												return 0;
											end
										elseif SN > "CRYRESIST" then
											if SN == "CRYRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("ResistAdd",L,N);
										end
									else
										return 75;
									end
								elseif SN > "DEFILERCANBLOCK" then
									if SN < "DEFILERCDHASPOWER" then
										if SN < "DEFILERCDCALCTYPENONPHYMIT" then
											if SN == "DEFILERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "DEFILERCDCALCTYPENONPHYMIT" then
											if SN == "DEFILERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									elseif SN > "DEFILERCDHASPOWER" then
										if SN < "DEVHITPPRAT" then
											if SN == "DEVHITPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "DEVHITPPRAT" then
											if SN == "DEVHITPRATP" then
												return CalcPercAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCapR",L),N);
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return CalcPercAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCap",L),N);
							end
						else
							return CalcStat("CritDefPRatPB",L)*CalcStat("CritDefPRatPC",L);
						end
					elseif SN > "DEVHITPRATPA" then
						if SN < "DWARFUNWEARBATTLEICPR" then
							if SN < "DWARFRDPSVTWONAME" then
								if SN < "DMGTYPEMIT" then
									if SN < "DEVHITPRATPCAP" then
										if SN > "DEVHITPRATPB" then
											if SN == "DEVHITPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "DEVHITPRATPB" then
											return CalcStat("BRatRounded",L,"BRatDevHit");
										else
											return 0;
										end
									elseif SN > "DEVHITPRATPCAP" then
										if SN < "DISEASERESIST" then
											if SN == "DEVHITPRATPCAPR" then
												return CalcStat("DevHitPRatPB",L)*CalcStat("DevHitPRatPC",L);
											else
												return 0;
											end
										elseif SN > "DISEASERESIST" then
											if SN == "DISEASERESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("ResistAdd",L,N);
										end
									else
										return 10;
									end
								elseif SN > "DMGTYPEMIT" then
									if SN < "DWARFLOSTDWARFKDSFATE" then
										if SN < "DWARFENDURVITALITY" then
											if SN == "DMGTYPEMITT" then
												return StatLinInter("PntMPDmgTypeMitT","TraitPntS","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										elseif SN > "DWARFENDURVITALITY" then
											if SN == "DWARFFATEFULDWARFFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									elseif SN > "DWARFLOSTDWARFKDSFATE" then
										if SN < "DWARFRDPSVONENAME" then
											if SN == "DWARFRDPSVONEFATE" then
												return CalcStat("DwarfFatefulDwarfFate",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDPSVONENAME" then
											if SN == "DWARFRDPSVTWOBLOCK" then
												return CalcStat("DwarfShieldBrwlBlock",L);
											else
												return 0;
											end
										else
											return "Fateful Dwarf";
										end
									else
										return -CalcStat("FateT",L,0.4);
									end
								else
									return StatLinInter("PntMPDmgTypeMit","ItemPntS","ProgBMitigation","AdjUmbarItemMit",L,N,0);
								end
							elseif SN > "DWARFRDPSVTWONAME" then
								if SN < "DWARFRDTRAITPHYMITP" then
									if SN < "DWARFRDTRAITICPR" then
										if SN < "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITAGILITY" then
												return CalcStat("DwarfStockyAgility",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITICMR" then
												return CalcStat("DwarfUnwearBattleICMR",L);
											else
												return 0;
											end
										else
											return CalcStat("DwarfLostDwarfKdsFate",L);
										end
									elseif SN > "DWARFRDTRAITICPR" then
										if SN < "DWARFRDTRAITNCMR" then
											if SN == "DWARFRDTRAITMIGHT" then
												return CalcStat("DwarfSturdinessMight",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITNCMR" then
											if SN == "DWARFRDTRAITNCPR" then
												return CalcStat("DwarfUnwearBattleNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("DwarfUnwearBattleNCMR",L);
										end
									else
										return CalcStat("DwarfUnwearBattleICPR",L);
									end
								elseif SN > "DWARFRDTRAITPHYMITP" then
									if SN < "DWARFSTURDINESSMIGHT" then
										if SN < "DWARFSHIELDBRWLBLOCK" then
											if SN == "DWARFRDTRAITVITALITY" then
												return CalcStat("DwarfSturdinessVitality",L);
											else
												return 0;
											end
										elseif SN > "DWARFSHIELDBRWLBLOCK" then
											if SN == "DWARFSTOCKYAGILITY" then
												return CalcStat("AgilityT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,0.8);
										end
									elseif SN > "DWARFSTURDINESSMIGHT" then
										if SN < "DWARFSTURDINESSVITALITY" then
											if SN == "DWARFSTURDINESSPHYMITP" then
												return 1;
											else
												return 0;
											end
										elseif SN > "DWARFSTURDINESSVITALITY" then
											if SN == "DWARFUNWEARBATTLEICMR" then
												return CalcStat("ICMRT",L,0.6);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									else
										return CalcStat("MightT",L,1.0);
									end
								else
									return CalcStat("DwarfSturdinessPhyMitP",L);
								end
							else
								return "Shield Brawler";
							end
						elseif SN > "DWARFUNWEARBATTLEICPR" then
							if SN < "EVADEC" then
								if SN < "ELFRDPSVTWONAME" then
									if SN < "ELFFADINGFIRSTBORNFATE" then
										if SN < "DWARFUNWEARBATTLENCPR" then
											if SN == "DWARFUNWEARBATTLENCMR" then
												return -CalcStat("NCMRT",L,0.4);
											else
												return 0;
											end
										elseif SN > "DWARFUNWEARBATTLENCPR" then
											if SN == "ELFAGILITYWOODSAGILITY" then
												return CalcStat("AgilityT",L,1.0);
											else
												return 0;
											end
										else
											return -CalcStat("NCPRT",L,0.4);
										end
									elseif SN > "ELFFADINGFIRSTBORNFATE" then
										if SN < "ELFRDPSVONEFATE" then
											if SN == "ELFFRIENDOFMANFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										elseif SN > "ELFRDPSVONEFATE" then
											if SN == "ELFRDPSVONENAME" then
												return "Friend Of Man";
											else
												return 0;
											end
										else
											return CalcStat("ElfFriendOfManFate",L);
										end
									else
										return -CalcStat("FateT",L,0.4);
									end
								elseif SN > "ELFRDPSVTWONAME" then
									if SN < "ELFRDTRAITNCMR" then
										if SN < "ELFRDTRAITFATE" then
											if SN == "ELFRDTRAITAGILITY" then
												return CalcStat("ElfAgilityWoodsAgility",L);
											else
												return 0;
											end
										elseif SN > "ELFRDTRAITFATE" then
											if SN == "ELFRDTRAITMORALE" then
												return CalcStat("ElfSorrowFirstbornMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ElfFadingFirstbornFate",L);
										end
									elseif SN > "ELFRDTRAITNCMR" then
										if SN < "ELFSORROWFIRSTBORNNCMR" then
											if SN == "ELFSORROWFIRSTBORNMORALE" then
												return -CalcStat("MoraleT",L,0.4);
											else
												return 0;
											end
										elseif SN > "ELFSORROWFIRSTBORNNCMR" then
											if SN == "EVADE" then
												return CalcStat("BPE",L,N);
											else
												return 0;
											end
										else
											return -CalcStat("NCMRT",L,0.4);
										end
									else
										return CalcStat("ElfSorrowFirstbornNCMR",L);
									end
								else
									return "";
								end
							elseif SN > "EVADEC" then
								if SN < "EVADEPRATPCAPR" then
									if SN < "EVADEPRATPA" then
										if SN < "EVADEPPRAT" then
											if SN == "EVADEPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										elseif SN > "EVADEPPRAT" then
											if SN == "EVADEPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPPRat",L,N);
										end
									elseif SN > "EVADEPRATPA" then
										if SN < "EVADEPRATPC" then
											if SN == "EVADEPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "EVADEPRATPC" then
											if SN == "EVADEPRATPCAP" then
												return CalcStat("BPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPC",L);
										end
									else
										return CalcStat("BPEPRatPA",L);
									end
								elseif SN > "EVADEPRATPCAPR" then
									if SN < "FATET" then
										if SN < "FATE" then
											if SN == "EVADET" then
												return CalcStat("BPET",L,N);
											else
												return 0;
											end
										elseif SN > "FATE" then
											if SN == "FATEC" then
												return RoundDblDown(StatLinInter("PntMPFateC","CreepPntS","ProgBFateC","",L,N,0));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPFate","ItemPntSVital","ProgBFate","",L,N,0));
										end
									elseif SN > "FATET" then
										if SN < "FEARRESISTT" then
											if SN == "FEARRESIST" then
												return CalcStat("ResistAdd",L,N);
											else
												return 0;
											end
										elseif SN > "FEARRESISTT" then
											if SN == "FINESSE" then
												return StatLinInter("PntMPFinesse","ItemPntS","ProgBFinesse","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										return RoundDblDown(StatLinInter("PntMPFate","TraitPntSVital","ProgBFate","",L,N,0));
									end
								else
									return CalcStat("BPEPRatPCapR",L);
								end
							else
								return CalcStat("BPEC",L,N);
							end
						else
							return CalcStat("ICPRT",L,0.6);
						end
					else
						return 30;
					end
				elseif SN > "FINESSEC" then
					if SN < "GUARDIANCDBASEMIGHT" then
						if SN < "FREEPAUDACITYTACDMGP" then
							if SN < "FOODNCMRM" then
								if SN < "FINESSEPRATPCAP" then
									if SN < "FINESSEPRATP" then
										if SN > "FINESSEPBONUS" then
											if SN == "FINESSEPPRAT" then
												return CalcRatAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN == "FINESSEPBONUS" then
											return 0;
										else
											return 0;
										end
									elseif SN > "FINESSEPRATP" then
										if SN < "FINESSEPRATPB" then
											if SN == "FINESSEPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN > "FINESSEPRATPB" then
											if SN == "FINESSEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									else
										return CalcPercAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCap",L),N);
									end
								elseif SN > "FINESSEPRATPCAP" then
									if SN < "FIREMITT" then
										if SN < "FINESSET" then
											if SN == "FINESSEPRATPCAPR" then
												return CalcStat("FinessePRatPB",L)*CalcStat("FinessePRatPC",L);
											else
												return 0;
											end
										elseif SN > "FINESSET" then
											if SN == "FIREMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPFinesseT","TraitPntS","ProgBFinesse","AdjUmbarTrait",L,N,0);
										end
									elseif SN > "FIREMITT" then
										if SN < "FOODNCMRH" then
											if SN == "FOODNCMRBASEPROG" then
												if Lm <= 1 then
													return (N*70.2+210)/60;
												elseif Lm <= 2 then
													return (N*84+216)/120;
												else
													return (N*83.7+270)/90;
												end
											else
												return 0;
											end
										elseif SN > "FOODNCMRH" then
											if SN == "FOODNCMRL" then
												return CalcStat("FoodNCMRProg",L,2)*N;
											else
												return 0;
											end
										else
											return CalcStat("FoodNCMRProg",L,1)*N;
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								else
									return 50;
								end
							elseif SN > "FOODNCMRM" then
								if SN < "FREEPAUDACITYCCDP" then
									if SN < "FOODNCPRL" then
										if SN < "FOODNCPRBASEPROG" then
											if SN == "FOODNCMRPROG" then
												if Lm <= 50 then
													return CalcStat("FoodNCMRBaseProg",N,L);
												elseif Lm <= 52 then
													return CalcStat("FoodNCMRBaseProg",N,50);
												elseif Lm <= 77 then
													return CalcStat("FoodNCMRBaseProg",N,L-3);
												elseif Lm <= 111 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+143)/3));
												elseif Lm <= 140 then
													return CalcStat("FoodNCMRBaseProg",N,85);
												elseif Lm <= 217 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+200)/4));
												elseif Lm <= 221 then
													return CalcStat("FoodNCMRBaseProg",N,104);
												elseif Lm <= 299 then
													return CalcStat("FoodNCMRBaseProg",N,105);
												elseif Lm <= 320 then
													return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L-53)/(7/3)));
												elseif Lm <= 325 then
													return CalcStat("FoodNCMRBaseProg",N,114);
												elseif Lm <= 326 then
													return CalcStat("FoodNCMRBaseProg",N,115);
												else
													return ExpFmod(CalcStat("FoodNCMRProg",326,N),327,1,L,1);
												end
											else
												return 0;
											end
										elseif SN > "FOODNCPRBASEPROG" then
											if SN == "FOODNCPRH" then
												return CalcStat("FoodNCPRProg",L,1)*N;
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return (N*75+300)/60;
											elseif Lm <= 2 then
												return (N*100+350)/120;
											else
												return (N*90+337.5)/90;
											end
										end
									elseif SN > "FOODNCPRL" then
										if SN < "FOODNCPRPROG" then
											if SN == "FOODNCPRM" then
												return CalcStat("FoodNCPRProg",L,3)*N;
											else
												return 0;
											end
										elseif SN > "FOODNCPRPROG" then
											if SN == "FOODRESIST" then
												return StatLinInter("PntMPFoodResist","ItemPntS","ProgBResist","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return CalcStat("FoodNCPRBaseProg",N,L);
											elseif Lm <= 6 then
												return CalcStat("FoodNCPRBaseProg",N,L)-0.25;
											elseif Lm <= 24 then
												return CalcStat("FoodNCPRBaseProg",N,L)-0.5;
											elseif Lm <= 48 then
												return CalcStat("FoodNCPRBaseProg",N,L)-0.75;
											elseif Lm <= 61 then
												return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+1;
											elseif Lm <= 66 then
												return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+2;
											elseif Lm <= 72 then
												return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+3;
											elseif Lm <= 326 then
												return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+4;
											else
												return ExpFmod(CalcStat("FoodNCPRProg",326,N),327,1,L,1);
											end
										end
									else
										return CalcStat("FoodNCPRProg",L,2)*N;
									end
								elseif SN > "FREEPAUDACITYCCDP" then
									if SN < "FREEPAUDACITYMORALEP" then
										if SN < "FREEPAUDACITYMELDMGP" then
											if SN == "FREEPAUDACITYDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return 0.75;
												elseif Lm <= 36 then
													return LinFmod(1,1.15,1.25,17,36,L);
												else
													return 1.25;
												end
											else
												return 0;
											end
										elseif SN > "FREEPAUDACITYMELDMGP" then
											if SN == "FREEPAUDACITYMELREDP" then
												return CalcStat("FreepAudacityRedP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepAudacityDmgP",L);
										end
									elseif SN > "FREEPAUDACITYMORALEP" then
										if SN < "FREEPAUDACITYRNGDMGP" then
											if SN == "FREEPAUDACITYREDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return 1.5;
												else
													return 0.55;
												end
											else
												return 0;
											end
										elseif SN > "FREEPAUDACITYRNGDMGP" then
											if SN == "FREEPAUDACITYRNGREDP" then
												return CalcStat("FreepAudacityRedP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepAudacityDmgP",L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 16 then
											return LinFmod(1,0.5,1.3,1,16,L);
										else
											return 1.4;
										end
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 16 then
										return 0.5;
									else
										return 0.4;
									end
								end
							else
								return CalcStat("FoodNCMRProg",L,3)*N;
							end
						elseif SN > "FREEPAUDACITYTACDMGP" then
							if SN < "GRDTENDERIZET4CRITHIT" then
								if SN < "FREEPBATPROMVITALP" then
									if SN < "FREEPBATPROMMELDMGP" then
										if SN < "FREEPBATPROMDMGP" then
											if SN == "FREEPAUDACITYTACREDP" then
												return CalcStat("FreepAudacityRedP",L);
											else
												return 0;
											end
										elseif SN > "FREEPBATPROMDMGP" then
											if SN == "FREEPBATPROMHEALTHP" then
												return CalcStat("FreepBatPromVitalP",L);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 1 then
												return 0.005;
											elseif Lm <= 14 then
												return LinFmod(1,0.005,0.065,2,14,L);
											else
												return 0.075;
											end
										end
									elseif SN > "FREEPBATPROMMELDMGP" then
										if SN < "FREEPBATPROMRNGDMGP" then
											if SN == "FREEPBATPROMPOWERP" then
												return CalcStat("FreepBatPromVitalP",L);
											else
												return 0;
											end
										elseif SN > "FREEPBATPROMRNGDMGP" then
											if SN == "FREEPBATPROMTACDMGP" then
												return CalcStat("FreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepBatPromDmgP",L);
										end
									else
										return CalcStat("FreepBatPromDmgP",L);
									end
								elseif SN > "FREEPBATPROMVITALP" then
									if SN < "GRDRELENTLASSFINESSE" then
										if SN < "FROSTMITT" then
											if SN == "FROSTMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN > "FROSTMITT" then
											if SN == "GRDCRITDEF" then
												return CalcStat("CritDef",L);
											else
												return 0;
											end
										else
											return CalcStat("DmgTypeMitT",L,N);
										end
									elseif SN > "GRDRELENTLASSFINESSE" then
										if SN < "GRDTENDERIZET2CRITHIT" then
											if SN == "GRDTENDERIZECRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN > "GRDTENDERIZET2CRITHIT" then
											if SN == "GRDTENDERIZET3CRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait357912Choice",N)*0.2);
											else
												return 0;
											end
										else
											return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.4);
										end
									else
										return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.2);
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 5 then
										return LinFmod(1,1.02,1.1,1,5,L);
									elseif Lm <= 10 then
										return LinFmod(1,1.1,1.15,5,10,L);
									elseif Lm <= 15 then
										return LinFmod(1,1.15,1.2,10,15,L);
									else
										return CalcStat("FreepBatPromVitalP",15);
									end
								end
							elseif SN > "GRDTENDERIZET4CRITHIT" then
								if SN < "GUARDIANCDARMOURTOCOMPHYMIT" then
									if SN < "GUARDIANCDAGILITYTOFINESSE" then
										if SN < "GRDWARDTACTTACMIT" then
											if SN == "GRDTENDERIZET5CRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait58121620Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN > "GRDWARDTACTTACMIT" then
											if SN == "GUARDIANCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.2);
										end
									elseif SN > "GUARDIANCDAGILITYTOFINESSE" then
										if SN < "GUARDIANCDAGILITYTOPARRY" then
											if SN == "GUARDIANCDAGILITYTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDAGILITYTOPARRY" then
											if SN == "GUARDIANCDAGILITYTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "GUARDIANCDARMOURTOCOMPHYMIT" then
									if SN < "GUARDIANCDBASEAGILITY" then
										if SN < "GUARDIANCDARMOURTOTACMIT" then
											if SN == "GUARDIANCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDARMOURTOTACMIT" then
											if SN == "GUARDIANCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "GUARDIANCDBASEAGILITY" then
										if SN < "GUARDIANCDBASEICMR" then
											if SN == "GUARDIANCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASEICMR" then
											if SN == "GUARDIANCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRH",L);
										end
									else
										return CalcStat("ClassBaseAgilityM",L);
									end
								else
									return 1;
								end
							else
								return CalcStat("CritHitT",L,CalcStat("Trait47101316Choice",N)*0.2);
							end
						else
							return CalcStat("FreepAudacityDmgP",L);
						end
					elseif SN > "GUARDIANCDBASEMIGHT" then
						if SN < "GUARDIANCDWILLTORESIST" then
							if SN < "GUARDIANCDMIGHTTOCRITHIT" then
								if SN < "GUARDIANCDCALCTYPECOMPHYMIT" then
									if SN < "GUARDIANCDBASENCPR" then
										if SN > "GUARDIANCDBASEMORALE" then
											if SN == "GUARDIANCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										elseif SN == "GUARDIANCDBASEMORALE" then
											return CalcStat("ClassBaseMorale",L);
										else
											return 0;
										end
									elseif SN > "GUARDIANCDBASENCPR" then
										if SN < "GUARDIANCDBASEVITALITY" then
											if SN == "GUARDIANCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASEVITALITY" then
											if SN == "GUARDIANCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									else
										return CalcStat("ClassBaseNCPR",L);
									end
								elseif SN > "GUARDIANCDCALCTYPECOMPHYMIT" then
									if SN < "GUARDIANCDFATETONCPR" then
										if SN < "GUARDIANCDCALCTYPETACMIT" then
											if SN == "GUARDIANCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDCALCTYPETACMIT" then
											if SN == "GUARDIANCDCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "GUARDIANCDFATETONCPR" then
										if SN < "GUARDIANCDHASPOWER" then
											if SN == "GUARDIANCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDHASPOWER" then
											if SN == "GUARDIANCDMIGHTTOBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.07;
									end
								else
									return 14;
								end
							elseif SN > "GUARDIANCDMIGHTTOCRITHIT" then
								if SN < "GUARDIANCDTACMASTOOUTHEAL" then
									if SN < "GUARDIANCDMIGHTTOPHYMIT" then
										if SN < "GUARDIANCDMIGHTTOPARRY" then
											if SN == "GUARDIANCDMIGHTTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDMIGHTTOPARRY" then
											if SN == "GUARDIANCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "GUARDIANCDMIGHTTOPHYMIT" then
										if SN < "GUARDIANCDPHYMITTOCOMPHYMIT" then
											if SN == "GUARDIANCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDPHYMITTOCOMPHYMIT" then
											if SN == "GUARDIANCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "GUARDIANCDTACMASTOOUTHEAL" then
									if SN < "GUARDIANCDWILLTOFINESSE" then
										if SN < "GUARDIANCDVITALITYTOMORALE" then
											if SN == "GUARDIANCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDVITALITYTOMORALE" then
											if SN == "GUARDIANCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 4.5;
										end
									elseif SN > "GUARDIANCDWILLTOFINESSE" then
										if SN < "GUARDIANCDWILLTOPHYMAS" then
											if SN == "GUARDIANCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDWILLTOPHYMAS" then
											if SN == "GUARDIANCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "GUARDIANCDWILLTORESIST" then
							if SN < "HNTARMOURRENDPARRY" then
								if SN < "HIGHELFRDPSVONEWILL" then
									if SN < "HELFPEACEELDARNCMR" then
										if SN < "HELFFADINGFIRSTBORNFATE" then
											if SN == "GUARDIANCDWILLTOTACMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "HELFFADINGFIRSTBORNFATE" then
											if SN == "HELFPEACEELDARMORALE" then
												return CalcStat("MoraleT",L,1.0);
											else
												return 0;
											end
										else
											return -CalcStat("FateT",L,0.4);
										end
									elseif SN > "HELFPEACEELDARNCMR" then
										if SN < "HELFTHOSEWHOREMAINWILL" then
											if SN == "HELFSORROWUNDYINGWILL" then
												return -CalcStat("WillT",L,0.4);
											else
												return 0;
											end
										elseif SN > "HELFTHOSEWHOREMAINWILL" then
											if SN == "HIGHELFRDPSVONENAME" then
												return "Those Who Remain";
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									else
										return CalcStat("NCMRT",L,0.6);
									end
								elseif SN > "HIGHELFRDPSVONEWILL" then
									if SN < "HIGHELFRDTRAITNCMR" then
										if SN < "HIGHELFRDTRAITFATE" then
											if SN == "HIGHELFRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										elseif SN > "HIGHELFRDTRAITFATE" then
											if SN == "HIGHELFRDTRAITMORALE" then
												return CalcStat("HElfPeaceEldarMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("HElfFadingFirstbornFate",L);
										end
									elseif SN > "HIGHELFRDTRAITNCMR" then
										if SN < "HNTARMOURRENDBLOCK" then
											if SN == "HIGHELFRDTRAITWILL" then
												return CalcStat("HElfSorrowUndyingWill",L);
											else
												return 0;
											end
										elseif SN > "HNTARMOURRENDBLOCK" then
											if SN == "HNTARMOURRENDEVADE" then
												return -CalcStat("EvadeT",L,CalcStat("Trait23456Choice",N)*0.4);
											else
												return 0;
											end
										else
											return -CalcStat("ShieldBlock",L,CalcStat("Trait23456Choice",N)*0.4);
										end
									else
										return CalcStat("HElfPeaceEldarNCMR",L);
									end
								else
									return CalcStat("HElfThoseWhoRemainWill",L);
								end
							elseif SN > "HNTARMOURRENDPARRY" then
								if SN < "HNTSTRENGTHSTANCECRITHIT" then
									if SN < "HNTPRECISIONSTANCEFINESSE" then
										if SN < "HNTCAMPFIRENCMR" then
											if SN == "HNTBREACHFINDERRNGMIT" then
												return (-20)*L;
											else
												return 0;
											end
										elseif SN > "HNTCAMPFIRENCMR" then
											if SN == "HNTCAMPFIRENCPR" then
												return CalcStat("NCPRT",L,1.2);
											else
												return 0;
											end
										else
											return CalcStat("NCMRT",L,1.2);
										end
									elseif SN > "HNTPRECISIONSTANCEFINESSE" then
										if SN < "HNTRAPIDFIREPHYMAS" then
											if SN == "HNTPURGEPOISONRESIST" then
												return CalcStat("PoisonResistT",L,4);
											else
												return 0;
											end
										elseif SN > "HNTRAPIDFIREPHYMAS" then
											if SN == "HNTRAPIDFIRESEL" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 8 then
													return L+2;
												elseif Lm <= 10 then
													return 2*L-6;
												else
													return CalcStat("HntRapidFireSel",10);
												end
											else
												return 0;
											end
										else
											return CalcStat("PhyMasT",L,CalcStat("HntRapidFireSel",N)*0.2);
										end
									else
										return CalcStat("FinesseT",L,1.6);
									end
								elseif SN > "HNTSTRENGTHSTANCECRITHIT" then
									if SN < "HOBBITRDTRAITMIGHT" then
										if SN < "HOBBITRDPSVONENAME" then
											if SN == "HOBBITRDPSVONEMIGHT" then
												return CalcStat("HobHobbitStatureMight",L);
											else
												return 0;
											end
										elseif SN > "HOBBITRDPSVONENAME" then
											if SN == "HOBBITRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										else
											return "Hobbit-stature";
										end
									elseif SN > "HOBBITRDTRAITMIGHT" then
										if SN < "HOBBITRDTRAITVITALITY" then
											if SN == "HOBBITRDTRAITNCMR" then
												return CalcStat("HobRapidRecoveryNCMR",L);
											else
												return 0;
											end
										elseif SN > "HOBBITRDTRAITVITALITY" then
											if SN == "HOBHOBBITSTATUREMIGHT" then
												return CalcStat("MightT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("HobHobbitToughnVitality",L);
										end
									else
										return CalcStat("HobSmallSizeMight",L);
									end
								else
									return CalcStat("CritHitT",L,4);
								end
							else
								return -CalcStat("ParryT",L,CalcStat("Trait23456Choice",N)*0.4);
							end
						else
							return 1;
						end
					else
						return CalcStat("ClassBaseMightH",L);
					end
				else
					return StatLinInter("PntMPFinesseC","CreepPntS","ProgBFinesseC","",L,N,0);
				end
			elseif SN > "HOBHOBBITTOUGHNVITALITY" then
				if SN < "LOREMASTERCDMIGHTTOPARRY" then
					if SN < "INDMGPRATPCAP" then
						if SN < "HUNTERCDHASPOWER" then
							if SN < "HUNTERCDBASEAGILITY" then
								if SN < "HUNTERCDAGILITYTOPARRY" then
									if SN < "HOPEMORALEP" then
										if SN > "HOBRAPIDRECOVERYNCMR" then
											if SN == "HOBSMALLSIZEMIGHT" then
												return -CalcStat("MightT",L,0.4);
											else
												return 0;
											end
										elseif SN == "HOBRAPIDRECOVERYNCMR" then
											return CalcStat("NCMRT",L,0.6);
										else
											return 0;
										end
									elseif SN > "HOPEMORALEP" then
										if SN < "HUNTERCDAGILITYTOEVADE" then
											if SN == "HUNTERCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDAGILITYTOEVADE" then
											if SN == "HUNTERCDAGILITYTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										if Lm <= 5 then
											return DataTableValue({-0.99,-0.97,-0.95,-0.9,-0.85,-0.8,-0.65,-0.6,-0.5,-0.4,-0.3,-0.2,-0.15,-0.1,-0.05,0,0.01,0.02,0.03,0.04,0.05},L+16);
										else
											return 0.05;
										end
									end
								elseif SN > "HUNTERCDAGILITYTOPARRY" then
									if SN < "HUNTERCDARMOURTOCOMPHYMIT" then
										if SN < "HUNTERCDAGILITYTOPHYMIT" then
											if SN == "HUNTERCDAGILITYTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "HUNTERCDAGILITYTOPHYMIT" then
											if SN == "HUNTERCDAGILITYTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "HUNTERCDARMOURTOCOMPHYMIT" then
										if SN < "HUNTERCDARMOURTOTACMIT" then
											if SN == "HUNTERCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "HUNTERCDARMOURTOTACMIT" then
											if SN == "HUNTERCDARMOURTYPE" then
												return 2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "HUNTERCDBASEAGILITY" then
								if SN < "HUNTERCDBASEPOWER" then
									if SN < "HUNTERCDBASEMIGHT" then
										if SN < "HUNTERCDBASEICMR" then
											if SN == "HUNTERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASEICMR" then
											if SN == "HUNTERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRM",L);
										end
									elseif SN > "HUNTERCDBASEMIGHT" then
										if SN < "HUNTERCDBASENCMR" then
											if SN == "HUNTERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASENCMR" then
											if SN == "HUNTERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRM",L);
										end
									else
										return CalcStat("ClassBaseMightM",L);
									end
								elseif SN > "HUNTERCDBASEPOWER" then
									if SN < "HUNTERCDCALCTYPENONPHYMIT" then
										if SN < "HUNTERCDBASEWILL" then
											if SN == "HUNTERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASEWILL" then
											if SN == "HUNTERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillL",L);
										end
									elseif SN > "HUNTERCDCALCTYPENONPHYMIT" then
										if SN < "HUNTERCDFATETONCPR" then
											if SN == "HUNTERCDCALCTYPETACMIT" then
												return 26;
											else
												return 0;
											end
										elseif SN > "HUNTERCDFATETONCPR" then
											if SN == "HUNTERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 0.07;
										end
									else
										return 13;
									end
								else
									return CalcStat("ClassBasePower",L);
								end
							else
								return CalcStat("ClassBaseAgilityH",L);
							end
						elseif SN > "HUNTERCDHASPOWER" then
							if SN < "HUNTERCDWILLTOPHYMIT" then
								if SN < "HUNTERCDTACMASTOOUTHEAL" then
									if SN < "HUNTERCDMIGHTTOOUTHEAL" then
										if SN < "HUNTERCDMIGHTTOEVADE" then
											if SN == "HUNTERCDMIGHTTOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "HUNTERCDMIGHTTOEVADE" then
											if SN == "HUNTERCDMIGHTTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "HUNTERCDMIGHTTOOUTHEAL" then
										if SN < "HUNTERCDPHYMITTOCOMPHYMIT" then
											if SN == "HUNTERCDMIGHTTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "HUNTERCDPHYMITTOCOMPHYMIT" then
											if SN == "HUNTERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								elseif SN > "HUNTERCDTACMASTOOUTHEAL" then
									if SN < "HUNTERCDWILLTOCRITHIT" then
										if SN < "HUNTERCDVITALITYTOMORALE" then
											if SN == "HUNTERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										elseif SN > "HUNTERCDVITALITYTOMORALE" then
											if SN == "HUNTERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 4.5;
										end
									elseif SN > "HUNTERCDWILLTOCRITHIT" then
										if SN < "HUNTERCDWILLTOOUTHEAL" then
											if SN == "HUNTERCDWILLTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "HUNTERCDWILLTOOUTHEAL" then
											if SN == "HUNTERCDWILLTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 0.5;
									end
								else
									return 1;
								end
							elseif SN > "HUNTERCDWILLTOPHYMIT" then
								if SN < "ICPRCRAW" then
									if SN < "ICMRCRAW" then
										if SN < "ICMR" then
											if SN == "HUNTERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "ICMR" then
											if SN == "ICMRC" then
												return StatLinInter("PntMPICMRC","CreepPntS","ProgBICMRC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPICMR","ItemPntSVital","ProgBICMR","",L,N,1);
										end
									elseif SN > "ICMRCRAW" then
										if SN < "ICPR" then
											if SN == "ICMRT" then
												return StatLinInter("PntMPICMR","TraitPntSVital","ProgBICMR","",L,N,1);
											else
												return 0;
											end
										elseif SN > "ICPR" then
											if SN == "ICPRC" then
												return StatLinInter("PntMPICPRC","CreepPntS","ProgBICPRC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPICPR","ItemPntSVital","ProgBICPR","",L,N,99);
										end
									else
										return StatLinInter("PntMPICMRCOld","CreepPntS","ProgBICMRC","",L,N,99);
									end
								elseif SN > "ICPRCRAW" then
									if SN < "INDMGPRATP" then
										if SN < "INDMGPBONUS" then
											if SN == "ICPRT" then
												return StatLinInter("PntMPICPR","TraitPntSVital","ProgBICPR","",L,N,99);
											else
												return 0;
											end
										elseif SN > "INDMGPBONUS" then
											if SN == "INDMGPPRAT" then
												return CalcRatAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "INDMGPRATP" then
										if SN < "INDMGPRATPB" then
											if SN == "INDMGPRATPA" then
												return 1200;
											else
												return 0;
											end
										elseif SN > "INDMGPRATPB" then
											if SN == "INDMGPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									else
										return CalcPercAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCap",L),N);
									end
								else
									return StatLinInter("PntMPICPRCOld","CreepPntS","ProgBICPRC","",L,N,99);
								end
							else
								return 1;
							end
						else
							return 1;
						end
					elseif SN > "INDMGPRATPCAP" then
						if SN < "LMSWSTAFFBUGMORALE" then
							if SN < "ITEMPNTSCLASSIC" then
								if SN < "INHEALPRATPB" then
									if SN < "INHEALPBONUS" then
										if SN > "INDMGPRATPCAPR" then
											if SN == "INHEAL" then
												return StatLinInter("PntMPInHeal","ItemPntS","ProgBInHeal","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										elseif SN == "INDMGPRATPCAPR" then
											return CalcStat("InDmgPRatPB",L)*CalcStat("InDmgPRatPC",L);
										else
											return 0;
										end
									elseif SN > "INHEALPBONUS" then
										if SN < "INHEALPRATP" then
											if SN == "INHEALPPRAT" then
												return CalcRatAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "INHEALPRATP" then
											if SN == "INHEALPRATPA" then
												return 75;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCap",L),N);
										end
									else
										return 0;
									end
								elseif SN > "INHEALPRATPB" then
									if SN < "INHEALT" then
										if SN < "INHEALPRATPCAP" then
											if SN == "INHEALPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "INHEALPRATPCAP" then
											if SN == "INHEALPRATPCAPR" then
												return CalcStat("InHealPRatPB",L)*CalcStat("InHealPRatPC",L);
											else
												return 0;
											end
										else
											return 25;
										end
									elseif SN > "INHEALT" then
										if SN < "INSTRPOWER" then
											if SN == "INSTRMORALE" then
												if Lm <= 105 then
													return RoundDbl(4.04*L);
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("InstrMorale",105));
												end
											else
												return 0;
											end
										elseif SN > "INSTRPOWER" then
											if SN == "ITEMPNTS" then
												return {{1,25,50,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,50,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										else
											if Lm <= 9 then
												return RoundDbl(0.95*L+12.6);
											elseif Lm <= 18 then
												return RoundDbl(1.95*L+3);
											elseif Lm <= 26 then
												return RoundDbl(2.95*L-15.55);
											elseif Lm <= 35 then
												return RoundDbl(3.95*L-42.15);
											elseif Lm <= 42 then
												return RoundDbl(4.95*L-77.7);
											elseif Lm <= 45 then
												return RoundDbl(5.95*L-120.35);
											elseif Lm <= 105 then
												return RoundDbl(6*L-124);
											else
												return CalcStat("ProgExtMpExpRnd",L,CalcStat("InstrPower",105));
											end
										end
									else
										return StatLinInter("PntMPInHeal","TraitPntS","ProgBInHeal","AdjUmbarTrait",L,N,0);
									end
								else
									return CalcStat("BRatRounded",L,"BRatStandard");
								end
							elseif SN > "ITEMPNTSCLASSIC" then
								if SN < "LIGHTMIT" then
									if SN < "L" then
										if SN < "ITEMPNTSVIRTUEMORALE" then
											if SN == "ITEMPNTSVIRTUEMASTERY" then
												return {{0,1,25,50,79,80,200,225,300,349,399,400,449,450,499,500,549,550,599},{0,1,25,50,75,76,100,105,106,115,116,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										elseif SN > "ITEMPNTSVIRTUEMORALE" then
											if SN == "ITEMPNTSVITAL" then
												return {{1,25,50,60,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,50,60,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										else
											return {{0,1,2,50,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{0,1,2,50,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
										end
									elseif SN > "L" then
										if SN < "LI2ILVLCAP" then
											if SN == "LEVELCAP" then
												return 150;
											else
												return 0;
											end
										elseif SN > "LI2ILVLCAP" then
											if SN == "LI2REFORGEILVL" then
												if Lm <= 145 then
													return CalcStat("AwardILvlI",L);
												else
													return N;
												end
											else
												return 0;
											end
										else
											return 520;
										end
									else
										return L;
									end
								elseif SN > "LIGHTMIT" then
									if SN < "LMANCIENTWISDOMWILL" then
										if SN < "LIGHTNINGMIT" then
											if SN == "LIGHTMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										elseif SN > "LIGHTNINGMIT" then
											if SN == "LIGHTNINGMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("DmgTypeMit",L,N);
										end
									elseif SN > "LMANCIENTWISDOMWILL" then
										if SN < "LMHEARTYDIETMORALECHOICE" then
											if SN == "LMHEARTYDIETMORALE" then
												return CalcStat("Morale",L,CalcStat("LmHeartyDietMoraleChoice",N));
											else
												return 0;
											end
										elseif SN > "LMHEARTYDIETMORALECHOICE" then
											if SN == "LMPREPFORWARTACMAS" then
												return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0.4,1,1.6,2.4,3.2},L);
											end
										end
									else
										if Lm <= 105 then
											return RoundDbl(1.1*L);
										else
											return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmAncientWisdomWill",105));
										end
									end
								else
									return CalcStat("DmgTypeMit",L,N);
								end
							else
								return {{1,25,50,60,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,50,60,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
							end
						elseif SN > "LMSWSTAFFBUGMORALE" then
							if SN < "LOREMASTERCDBASEICPR" then
								if SN < "LOREMASTERCDAGILITYTOTACMAS" then
									if SN < "LOEPASSIVE" then
										if SN < "LMSWSTAFFBUGPARRYOLD" then
											if SN == "LMSWSTAFFBUGPARRY" then
												if Lm <= 140 then
													return CalcStat("U371LegacyStatFix",L,"LmSwStaffBugParryOld");
												else
													return CalcStat("LMSwStaffBugParry",140);
												end
											else
												return 0;
											end
										elseif SN > "LMSWSTAFFBUGPARRYOLD" then
											if SN == "LOE" then
												return 2*N;
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return RoundDbl(44.44*L);
											else
												return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("LmSwStaffBugParryOld",105));
											end
										end
									elseif SN > "LOEPASSIVE" then
										if SN < "LOREMASTERCDAGILITYTOEVADE" then
											if SN == "LOREMASTERCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDAGILITYTOEVADE" then
											if SN == "LOREMASTERCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										if Lm <= 115 then
											return 0;
										elseif Lm <= 119 then
											return 20*L-2300;
										else
											return CalcStat("LoEPassive",119);
										end
									end
								elseif SN > "LOREMASTERCDAGILITYTOTACMAS" then
									if SN < "LOREMASTERCDARMOURTYPE" then
										if SN < "LOREMASTERCDARMOURTONONPHYMIT" then
											if SN == "LOREMASTERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDARMOURTONONPHYMIT" then
											if SN == "LOREMASTERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "LOREMASTERCDARMOURTYPE" then
										if SN < "LOREMASTERCDBASEFATE" then
											if SN == "LOREMASTERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityL",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEFATE" then
											if SN == "LOREMASTERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									else
										return 1;
									end
								else
									return 2;
								end
							elseif SN > "LOREMASTERCDBASEICPR" then
								if SN < "LOREMASTERCDCALCTYPECOMPHYMIT" then
									if SN < "LOREMASTERCDBASENCPR" then
										if SN < "LOREMASTERCDBASEMORALE" then
											if SN == "LOREMASTERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEMORALE" then
											if SN == "LOREMASTERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									elseif SN > "LOREMASTERCDBASENCPR" then
										if SN < "LOREMASTERCDBASEVITALITY" then
											if SN == "LOREMASTERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEVITALITY" then
											if SN == "LOREMASTERCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									else
										return CalcStat("ClassBaseNCPR",L);
									end
								elseif SN > "LOREMASTERCDCALCTYPECOMPHYMIT" then
									if SN < "LOREMASTERCDFATETOPOWER" then
										if SN < "LOREMASTERCDCALCTYPETACMIT" then
											if SN == "LOREMASTERCDCALCTYPENONPHYMIT" then
												return 12;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDCALCTYPETACMIT" then
											if SN == "LOREMASTERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											return 25;
										end
									elseif SN > "LOREMASTERCDFATETOPOWER" then
										if SN < "LOREMASTERCDMIGHTTOCRITHIT" then
											if SN == "LOREMASTERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDMIGHTTOCRITHIT" then
											if SN == "LOREMASTERCDMIGHTTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1;
									end
								else
									return 12;
								end
							else
								return CalcStat("ClassBaseICPR",L);
							end
						else
							if Lm <= 105 then
								return RoundDbl(LinFmod(1,16.2,696.9,1,105,L));
							else
								return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmSwStaffBugMorale",105));
							end
						end
					else
						return 400;
					end
				elseif SN > "LOREMASTERCDMIGHTTOPARRY" then
					if SN < "MARINERCDFATETONCPR" then
						if SN < "MANRDPSVTWOEVADE" then
							if SN < "LVLBONUSPHYDMG" then
								if SN < "LOREMASTERCDVITALITYTONCMR" then
									if SN < "LOREMASTERCDPHYMITTONONPHYMIT" then
										if SN > "LOREMASTERCDMIGHTTOTACMAS" then
											if SN == "LOREMASTERCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDMIGHTTOTACMAS" then
											return 2;
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDPHYMITTONONPHYMIT" then
										if SN < "LOREMASTERCDVITALITYTOICMR" then
											if SN == "LOREMASTERCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDVITALITYTOICMR" then
											if SN == "LOREMASTERCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 0.012;
										end
									else
										return 1;
									end
								elseif SN > "LOREMASTERCDVITALITYTONCMR" then
									if SN < "LOREMASTERCDWILLTORESIST" then
										if SN < "LOREMASTERCDWILLTOEVADE" then
											if SN == "LOREMASTERCDWILLTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDWILLTOEVADE" then
											if SN == "LOREMASTERCDWILLTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "LOREMASTERCDWILLTORESIST" then
										if SN < "LOREMASTERCDWILLTOTACMIT" then
											if SN == "LOREMASTERCDWILLTOTACMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDWILLTOTACMIT" then
											if SN == "LVLBONUSMORRES" then
												return 0.1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 0.12;
								end
							elseif SN > "LVLBONUSPHYDMG" then
								if SN < "MAINT" then
									if SN < "LVLEXPCOSTTOT" then
										if SN < "LVLBONUSTACDMG" then
											if SN == "LVLBONUSPOWRES" then
												return CalcStat("SkillPowerCost",L,N);
											else
												return 0;
											end
										elseif SN > "LVLBONUSTACDMG" then
											if SN == "LVLEXPCOST" then
												if Lm <= 1 then
													return 0;
												elseif Lm <= 5 then
													return RoundDbl(12.5*L*L+12.5666666666667*L+24.8666666666667);
												elseif Lm <= 10 then
													return RoundDbl(33.8*L*L-179.48*L+452.6);
												elseif Lm <= 15 then
													return RoundDbl(55.05*L*L-583.77*L+2370.5);
												elseif Lm <= 20 then
													return RoundDbl(76.2*L*L-1196.96*L+6809);
												elseif Lm <= 25 then
													return RoundDbl(97.4*L*L-2023*L+14849.8);
												elseif Lm <= 30 then
													return RoundDbl(118.7*L*L-3066.02 *L+27612.8);
												elseif Lm <= 35 then
													return RoundDbl(139.95*L*L-4319.23*L+46084.1);
												elseif Lm <= 40 then
													return RoundDbl(161.2*L*L-5785.04*L+71356.2);
												elseif Lm <= 45 then
													return RoundDbl(182.5*L*L-7467.38*L+104569.8);
												elseif Lm <= 50 then
													return RoundDbl(203.8*L*L-9363.48*L+146761.8);
												elseif Lm <= 55 then
													return RoundDbl(225.05*L*L-11467.77*L+198851.3);
												elseif Lm <= 60 then
													return RoundDbl(246.3*L*L-13784.46*L+261988);
												elseif Lm <= 70 then
													return RoundDbl(ExpFmod(CalcStat("LvlExpCost",60),61,5.071,L,nil,3.485));
												elseif Lm <= 75 then
													return RoundDbl(ExpFmod(CalcStat("LvlExpCost",70),71,5.072,L,nil,-0.95));
												else
													return ExpFmod(CalcStat("LvlExpCost",75),76,5,L,0,-0.5);
												end
											else
												return 0;
											end
										else
											return 0.1;
										end
									elseif SN > "LVLEXPCOSTTOT" then
										if SN < "MAIN" then
											if SN == "LVLTOILVL" then
												if Lm <= 106 then
													return LinFmod(1,225,300,105,106,L);
												elseif Lm <= 115 then
													return LinFmod(1,300,349,106,115,L);
												elseif Lm <= 116 then
													return LinFmod(1,349,350,115,116,L);
												elseif Lm <= 120 then
													return LinFmod(1,350,399,116,120,L);
												elseif Lm <= 121 then
													return LinFmod(1,399,400,120,121,L);
												elseif Lm <= 130 then
													return LinFmod(1,400,449,121,130,L);
												elseif Lm <= 131 then
													return LinFmod(1,449,450,130,131,L);
												elseif Lm <= 140 then
													return LinFmod(1,450,499,131,140,L);
												elseif Lm <= 141 then
													return LinFmod(1,499,500,140,141,L);
												else
													return LinFmod(1,500,549,141,150,L);
												end
											else
												return 0;
											end
										elseif SN > "MAIN" then
											if SN == "MAINC" then
												return RoundDblDown(StatLinInter("PntMPMainC","CreepPntS","ProgBMainC","",L,N,0));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPMain","ItemPntS","ProgBMain","AdjUmbarItem",L,N,0));
										end
									else
										if Lm <= 1 then
											return 0;
										else
											return CalcStat("LvlExpCostTot",L-1)+CalcStat("LvlExpCost",L);
										end
									end
								elseif SN > "MAINT" then
									if SN < "MANGIFTOFMENFATE" then
										if SN < "MANEASILYINSPINHEALP" then
											if SN == "MANDIMMANKINDWILL" then
												return -CalcStat("WillT",L,0.4);
											else
												return 0;
											end
										elseif SN > "MANEASILYINSPINHEALP" then
											if SN == "MANFOURTHAGEWILL" then
												return CalcStat("WillT",L,1.0);
											else
												return 0;
											end
										else
											return 5;
										end
									elseif SN > "MANGIFTOFMENFATE" then
										if SN < "MANRDPSVONEWILL" then
											if SN == "MANRDPSVONENAME" then
												return "Man of the Fourth Age";
											else
												return 0;
											end
										elseif SN > "MANRDPSVONEWILL" then
											if SN == "MANRDPSVTWOBLOCK" then
												return CalcStat("BalanceOfManBlock",L);
											else
												return 0;
											end
										else
											return CalcStat("ManFourthAgeWill",L);
										end
									else
										return CalcStat("FateT",L,1.0);
									end
								else
									return RoundDblDown(StatLinInter("PntMPMain","TraitPntS","ProgBMain","AdjUmbarTrait",L,N,0));
								end
							else
								return 0.1;
							end
						elseif SN > "MANRDPSVTWOEVADE" then
							if SN < "MARINERCDARMOURTOTACMIT" then
								if SN < "MARINERCDAGILITYTOCRITHIT" then
									if SN < "MANRDTRAITINHEALP" then
										if SN < "MANRDPSVTWOPARRY" then
											if SN == "MANRDPSVTWONAME" then
												return "Balance of Man";
											else
												return 0;
											end
										elseif SN > "MANRDPSVTWOPARRY" then
											if SN == "MANRDTRAITFATE" then
												return CalcStat("ManGiftOfMenFate",L);
											else
												return 0;
											end
										else
											return CalcStat("BalanceOfManParry",L);
										end
									elseif SN > "MANRDTRAITINHEALP" then
										if SN < "MANRDTRAITWILL" then
											if SN == "MANRDTRAITMIGHT" then
												return CalcStat("ManStrongMenMight",L);
											else
												return 0;
											end
										elseif SN > "MANRDTRAITWILL" then
											if SN == "MANSTRONGMENMIGHT" then
												return CalcStat("MightT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("ManDimMankindWill",L);
										end
									else
										return CalcStat("ManEasilyInspInHealP",L);
									end
								elseif SN > "MARINERCDAGILITYTOCRITHIT" then
									if SN < "MARINERCDAGILITYTOPHYMIT" then
										if SN < "MARINERCDAGILITYTOPARRY" then
											if SN == "MARINERCDAGILITYTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										elseif SN > "MARINERCDAGILITYTOPARRY" then
											if SN == "MARINERCDAGILITYTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "MARINERCDAGILITYTOPHYMIT" then
										if SN < "MARINERCDARMOURTOCOMPHYMIT" then
											if SN == "MARINERCDAGILITYTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDARMOURTOCOMPHYMIT" then
											if SN == "MARINERCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "MARINERCDARMOURTOTACMIT" then
								if SN < "MARINERCDBASENCMR" then
									if SN < "MARINERCDBASEICMR" then
										if SN < "MARINERCDBASEAGILITY" then
											if SN == "MARINERCDARMOURTYPE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEAGILITY" then
											if SN == "MARINERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityM",L);
										end
									elseif SN > "MARINERCDBASEICMR" then
										if SN < "MARINERCDBASEMIGHT" then
											if SN == "MARINERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEMIGHT" then
											if SN == "MARINERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMightM",L);
										end
									else
										return CalcStat("ClassBaseICMRL",L);
									end
								elseif SN > "MARINERCDBASENCMR" then
									if SN < "MARINERCDBASEWILL" then
										if SN < "MARINERCDBASEPOWER" then
											if SN == "MARINERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEPOWER" then
											if SN == "MARINERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBasePower",L);
										end
									elseif SN > "MARINERCDBASEWILL" then
										if SN < "MARINERCDCALCTYPENONPHYMIT" then
											if SN == "MARINERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "MARINERCDCALCTYPENONPHYMIT" then
											if SN == "MARINERCDCALCTYPETACMIT" then
												return 26;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ClassBaseWillM",L);
									end
								else
									return CalcStat("ClassBaseNCMRL",L);
								end
							else
								return 0.2;
							end
						else
							return CalcStat("BalanceOfManEvade",L);
						end
					elseif SN > "MARINERCDFATETONCPR" then
						if SN < "MINCOURAGERESIST" then
							if SN < "MARINERCDWILLTOOUTHEAL" then
								if SN < "MARINERCDPHYMITTOCOMPHYMIT" then
									if SN < "MARINERCDMIGHTTOFINESSE" then
										if SN < "MARINERCDHASPOWER" then
											if SN == "MARINERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDHASPOWER" then
											if SN == "MARINERCDMIGHTTOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MARINERCDMIGHTTOFINESSE" then
										if SN < "MARINERCDMIGHTTOPARRY" then
											if SN == "MARINERCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDMIGHTTOPARRY" then
											if SN == "MARINERCDMIGHTTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1.5;
									end
								elseif SN > "MARINERCDPHYMITTOCOMPHYMIT" then
									if SN < "MARINERCDVITALITYTOMORALE" then
										if SN < "MARINERCDTACMASTOOUTHEAL" then
											if SN == "MARINERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDTACMASTOOUTHEAL" then
											if SN == "MARINERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MARINERCDVITALITYTOMORALE" then
										if SN < "MARINERCDWILLTOCRITHIT" then
											if SN == "MARINERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "MARINERCDWILLTOCRITHIT" then
											if SN == "MARINERCDWILLTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 4.5;
									end
								else
									return 1;
								end
							elseif SN > "MARINERCDWILLTOOUTHEAL" then
								if SN < "MASTERYT" then
									if SN < "MASTERY" then
										if SN < "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MASTERY" then
										if SN < "MASTERYC" then
											if SN == "MASTERYADJ" then
												if Lm <= 450 then
													return 1;
												elseif Lm <= 499 then
													return 0.9;
												elseif Lm <= 550 then
													return 741/900;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "MASTERYC" then
											if SN == "MASTERYOLD" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPMasteryC","CreepPntS","ProgBMasteryC","",L,N,0);
										end
									else
										return StatLinInter("PntMPMastery","ItemPntS","ProgBMastery","MasteryAdj",L,N,0);
									end
								elseif SN > "MASTERYT" then
									if SN < "MIGHTC" then
										if SN < "MATHOMLVLTOILVL" then
											if SN == "MASTERYTADJ" then
												if Lm <= 140 then
													return 1;
												elseif Lm <= 141 then
													return 0.9;
												else
													return 247/300;
												end
											else
												return 0;
											end
										elseif SN > "MATHOMLVLTOILVL" then
											if SN == "MIGHT" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										else
											return CalcStat("AwardLvlToILvl",L);
										end
									elseif SN > "MIGHTC" then
										if SN < "MINCOMPOSURERESIST" then
											if SN == "MIGHTT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										elseif SN > "MINCOMPOSURERESIST" then
											if SN == "MINCOMPOSURETACMIT" then
												return CalcStat("TacMitT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("ResistT",L,1.6);
										end
									else
										return CalcStat("MainC",L,N);
									end
								else
									return StatLinInter("PntMPMastery","TraitPntS","ProgBMastery","MasteryTAdj",L,N,0);
								end
							else
								return 2;
							end
						elseif SN > "MINCOURAGERESIST" then
							if SN < "MINSTRELCDBASEICPR" then
								if SN < "MINSTRELCDAGILITYTOTACMAS" then
									if SN < "MINPIERCINGBALFINESSE" then
										if SN < "MINECHOESBATTLERESIST" then
											if SN == "MINECHOESBATTLECRITDEF" then
												return -CalcStat("CritDefT",L,2.0);
											else
												return 0;
											end
										elseif SN > "MINECHOESBATTLERESIST" then
											if SN == "MINENDMORALE" then
												return CalcStat("MoraleT",L,CalcStat("Trait12345Choice",N)*0.8);
											else
												return 0;
											end
										else
											return -CalcStat("SongResistT",L,1.0);
										end
									elseif SN > "MINPIERCINGBALFINESSE" then
										if SN < "MINSTRELCDAGILITYTOEVADE" then
											if SN == "MINSTRELCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDAGILITYTOEVADE" then
											if SN == "MINSTRELCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
									end
								elseif SN > "MINSTRELCDAGILITYTOTACMAS" then
									if SN < "MINSTRELCDARMOURTYPE" then
										if SN < "MINSTRELCDARMOURTONONPHYMIT" then
											if SN == "MINSTRELCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDARMOURTONONPHYMIT" then
											if SN == "MINSTRELCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "MINSTRELCDARMOURTYPE" then
										if SN < "MINSTRELCDBASEFATE" then
											if SN == "MINSTRELCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEFATE" then
											if SN == "MINSTRELCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									else
										return 1;
									end
								else
									return 2;
								end
							elseif SN > "MINSTRELCDBASEICPR" then
								if SN < "MINSTRELCDCALCTYPECOMPHYMIT" then
									if SN < "MINSTRELCDBASENCPR" then
										if SN < "MINSTRELCDBASEMORALE" then
											if SN == "MINSTRELCDBASEMIGHT" then
												return CalcStat("ClassBaseMightL",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEMORALE" then
											if SN == "MINSTRELCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									elseif SN > "MINSTRELCDBASENCPR" then
										if SN < "MINSTRELCDBASEVITALITY" then
											if SN == "MINSTRELCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEVITALITY" then
											if SN == "MINSTRELCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									else
										return CalcStat("ClassBaseNCPR",L);
									end
								elseif SN > "MINSTRELCDCALCTYPECOMPHYMIT" then
									if SN < "MINSTRELCDFATETONCPR" then
										if SN < "MINSTRELCDCALCTYPETACMIT" then
											if SN == "MINSTRELCDCALCTYPENONPHYMIT" then
												return 12;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDCALCTYPETACMIT" then
											if SN == "MINSTRELCDCANBLOCK" then
												if Lm <= 19 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return 25;
										end
									elseif SN > "MINSTRELCDFATETONCPR" then
										if SN < "MINSTRELCDHASPOWER" then
											if SN == "MINSTRELCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDHASPOWER" then
											if SN == "MINSTRELCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.07;
									end
								else
									return 12;
								end
							else
								return CalcStat("ClassBaseICPR",L);
							end
						else
							return CalcStat("FearResistT",L,1.0);
						end
					else
						return 0.07;
					end
				else
					return 1;
				end
			else
				return CalcStat("VitalityT",L,1.0);
			end
		else
			return CalcStat("CreepAudacityCostBase",L)*25;
		end
	elseif SN > "MINSTRELCDMIGHTTOOUTHEAL" then
		if SN < "TACMITLPRATPCAP" then
			if SN < "PNTMPICMRCOLD" then
				if SN < "PARTEVADEMITPRATPCAP" then
					if SN < "NCPRC" then
						if SN < "MITHEAVYPRATPA" then
							if SN < "MINSTRELCDWILLTOTACMIT" then
								if SN < "MINSTRELCDVITALITYTOMORALE" then
									if SN < "MINSTRELCDPHYMITTOCOMPHYMIT" then
										if SN > "MINSTRELCDMIGHTTOTACMAS" then
											if SN == "MINSTRELCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "MINSTRELCDMIGHTTOTACMAS" then
											return 2;
										else
											return 0;
										end
									elseif SN > "MINSTRELCDPHYMITTOCOMPHYMIT" then
										if SN < "MINSTRELCDTACMASTOOUTHEAL" then
											if SN == "MINSTRELCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDTACMASTOOUTHEAL" then
											if SN == "MINSTRELCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "MINSTRELCDVITALITYTOMORALE" then
									if SN < "MINSTRELCDWILLTOEVADE" then
										if SN < "MINSTRELCDWILLTOBLOCK" then
											if SN == "MINSTRELCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDWILLTOBLOCK" then
											if SN == "MINSTRELCDWILLTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MINSTRELCDWILLTOEVADE" then
										if SN < "MINSTRELCDWILLTORESIST" then
											if SN == "MINSTRELCDWILLTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDWILLTORESIST" then
											if SN == "MINSTRELCDWILLTOTACMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 4.5;
								end
							elseif SN > "MINSTRELCDWILLTOTACMIT" then
								if SN < "MINTOTFINESSESEL" then
									if SN < "MINTOTCRITHITSEL" then
										if SN < "MINTIMEECHOESBATTLERESIST" then
											if SN == "MINTACMAS" then
												return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "MINTIMEECHOESBATTLERESIST" then
											if SN == "MINTOTCRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("MinToTCritHitSel",N));
											else
												return 0;
											end
										else
											return -CalcStat("SongResistT",L,0.6);
										end
									elseif SN > "MINTOTCRITHITSEL" then
										if SN < "MINTOTFATESEL" then
											if SN == "MINTOTFATE" then
												return CalcStat("FateT",L,CalcStat("MinToTFateSel",N));
											else
												return 0;
											end
										elseif SN > "MINTOTFATESEL" then
											if SN == "MINTOTFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("MinToTFinesseSel",N));
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0.2,0.3,0.4,0.5,0.6,0},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({0,0,0,0.4,0.6,0},L);
										end
									end
								elseif SN > "MINTOTFINESSESEL" then
									if SN < "MINTOTVITALITYSEL" then
										if SN < "MINTOTRESISTSEL" then
											if SN == "MINTOTRESIST" then
												return CalcStat("ResistT",L,CalcStat("MinToTResistSel",N));
											else
												return 0;
											end
										elseif SN > "MINTOTRESISTSEL" then
											if SN == "MINTOTVITALITY" then
												return CalcStat("VitalityT",L,CalcStat("MinToTVitalitySel",N));
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({0,0.2,0.3,0.4,0.5,0},L);
											end
										end
									elseif SN > "MINTOTVITALITYSEL" then
										if SN < "MITHEAVYPPRAT" then
											if SN == "MITHEAVYPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "MITHEAVYPPRAT" then
											if SN == "MITHEAVYPRATP" then
												return CalcPercAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCapR",L),N);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({0,0,0,0,0.4,0},L);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({0,0,0.2,0.4,0.6,0},L);
									end
								end
							else
								return 1;
							end
						elseif SN > "MITHEAVYPRATPA" then
							if SN < "MITMEDIUMPRATP" then
								if SN < "MITLIGHTPRATP" then
									if SN < "MITHEAVYPRATPCAPR" then
										if SN < "MITHEAVYPRATPC" then
											if SN == "MITHEAVYPRATPB" then
												return CalcStat("BRatRounded",L,"BRatMitHeavy");
											else
												return 0;
											end
										elseif SN > "MITHEAVYPRATPC" then
											if SN == "MITHEAVYPRATPCAP" then
												return 60;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "MITHEAVYPRATPCAPR" then
										if SN < "MITLIGHTPBONUS" then
											if SN == "MITIGATION" then
												return StatLinInter("PntMPMitigation","ItemPntS","ProgBMitigation","AdjUmbarItemMit",L,N,0);
											else
												return 0;
											end
										elseif SN > "MITLIGHTPBONUS" then
											if SN == "MITLIGHTPPRAT" then
												return CalcRatAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("MitHeavyPRatPB",L)*CalcStat("MitHeavyPRatPC",L);
									end
								elseif SN > "MITLIGHTPRATP" then
									if SN < "MITLIGHTPRATPCAP" then
										if SN < "MITLIGHTPRATPB" then
											if SN == "MITLIGHTPRATPA" then
												return 120;
											else
												return 0;
											end
										elseif SN > "MITLIGHTPRATPB" then
											if SN == "MITLIGHTPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatMitLight");
										end
									elseif SN > "MITLIGHTPRATPCAP" then
										if SN < "MITMEDIUMPBONUS" then
											if SN == "MITLIGHTPRATPCAPR" then
												return CalcStat("MitLightPRatPB",L)*CalcStat("MitLightPRatPC",L);
											else
												return 0;
											end
										elseif SN > "MITMEDIUMPBONUS" then
											if SN == "MITMEDIUMPPRAT" then
												return CalcRatAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return 40;
									end
								else
									return CalcPercAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCap",L),N);
								end
							elseif SN > "MITMEDIUMPRATP" then
								if SN < "MORALET" then
									if SN < "MITMEDIUMPRATPCAP" then
										if SN < "MITMEDIUMPRATPB" then
											if SN == "MITMEDIUMPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN > "MITMEDIUMPRATPB" then
											if SN == "MITMEDIUMPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatMitMedium");
										end
									elseif SN > "MITMEDIUMPRATPCAP" then
										if SN < "MORALE" then
											if SN == "MITMEDIUMPRATPCAPR" then
												return CalcStat("MitMediumPRatPB",L)*CalcStat("MitMediumPRatPC",L);
											else
												return 0;
											end
										elseif SN > "MORALE" then
											if SN == "MORALEADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 79 then
													return 0.8;
												elseif Lm <= 80 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPMorale","ItemPntSVital","ProgBMorale","MoraleAdj",L,N,0);
										end
									else
										return 50;
									end
								elseif SN > "MORALET" then
									if SN < "NCMRC" then
										if SN < "N" then
											if SN == "MORALETADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 65 then
													return 0.8;
												elseif Lm <= 75 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "N" then
											if SN == "NCMR" then
												return StatLinInter("PntMPNCMR","ItemPntSVital","ProgBNCMR","",L,N,1);
											else
												return 0;
											end
										else
											return N;
										end
									elseif SN > "NCMRC" then
										if SN < "NCMRT" then
											if SN == "NCMRCRAW" then
												return StatLinInter("PntMPNCMRCOld","CreepPntS","ProgBNCMRC","",L,N,99);
											else
												return 0;
											end
										elseif SN > "NCMRT" then
											if SN == "NCPR" then
												return StatLinInter("PntMPNCPR","ItemPntSClassic","ProgBNCPR","",L,N,99);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPNCMR","TraitPntSVital","ProgBNCMR","",L,N,1);
										end
									else
										return StatLinInter("PntMPNCMRC","CreepPntS","ProgBNCMRC","",L,N,0);
									end
								else
									return StatLinInter("PntMPMorale","TraitPntSVital","ProgBMorale","MoraleTAdj",L,N,0);
								end
							else
								return CalcPercAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCap",L),N);
							end
						else
							return 180;
						end
					elseif SN > "NCPRC" then
						if SN < "PARRYPRATPCAPR" then
							if SN < "OUTHEALPRATP" then
								if SN < "OUTDMGPRATPA" then
									if SN < "OFFSET" then
										if SN > "NCPRCRAW" then
											if SN == "NCPRT" then
												return StatLinInter("PntMPNCPR","TraitPntSVital","ProgBNCPR","",L,N,99);
											else
												return 0;
											end
										elseif SN == "NCPRCRAW" then
											return StatLinInter("PntMPNCPRCOld","CreepPntS","ProgBNCPRC","",L,N,99);
										else
											return 0;
										end
									elseif SN > "OFFSET" then
										if SN < "OUTDMGPPRAT" then
											if SN == "OUTDMGPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "OUTDMGPPRAT" then
											if SN == "OUTDMGPRATP" then
												return CalcPercAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCapR",L),N);
										end
									else
										return L+N;
									end
								elseif SN > "OUTDMGPRATPA" then
									if SN < "OUTDMGPRATPCAPR" then
										if SN < "OUTDMGPRATPC" then
											if SN == "OUTDMGPRATPB" then
												return CalcStat("BRatRounded",L,"BRatExtra");
											else
												return 0;
											end
										elseif SN > "OUTDMGPRATPC" then
											if SN == "OUTDMGPRATPCAP" then
												return 200;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "OUTDMGPRATPCAPR" then
										if SN < "OUTHEALPBONUS" then
											if SN == "OUTHEAL" then
												return StatLinInter("PntMPOutHeal","ItemPntS","ProgBOutHeal","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										elseif SN > "OUTHEALPBONUS" then
											if SN == "OUTHEALPPRAT" then
												return CalcRatAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("OutDmgPRatPB",L)*CalcStat("OutDmgPRatPC",L);
									end
								else
									return 600;
								end
							elseif SN > "OUTHEALPRATP" then
								if SN < "PARRYC" then
									if SN < "OUTHEALPRATPCAP" then
										if SN < "OUTHEALPRATPB" then
											if SN == "OUTHEALPRATPA" then
												return 210;
											else
												return 0;
											end
										elseif SN > "OUTHEALPRATPB" then
											if SN == "OUTHEALPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatOutHeal");
										end
									elseif SN > "OUTHEALPRATPCAP" then
										if SN < "OUTHEALT" then
											if SN == "OUTHEALPRATPCAPR" then
												return CalcStat("OutHealPRatPB",L)*CalcStat("OutHealPRatPC",L);
											else
												return 0;
											end
										elseif SN > "OUTHEALT" then
											if SN == "PARRY" then
												return CalcStat("BPE",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPOutHeal","TraitPntS","ProgBOutHeal","AdjUmbarTrait",L,N,0);
										end
									else
										return 70;
									end
								elseif SN > "PARRYC" then
									if SN < "PARRYPRATPA" then
										if SN < "PARRYPPRAT" then
											if SN == "PARRYPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										elseif SN > "PARRYPPRAT" then
											if SN == "PARRYPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPPRat",L,N);
										end
									elseif SN > "PARRYPRATPA" then
										if SN < "PARRYPRATPC" then
											if SN == "PARRYPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PARRYPRATPC" then
											if SN == "PARRYPRATPCAP" then
												return CalcStat("BPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPC",L);
										end
									else
										return CalcStat("BPEPRatPA",L);
									end
								else
									return CalcStat("BPEC",L,N);
								end
							else
								return CalcPercAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCap",L),N);
							end
						elseif SN > "PARRYPRATPCAPR" then
							if SN < "PARTBLOCKPRATPCAP" then
								if SN < "PARTBLOCKMITPRATPCAP" then
									if SN < "PARTBLOCKMITPRATP" then
										if SN < "PARTBLOCKMITPBONUS" then
											if SN == "PARRYT" then
												return CalcStat("BPET",L,N);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKMITPBONUS" then
											if SN == "PARTBLOCKMITPPRAT" then
												return CalcStat("PartMitPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPBonus",L);
										end
									elseif SN > "PARTBLOCKMITPRATP" then
										if SN < "PARTBLOCKMITPRATPB" then
											if SN == "PARTBLOCKMITPRATPA" then
												return CalcStat("PartMitPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKMITPRATPB" then
											if SN == "PARTBLOCKMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPB",L);
										end
									else
										return CalcStat("PartMitPRatP",L,N);
									end
								elseif SN > "PARTBLOCKMITPRATPCAP" then
									if SN < "PARTBLOCKPRATP" then
										if SN < "PARTBLOCKPBONUS" then
											if SN == "PARTBLOCKMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPBONUS" then
											if SN == "PARTBLOCKPPRAT" then
												return CalcStat("PartBPEPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPBonus",L);
										end
									elseif SN > "PARTBLOCKPRATP" then
										if SN < "PARTBLOCKPRATPB" then
											if SN == "PARTBLOCKPRATPA" then
												return CalcStat("PartBPEPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPRATPB" then
											if SN == "PARTBLOCKPRATPC" then
												return CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPB",L);
										end
									else
										return CalcStat("PartBPEPRatP",L,N);
									end
								else
									return CalcStat("PartMitPRatPCap",L);
								end
							elseif SN > "PARTBLOCKPRATPCAP" then
								if SN < "PARTBPEPRATPCAP" then
									if SN < "PARTBPEPRATP" then
										if SN < "PARTBPEPBONUS" then
											if SN == "PARTBLOCKPRATPCAPR" then
												return CalcStat("PartBPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTBPEPBONUS" then
											if SN == "PARTBPEPPRAT" then
												return CalcRatAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "PARTBPEPRATP" then
										if SN < "PARTBPEPRATPB" then
											if SN == "PARTBPEPRATPA" then
												return 75;
											else
												return 0;
											end
										elseif SN > "PARTBPEPRATPB" then
											if SN == "PARTBPEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatPartBPE");
										end
									else
										return CalcPercAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCap",L),N);
									end
								elseif SN > "PARTBPEPRATPCAP" then
									if SN < "PARTEVADEMITPRATP" then
										if SN < "PARTEVADEMITPBONUS" then
											if SN == "PARTBPEPRATPCAPR" then
												return CalcStat("PartBPEPRatPB",L)*CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEMITPBONUS" then
											if SN == "PARTEVADEMITPPRAT" then
												return CalcStat("PartMitPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPBonus",L);
										end
									elseif SN > "PARTEVADEMITPRATP" then
										if SN < "PARTEVADEMITPRATPB" then
											if SN == "PARTEVADEMITPRATPA" then
												return CalcStat("PartMitPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEMITPRATPB" then
											if SN == "PARTEVADEMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPB",L);
										end
									else
										return CalcStat("PartMitPRatP",L,N);
									end
								else
									return 25;
								end
							else
								return CalcStat("PartBPEPRatPCap",L);
							end
						else
							return CalcStat("BPEPRatPCapR",L);
						end
					else
						return StatLinInter("PntMPNCPRC","CreepPntS","ProgBNCPRC","",L,N,0);
					end
				elseif SN > "PARTEVADEMITPRATPCAP" then
					if SN < "PHYDMGPRATPCAPR" then
						if SN < "PARTMITPRATPC" then
							if SN < "PARTFINESSEDMGPRATPC" then
								if SN < "PARTEVADEPRATPC" then
									if SN < "PARTEVADEPPRAT" then
										if SN > "PARTEVADEMITPRATPCAPR" then
											if SN == "PARTEVADEPBONUS" then
												return CalcStat("PartBPEPBonus",L);
											else
												return 0;
											end
										elseif SN == "PARTEVADEMITPRATPCAPR" then
											return CalcStat("PartMitPRatPCapR",L);
										else
											return 0;
										end
									elseif SN > "PARTEVADEPPRAT" then
										if SN < "PARTEVADEPRATPA" then
											if SN == "PARTEVADEPRATP" then
												return CalcStat("PartBPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PARTEVADEPRATPA" then
											if SN == "PARTEVADEPRATPB" then
												return CalcStat("PartBPEPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPA",L);
										end
									else
										return CalcStat("PartBPEPPRat",L,N);
									end
								elseif SN > "PARTEVADEPRATPC" then
									if SN < "PARTFINESSEDMGPPRAT" then
										if SN < "PARTEVADEPRATPCAPR" then
											if SN == "PARTEVADEPRATPCAP" then
												return CalcStat("PartBPEPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEPRATPCAPR" then
											if SN == "PARTFINESSEDMGPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPCapR",L);
										end
									elseif SN > "PARTFINESSEDMGPPRAT" then
										if SN < "PARTFINESSEDMGPRATPA" then
											if SN == "PARTFINESSEDMGPRATP" then
												return CalcPercAB(CalcStat("PartFinesseDmgPRatPA",L),CalcStat("PartFinesseDmgPRatPB",L),CalcStat("PartFinesseDmgPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN > "PARTFINESSEDMGPRATPA" then
											if SN == "PARTFINESSEDMGPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										else
											return 150;
										end
									else
										return CalcRatAB(CalcStat("PartFinesseDmgPRatPA",L),CalcStat("PartFinesseDmgPRatPB",L),CalcStat("PartFinesseDmgPRatPCapR",L),N);
									end
								else
									return CalcStat("PartBPEPRatPC",L);
								end
							elseif SN > "PARTFINESSEDMGPRATPC" then
								if SN < "PARTFINESSEPRATPC" then
									if SN < "PARTFINESSEPPRAT" then
										if SN < "PARTFINESSEDMGPRATPCAPR" then
											if SN == "PARTFINESSEDMGPRATPCAP" then
												return 50;
											else
												return 0;
											end
										elseif SN > "PARTFINESSEDMGPRATPCAPR" then
											if SN == "PARTFINESSEPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return CalcStat("PartFinesseDmgPRatPB",L)*CalcStat("PartFinesseDmgPRatPC",L);
										end
									elseif SN > "PARTFINESSEPPRAT" then
										if SN < "PARTFINESSEPRATPA" then
											if SN == "PARTFINESSEPRATP" then
												return CalcPercAB(CalcStat("PartFinessePRatPA",L),CalcStat("PartFinessePRatPB",L),CalcStat("PartFinessePRatPCap",L),N);
											else
												return 0;
											end
										elseif SN > "PARTFINESSEPRATPA" then
											if SN == "PARTFINESSEPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										else
											return 150;
										end
									else
										return CalcRatAB(CalcStat("PartFinessePRatPA",L),CalcStat("PartFinessePRatPB",L),CalcStat("PartFinessePRatPCapR",L),N);
									end
								elseif SN > "PARTFINESSEPRATPC" then
									if SN < "PARTMITPPRAT" then
										if SN < "PARTFINESSEPRATPCAPR" then
											if SN == "PARTFINESSEPRATPCAP" then
												return 50;
											else
												return 0;
											end
										elseif SN > "PARTFINESSEPRATPCAPR" then
											if SN == "PARTMITPBONUS" then
												return 0.1;
											else
												return 0;
											end
										else
											return CalcStat("PartFinessePRatPB",L)*CalcStat("PartFinessePRatPC",L);
										end
									elseif SN > "PARTMITPPRAT" then
										if SN < "PARTMITPRATPA" then
											if SN == "PARTMITPRATP" then
												return CalcPercAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN > "PARTMITPRATPA" then
											if SN == "PARTMITPRATPB" then
												return CalcStat("BRatRounded",L,"BRatPartBPE");
											else
												return 0;
											end
										else
											return 105;
										end
									else
										return CalcRatAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCapR",L),N);
									end
								else
									return 0.5;
								end
							else
								return 0.5;
							end
						elseif SN > "PARTMITPRATPC" then
							if SN < "PARTPARRYPRATPC" then
								if SN < "PARTPARRYMITPRATPC" then
									if SN < "PARTPARRYMITPPRAT" then
										if SN < "PARTMITPRATPCAPR" then
											if SN == "PARTMITPRATPCAP" then
												return 35;
											else
												return 0;
											end
										elseif SN > "PARTMITPRATPCAPR" then
											if SN == "PARTPARRYMITPBONUS" then
												return CalcStat("PartMitPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPB",L)*CalcStat("PartMitPRatPC",L);
										end
									elseif SN > "PARTPARRYMITPPRAT" then
										if SN < "PARTPARRYMITPRATPA" then
											if SN == "PARTPARRYMITPRATP" then
												return CalcStat("PartMitPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PARTPARRYMITPRATPA" then
											if SN == "PARTPARRYMITPRATPB" then
												return CalcStat("PartMitPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPA",L);
										end
									else
										return CalcStat("PartMitPPRat",L,N);
									end
								elseif SN > "PARTPARRYMITPRATPC" then
									if SN < "PARTPARRYPPRAT" then
										if SN < "PARTPARRYMITPRATPCAPR" then
											if SN == "PARTPARRYMITPRATPCAP" then
												return CalcStat("PartMitPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYMITPRATPCAPR" then
											if SN == "PARTPARRYPBONUS" then
												return CalcStat("PartBPEPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPCapR",L);
										end
									elseif SN > "PARTPARRYPPRAT" then
										if SN < "PARTPARRYPRATPA" then
											if SN == "PARTPARRYPRATP" then
												return CalcStat("PartBPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PARTPARRYPRATPA" then
											if SN == "PARTPARRYPRATPB" then
												return CalcStat("PartBPEPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPA",L);
										end
									else
										return CalcStat("PartBPEPPRat",L,N);
									end
								else
									return CalcStat("PartMitPRatPC",L);
								end
							elseif SN > "PARTPARRYPRATPC" then
								if SN < "PERKTACMIT" then
									if SN < "PERKNCMR" then
										if SN < "PARTPARRYPRATPCAPR" then
											if SN == "PARTPARRYPRATPCAP" then
												return CalcStat("PartBPEPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYPRATPCAPR" then
											if SN == "PERKMORALE" then
												return DataTableValue({10,20,30,40},L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPCapR",L);
										end
									elseif SN > "PERKNCMR" then
										if SN < "PERKPHYMIT" then
											if SN == "PERKNCPR" then
												return CalcStat("FoodNCPRL",L);
											else
												return 0;
											end
										elseif SN > "PERKPHYMIT" then
											if SN == "PERKPOWER" then
												return DataTableValue({10,20,30,40},L);
											else
												return 0;
											end
										else
											return CalcStat("PhyMitT",L,0.2*N);
										end
									else
										return CalcStat("FoodNCMRL",L);
									end
								elseif SN > "PERKTACMIT" then
									if SN < "PHYDMGPRATPA" then
										if SN < "PHYDMGPPRAT" then
											if SN == "PHYDMGPBONUS" then
												return CalcStat("OutDmgPBonus",L);
											else
												return 0;
											end
										elseif SN > "PHYDMGPPRAT" then
											if SN == "PHYDMGPRATP" then
												return CalcStat("OutDmgPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPPRat",L,N);
										end
									elseif SN > "PHYDMGPRATPA" then
										if SN < "PHYDMGPRATPC" then
											if SN == "PHYDMGPRATPB" then
												return CalcStat("OutDmgPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PHYDMGPRATPC" then
											if SN == "PHYDMGPRATPCAP" then
												return CalcStat("OutDmgPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatPC",L);
										end
									else
										return CalcStat("OutDmgPRatPA",L);
									end
								else
									return CalcStat("TacMitT",L,0.2*N);
								end
							else
								return CalcStat("PartBPEPRatPC",L);
							end
						else
							return 0.5;
						end
					elseif SN > "PHYDMGPRATPCAPR" then
						if SN < "PHYMITT" then
							if SN < "PHYMITLPBONUS" then
								if SN < "PHYMITHPBONUS" then
									if SN < "PHYMASOLD" then
										if SN > "PHYMAS" then
											if SN == "PHYMASC" then
												return CalcStat("MasteryC",L,N);
											else
												return 0;
											end
										elseif SN == "PHYMAS" then
											return CalcStat("Mastery",L,N);
										else
											return 0;
										end
									elseif SN > "PHYMASOLD" then
										if SN < "PHYMIT" then
											if SN == "PHYMAST" then
												return CalcStat("MasteryT",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMIT" then
											if SN == "PHYMITC" then
												return StatLinInter("PntMPPhyMitC","CreepPntS","ProgBMitigationC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPPhyMit","ItemPntS","ProgBMitigation","AdjUmbarItemMit",L,N,0);
										end
									else
										return CalcStat("Mastery",L,N);
									end
								elseif SN > "PHYMITHPBONUS" then
									if SN < "PHYMITHPRATPB" then
										if SN < "PHYMITHPRATP" then
											if SN == "PHYMITHPPRAT" then
												return CalcStat("MitHeavyPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITHPRATP" then
											if SN == "PHYMITHPRATPA" then
												return CalcStat("MitHeavyPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatP",L,N);
										end
									elseif SN > "PHYMITHPRATPB" then
										if SN < "PHYMITHPRATPCAP" then
											if SN == "PHYMITHPRATPC" then
												return CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PHYMITHPRATPCAP" then
											if SN == "PHYMITHPRATPCAPR" then
												return CalcStat("MitHeavyPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPCap",L);
										end
									else
										return CalcStat("MitHeavyPRatPB",L);
									end
								else
									return CalcStat("MitHeavyPBonus",L);
								end
							elseif SN > "PHYMITLPBONUS" then
								if SN < "PHYMITMPBONUS" then
									if SN < "PHYMITLPRATPB" then
										if SN < "PHYMITLPRATP" then
											if SN == "PHYMITLPPRAT" then
												return CalcStat("MitLightPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITLPRATP" then
											if SN == "PHYMITLPRATPA" then
												return CalcStat("MitLightPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatP",L,N);
										end
									elseif SN > "PHYMITLPRATPB" then
										if SN < "PHYMITLPRATPCAP" then
											if SN == "PHYMITLPRATPC" then
												return CalcStat("MitLightPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PHYMITLPRATPCAP" then
											if SN == "PHYMITLPRATPCAPR" then
												return CalcStat("MitLightPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatPCap",L);
										end
									else
										return CalcStat("MitLightPRatPB",L);
									end
								elseif SN > "PHYMITMPBONUS" then
									if SN < "PHYMITMPRATPB" then
										if SN < "PHYMITMPRATP" then
											if SN == "PHYMITMPPRAT" then
												return CalcStat("MitMediumPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITMPRATP" then
											if SN == "PHYMITMPRATPA" then
												return CalcStat("MitMediumPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatP",L,N);
										end
									elseif SN > "PHYMITMPRATPB" then
										if SN < "PHYMITMPRATPCAP" then
											if SN == "PHYMITMPRATPC" then
												return CalcStat("MitMediumPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PHYMITMPRATPCAP" then
											if SN == "PHYMITMPRATPCAPR" then
												return CalcStat("MitMediumPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPCap",L);
										end
									else
										return CalcStat("MitMediumPRatPB",L);
									end
								else
									return CalcStat("MitMediumPBonus",L);
								end
							else
								return CalcStat("MitLightPBonus",L);
							end
						elseif SN > "PHYMITT" then
							if SN < "PNTMPCLASSBASENCPR" then
								if SN < "PNTMPARMOUR" then
									if SN < "PLAYERBASEMORALE" then
										if SN < "PHYRESISTT" then
											if SN == "PHYRESIST" then
												return CalcStat("ResistAdd",L,N);
											else
												return 0;
											end
										elseif SN > "PHYRESISTT" then
											if SN == "PLAYERBASEEVADE" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									elseif SN > "PLAYERBASEMORALE" then
										if SN < "PLAYERBASEPHYMAS" then
											if SN == "PLAYERBASEPARRY" then
												return 3;
											else
												return 0;
											end
										elseif SN > "PLAYERBASEPHYMAS" then
											if SN == "PLAYERBASETACMAS" then
												return 11.5;
											else
												return 0;
											end
										else
											return 11.5;
										end
									else
										return 52.5;
									end
								elseif SN > "PNTMPARMOUR" then
									if SN < "PNTMPARMOURVIRTUES" then
										if SN < "PNTMPARMOURPENT" then
											if SN == "PNTMPARMOURC" then
												return 16990/6;
											else
												return 0;
											end
										elseif SN > "PNTMPARMOURPENT" then
											if SN == "PNTMPARMOURT" then
												return 25/1200;
											else
												return 0;
											end
										else
											return 72/1200;
										end
									elseif SN > "PNTMPARMOURVIRTUES" then
										if SN < "PNTMPBPEC" then
											if SN == "PNTMPBPE" then
												return 42/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPBPEC" then
											if SN == "PNTMPCLASSBASEICPR" then
												return 0.125;
											else
												return 0;
											end
										else
											return 20430/6;
										end
									else
										return 24/1200;
									end
								else
									return 4.4/1200;
								end
							elseif SN > "PNTMPCLASSBASENCPR" then
								if SN < "PNTMPFATE" then
									if SN < "PNTMPCRITHIT" then
										if SN < "PNTMPCRITDEF" then
											if SN == "PNTMPCOMBATDAMAGEMOD" then
												return 300/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPCRITDEF" then
											if SN == "PNTMPCRITDEFC" then
												return 20430/6;
											else
												return 0;
											end
										else
											return 40/1200;
										end
									elseif SN > "PNTMPCRITHIT" then
										if SN < "PNTMPDMGTYPEMIT" then
											if SN == "PNTMPCRITHITC" then
												return 30610/6;
											else
												return 0;
											end
										elseif SN > "PNTMPDMGTYPEMIT" then
											if SN == "PNTMPDMGTYPEMITT" then
												return 60/1200;
											else
												return 0;
											end
										else
											return 24/1200;
										end
									else
										return 20/1200;
									end
								elseif SN > "PNTMPFATE" then
									if SN < "PNTMPFINESSET" then
										if SN < "PNTMPFINESSE" then
											if SN == "PNTMPFATEC" then
												return 1803/6;
											else
												return 0;
											end
										elseif SN > "PNTMPFINESSE" then
											if SN == "PNTMPFINESSEC" then
												return 20430/6;
											else
												return 0;
											end
										else
											return 40/1200;
										end
									elseif SN > "PNTMPFINESSET" then
										if SN < "PNTMPICMR" then
											if SN == "PNTMPFOODRESIST" then
												return 30/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPICMR" then
											if SN == "PNTMPICMRC" then
												return 44.4;
											else
												return 0;
											end
										else
											return 0.015;
										end
									else
										return 36/1200;
									end
								else
									return 1.25;
								end
							else
								return 0.25;
							end
						else
							return StatLinInter("PntMPPhyMit","TraitPntS","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
						end
					else
						return CalcStat("OutDmgPRatPCapR",L);
					end
				else
					return CalcStat("PartMitPRatPCap",L);
				end
			elseif SN > "PNTMPICMRCOLD" then
				if SN < "RIVERHOBBITRDTRAITFROSTMITP" then
					if SN < "PROGBMITIGATION" then
						if SN < "PNTMPVITALITYT" then
							if SN < "PNTMPNCPR" then
								if SN < "PNTMPMASTERY" then
									if SN < "PNTMPICPRCOLD" then
										if SN > "PNTMPICPR" then
											if SN == "PNTMPICPRC" then
												return 5.1;
											else
												return 0;
											end
										elseif SN == "PNTMPICPR" then
											return 0.0625;
										else
											return 0;
										end
									elseif SN > "PNTMPICPRCOLD" then
										if SN < "PNTMPMAIN" then
											if SN == "PNTMPINHEAL" then
												return 40/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPMAIN" then
											if SN == "PNTMPMAINC" then
												return 1803/6;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 3.125;
									end
								elseif SN > "PNTMPMASTERY" then
									if SN < "PNTMPMORALEVIRTUES" then
										if SN < "PNTMPMITIGATION" then
											if SN == "PNTMPMASTERYC" then
												return 30610/6;
											else
												return 0;
											end
										elseif SN > "PNTMPMITIGATION" then
											if SN == "PNTMPMORALE" then
												return 1;
											else
												return 0;
											end
										else
											return 28/1200;
										end
									elseif SN > "PNTMPMORALEVIRTUES" then
										if SN < "PNTMPNCMRC" then
											if SN == "PNTMPNCMR" then
												return 0.15;
											else
												return 0;
											end
										elseif SN > "PNTMPNCMRC" then
											if SN == "PNTMPNCMRCOLD" then
												return 108;
											else
												return 0;
											end
										else
											return 164.8;
										end
									else
										return 0.75;
									end
								else
									return 17/1200;
								end
							elseif SN > "PNTMPNCPR" then
								if SN < "PNTMPRESIST" then
									if SN < "PNTMPPHYMIT" then
										if SN < "PNTMPNCPRCOLD" then
											if SN == "PNTMPNCPRC" then
												return 19;
											else
												return 0;
											end
										elseif SN > "PNTMPNCPRCOLD" then
											if SN == "PNTMPOUTHEAL" then
												return 30/1200;
											else
												return 0;
											end
										else
											return 12.5;
										end
									elseif SN > "PNTMPPHYMIT" then
										if SN < "PNTMPPOWER" then
											if SN == "PNTMPPHYMITC" then
												return 16990/6;
											else
												return 0;
											end
										elseif SN > "PNTMPPOWER" then
											if SN == "PNTMPPOWERT" then
												return 0.6666;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 36/1200;
									end
								elseif SN > "PNTMPRESIST" then
									if SN < "PNTMPTACMIT" then
										if SN < "PNTMPSHIELDBLOCK" then
											if SN == "PNTMPRESISTC" then
												return 30610/6;
											else
												return 0;
											end
										elseif SN > "PNTMPSHIELDBLOCK" then
											if SN == "PNTMPTACHPS" then
												return 0.0875;
											else
												return 0;
											end
										else
											return 80/1200;
										end
									elseif SN > "PNTMPTACMIT" then
										if SN < "PNTMPVITALITY" then
											if SN == "PNTMPTACMITC" then
												return 16990/6;
											else
												return 0;
											end
										elseif SN > "PNTMPVITALITY" then
											if SN == "PNTMPVITALITYC" then
												return 4500/6;
											else
												return 0;
											end
										else
											return 0.175;
										end
									else
										return 36/1200;
									end
								else
									return 36/1200;
								end
							else
								return 0.5;
							end
						elseif SN > "PNTMPVITALITYT" then
							if SN < "PROGBCRITHITC" then
								if SN < "PROGBARMOURLIGHT" then
									if SN < "POWERT" then
										if SN < "POISONRESISTT" then
											if SN == "POISONRESIST" then
												return CalcStat("ResistAdd",L,N);
											else
												return 0;
											end
										elseif SN > "POISONRESISTT" then
											if SN == "POWER" then
												return StatLinInter("PntMPPower","ItemPntSVital","ProgBPower","",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									elseif SN > "POWERT" then
										if SN < "PROGBARMOURC" then
											if SN == "PROGBARMOUR" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "PROGBARMOURC" then
											if SN == "PROGBARMOURHEAVY" then
												return CalcStat("BRatProgB",L,"BRatMitHeavy");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									else
										return StatLinInter("PntMPPowerT","TraitPntSVital","ProgBPower","",L,N,0);
									end
								elseif SN > "PROGBARMOURLIGHT" then
									if SN < "PROGBCOMBATDAMAGEMOD" then
										if SN < "PROGBBPE" then
											if SN == "PROGBARMOURMEDIUM" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "PROGBBPE" then
											if SN == "PROGBBPEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatStandard");
										end
									elseif SN > "PROGBCOMBATDAMAGEMOD" then
										if SN < "PROGBCRITDEFC" then
											if SN == "PROGBCRITDEF" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "PROGBCRITDEFC" then
											if SN == "PROGBCRITHIT" then
												return CalcStat("BRatProgB",L,"BRatExtra");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									else
										if Lm <= 25 then
											return LinFmod(1,5,45.58309,1,25,L);
										elseif Lm <= 50 then
											return LinFmod(1,45.58309,120,25,50,L);
										elseif Lm <= 60 then
											return LinFmod(1,120,175.56,50,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,175.56,200.79,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,200.79,292.5,65,75,L);
										elseif Lm <= 85 then
											return LinFmod(1,292.5,421.2,75,85,L);
										elseif Lm <= 95 then
											return LinFmod(1,421.2,600.3,85,95,L);
										elseif Lm <= 100 then
											return LinFmod(1,600.3,774,95,100,L);
										elseif Lm <= 105 then
											return LinFmod(1,774,985.5,100,105,L);
										elseif Lm <= 115 then
											return LinFmod(1,985.5,1350,105,115,L);
										elseif Lm <= 120 then
											return LinFmod(1,1350,1710,115,120,L);
										elseif Lm <= 130 then
											return LinFmod(1,1710,2330.25,120,130,L);
										elseif Lm <= 140 then
											return LinFmod(1,2330.25,3600,130,140,L);
										else
											return LinFmod(1,4010,5535,141,150,L);
										end
									end
								else
									return CalcStat("BRatProgB",L,"BRatMitLight");
								end
							elseif SN > "PROGBCRITHITC" then
								if SN < "PROGBICMRC" then
									if SN < "PROGBFINESSE" then
										if SN < "PROGBFATE" then
											if SN == "PROGBENERGY" then
												if Lm <= 50 then
													return LinFmod(1,4,8,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,8,10.6,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,10.6,13.2,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,13.2,19.8,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,19.8,29.8,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,29.8,39.6,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,39.6,52,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,52,70,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,78,106,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,122,132,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,152,198,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,228,396,131,140,L);
												elseif Lm <= 150 then
													return LinFmod(1,456,800,141,150,L);
												else
													return LinFmod(1,800,1600,151,160,L);
												end
											else
												return 0;
											end
										elseif SN > "PROGBFATE" then
											if SN == "PROGBFATEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									elseif SN > "PROGBFINESSE" then
										if SN < "PROGBHEALTH" then
											if SN == "PROGBFINESSEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										elseif SN > "PROGBHEALTH" then
											if SN == "PROGBICMR" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,8,60,1,50,L);
											elseif Lm <= 60 then
												return LinFmod(1,60,80,50,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,80,100,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,100,150,65,75,L);
											elseif Lm <= 85 then
												return LinFmod(1,150,226,75,85,L);
											elseif Lm <= 95 then
												return LinFmod(1,226,300,85,95,L);
											elseif Lm <= 100 then
												return LinFmod(1,300,450,95,100,L);
											elseif Lm <= 105 then
												return LinFmod(1,450,600,100,105,L);
											elseif Lm <= 115 then
												return LinFmod(1,660,900,106,115,L);
											elseif Lm <= 120 then
												return LinFmod(1,1040,1120,116,120,L);
											elseif Lm <= 130 then
												return LinFmod(1,1280,1680,121,130,L);
											elseif Lm <= 140 then
												return LinFmod(1,1940,3360,131,140,L);
											elseif Lm <= 150 then
												return LinFmod(1,3860,6800,141,150,L);
											else
												return LinFmod(1,6800,13600,151,160,L);
											end
										end
									else
										return CalcStat("BRatProgB",L,"BRatStandard");
									end
								elseif SN > "PROGBICMRC" then
									if SN < "PROGBMAIN" then
										if SN < "PROGBICPRC" then
											if SN == "PROGBICPR" then
												return CalcStat("ProgBEnergy",L);
											else
												return 0;
											end
										elseif SN > "PROGBICPRC" then
											if SN == "PROGBINHEAL" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									elseif SN > "PROGBMAIN" then
										if SN < "PROGBMASTERY" then
											if SN == "PROGBMAINC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										elseif SN > "PROGBMASTERY" then
											if SN == "PROGBMASTERYC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatExtra");
										end
									else
										return CalcStat("StdProg",L,1.75);
									end
								else
									return CalcStat("CreepProgB",L);
								end
							else
								return CalcStat("CreepProgB",L);
							end
						else
							return 0.225;
						end
					elseif SN > "PROGBMITIGATION" then
						if SN < "REPFATEH" then
							if SN < "PROGEXTCOMHIGHRND" then
								if SN < "PROGBOUTHEAL" then
									if SN < "PROGBNCMR" then
										if SN > "PROGBMITIGATIONC" then
											if SN == "PROGBMORALE" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN == "PROGBMITIGATIONC" then
											return CalcStat("CreepProgB",L);
										else
											return 0;
										end
									elseif SN > "PROGBNCMR" then
										if SN < "PROGBNCPR" then
											if SN == "PROGBNCMRC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										elseif SN > "PROGBNCPR" then
											if SN == "PROGBNCPRC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									else
										return CalcStat("ProgBHealth",L);
									end
								elseif SN > "PROGBOUTHEAL" then
									if SN < "PROGBTACHPS" then
										if SN < "PROGBRESIST" then
											if SN == "PROGBPOWER" then
												return CalcStat("ProgBEnergy",L);
											else
												return 0;
											end
										elseif SN > "PROGBRESIST" then
											if SN == "PROGBRESISTC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatExtra");
										end
									elseif SN > "PROGBTACHPS" then
										if SN < "PROGBVITALITYC" then
											if SN == "PROGBVITALITY" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN > "PROGBVITALITYC" then
											if SN == "PROGEXTCOMHIGHRAW" then
												if Lm <= 121 then
													return ExpFmod(N,121,20,L,nil);
												elseif Lm <= 125 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",121,N),122,5.5,L,nil);
												elseif Lm <= 126 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",125,N),126,20,L,nil);
												elseif Lm <= 130 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",126,N),127,5.5,L,nil);
												elseif Lm <= 131 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",130,N),131,20,L,nil);
												elseif Lm <= 150 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",131,N),132,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",150,N);
												end
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									else
										return CalcStat("ProgBHealth",L);
									end
								else
									return CalcStat("BRatProgB",L,"BRatOutHeal");
								end
							elseif SN > "PROGEXTCOMHIGHRND" then
								if SN < "REAVERCDCALCTYPECOMPHYMIT" then
									if SN < "PROGEXTLOWEXPRND" then
										if SN < "PROGEXTCOMLOWRND" then
											if SN == "PROGEXTCOMLOWRAW" then
												if Lm <= 116 then
													return ExpFmod(N,116,20,L,nil);
												elseif Lm <= 120 then
													return ExpFmod(CalcStat("ProgExtComLowRaw",116,N),117,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",L,CalcStat("ProgExtComLowRaw",120,N));
												end
											else
												return 0;
											end
										elseif SN > "PROGEXTCOMLOWRND" then
											if SN == "PROGEXTHIGHLINEXPRND" then
												if Lm <= 115 then
													return LinFmod(N,2,4,106,115,L,-1);
												else
													return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtHighLinExpRnd",115,N));
												end
											else
												return 0;
											end
										else
											if Lm <= 116 then
												return ExpFmod(N,116,20,L,0);
											elseif Lm <= 120 then
												return ExpFmod(CalcStat("ProgExtComLowRnd",116,N),117,5.5,L,0);
											else
												return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtComLowRnd",120,N));
											end
										end
									elseif SN > "PROGEXTLOWEXPRND" then
										if SN < "PROGEXTMPEXPRND" then
											if SN == "PROGEXTMEDEXPRAW" then
												if Lm <= 106 then
													return ExpFmod(N,106,10,L,nil);
												elseif Lm <= 115 then
													return ExpFmod(CalcStat("ProgExtMedExpRaw",106,N),107,5.5,L,nil);
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("ProgExtMedExpRaw",115,N));
												end
											else
												return 0;
											end
										elseif SN > "PROGEXTMPEXPRND" then
											if SN == "REAVERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 114 then
												return ExpFmod(N,106,7.5,L,0);
											elseif Lm <= 115 then
												return 2*N;
											elseif Lm <= 119 then
												return ExpFmod(CalcStat("ProgExtMpExpRnd",115,N),116,25,L,0);
											elseif Lm <= 120 then
												return RoundDbl((16/3)*N);
											else
												return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtMpExpRnd",120,N));
											end
										end
									else
										if Lm <= 115 then
											return ExpFmod(N,106,5.5,L,0);
										else
											return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtLowExpRnd",115,N));
										end
									end
								elseif SN > "REAVERCDCALCTYPECOMPHYMIT" then
									if SN < "REPAGILITYH" then
										if SN < "REAVERCDCALCTYPETACMIT" then
											if SN == "REAVERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "REAVERCDCALCTYPETACMIT" then
											if SN == "REAVERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "REPAGILITYH" then
										if SN < "REPCRITDEF" then
											if SN == "REPAGILITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "REPCRITDEF" then
											if SN == "REPCRITHIT" then
												if Lm <= 50 then
													return LinFmod(1,300,496,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,496,636,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,636,716,85,105,L);
												else
													return LinFmod(1,716,776,105,120,L);
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,900,1488,1,50,L);
											elseif Lm <= 85 then
												return LinFmod(1,1488,1908,50,85,L);
											elseif Lm <= 105 then
												return LinFmod(1,1908,2148,85,105,L);
											else
												return LinFmod(1,2148,2328,105,120,L);
											end
										end
									else
										return CalcStat("RepMainH",L);
									end
								else
									return 13;
								end
							else
								if Lm <= 121 then
									return ExpFmod(N,121,20,L,0);
								elseif Lm <= 125 then
									return ExpFmod(CalcStat("ProgExtComHighRnd",121,N),122,5.5,L,0);
								elseif Lm <= 126 then
									return ExpFmod(CalcStat("ProgExtComHighRnd",125,N),126,20,L,0);
								elseif Lm <= 130 then
									return ExpFmod(CalcStat("ProgExtComHighRnd",126,N),127,5.5,L,0);
								elseif Lm <= 131 then
									return ExpFmod(CalcStat("ProgExtComHighRnd",130,N),131,20,L,0);
								elseif Lm <= 150 then
									return ExpFmod(CalcStat("ProgExtComHighRnd",131,N),132,5.5,L,0);
								else
									return CalcStat("ProgExtComHighRnd",150,N);
								end
							end
						elseif SN > "REPFATEH" then
							if SN < "RESISTADDT" then
								if SN < "REPPOWER" then
									if SN < "REPMAINL" then
										if SN < "REPFINESSE" then
											if SN == "REPFATEL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "REPFINESSE" then
											if SN == "REPMAINH" then
												if Lm <= 50 then
													return RoundDblDown(LinFmod(1,80,153,1,50,L));
												elseif Lm <= 85 then
													return RoundDblDown(LinFmod(1,153,206,50,85,L));
												elseif Lm <= 105 then
													return RoundDblDown(LinFmod(1,206,236,85,105,L));
												else
													return RoundDblDown(LinFmod(1,236,258,105,120,L));
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,322,557,1,50,L);
											elseif Lm <= 85 then
												return LinFmod(1,557,749.9477,50,85,L);
											elseif Lm <= 105 then
												return LinFmod(1,749.9477,859.1634,85,105,L);
											else
												return LinFmod(1,859.1634,939.2549,105,120,L);
											end
										end
									elseif SN > "REPMAINL" then
										if SN < "REPMIGHTL" then
											if SN == "REPMIGHTH" then
												return CalcStat("RepMainH",L);
											else
												return 0;
											end
										elseif SN > "REPMIGHTL" then
											if SN == "REPMORALE" then
												if Lm <= 50 then
													return LinFmod(1,187,427,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,427,599,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,599,697,85,105,L);
												else
													return LinFmod(1,697,770,105,120,L);
												end
											else
												return 0;
											end
										else
											return CalcStat("RepMainL",L);
										end
									else
										if Lm <= 50 then
											return RoundDblDown(LinFmod(1,53,102,1,50,L));
										elseif Lm <= 85 then
											return RoundDblDown(LinFmod(1,102,137,50,85,L));
										elseif Lm <= 105 then
											return RoundDblDown(LinFmod(1,137,157,85,105,L));
										else
											return RoundDblDown(LinFmod(1,157,172,105,120,L));
										end
									end
								elseif SN > "REPPOWER" then
									if SN < "REPWILLH" then
										if SN < "REPVITALITYH" then
											if SN == "REPTACMIT" then
												if Lm <= 50 then
													return LinFmod(1,675,1116,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,1116,1431,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,1431,1611,85,105,L);
												else
													return LinFmod(1,1611,1746,105,120,L);
												end
											else
												return 0;
											end
										elseif SN > "REPVITALITYH" then
											if SN == "REPVITALITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										else
											return CalcStat("RepMainH",L);
										end
									elseif SN > "REPWILLH" then
										if SN < "RESIST" then
											if SN == "REPWILLL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "RESIST" then
											if SN == "RESISTADD" then
												return CalcStat("Resist",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPResist","ItemPntS","ProgBResist","AdjUmbarItem",L,N,0);
										end
									else
										return CalcStat("RepMainH",L);
									end
								else
									if Lm <= 50 then
										return LinFmod(1,94,212,1,50,L);
									elseif Lm <= 85 then
										return LinFmod(1,212,296,50,85,L);
									elseif Lm <= 105 then
										return LinFmod(1,296,344,85,105,L);
									else
										return LinFmod(1,344,380,105,120,L);
									end
								end
							elseif SN > "RESISTADDT" then
								if SN < "RESISTPRATPCAP" then
									if SN < "RESISTPRATP" then
										if SN < "RESISTPBONUS" then
											if SN == "RESISTC" then
												return StatLinInter("PntMPResistC","CreepPntS","ProgBResistC","",L,N,0);
											else
												return 0;
											end
										elseif SN > "RESISTPBONUS" then
											if SN == "RESISTPPRAT" then
												return CalcRatAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "RESISTPRATP" then
										if SN < "RESISTPRATPB" then
											if SN == "RESISTPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN > "RESISTPRATPB" then
											if SN == "RESISTPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatExtra");
										end
									else
										return CalcPercAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCap",L),N);
									end
								elseif SN > "RESISTPRATPCAP" then
									if SN < "RIVERHOBBITRDPSVONENAME" then
										if SN < "RESISTT" then
											if SN == "RESISTPRATPCAPR" then
												return CalcStat("ResistPRatPB",L)*CalcStat("ResistPRatPC",L);
											else
												return 0;
											end
										elseif SN > "RESISTT" then
											if SN == "REVERSE" then
												return ReverseCalc(C,L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPResist","TraitPntS","ProgBResist","AdjUmbarTrait",L,N,0);
										end
									elseif SN > "RIVERHOBBITRDPSVONENAME" then
										if SN < "RIVERHOBBITRDPSVTWONAME" then
											if SN == "RIVERHOBBITRDPSVONEWILL" then
												return CalcStat("RivHobSeenWorldWill",L);
											else
												return 0;
											end
										elseif SN > "RIVERHOBBITRDPSVTWONAME" then
											if SN == "RIVERHOBBITRDTRAITAGILITY" then
												return CalcStat("RivHobSlipperyAgility",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return "Seen the World";
									end
								else
									return 50;
								end
							else
								return CalcStat("ResistT",L,N);
							end
						else
							return CalcStat("RepMainH",L);
						end
					else
						return CalcStat("BRatProgB",L,"BRatMitMedium");
					end
				elseif SN > "RIVERHOBBITRDTRAITFROSTMITP" then
					if SN < "STALKERCDHASPOWER" then
						if SN < "RUNEKEEPERCDCALCTYPETACMIT" then
							if SN < "RUNEKEEPERCDARMOURTONONPHYMIT" then
								if SN < "RIVHOBSWIMMERFROSTMITP" then
									if SN < "RIVHOBHARDYHOLBMORALE" then
										if SN > "RIVERHOBBITRDTRAITMORALE" then
											if SN == "RIVERHOBBITRDTRAITWILL" then
												return CalcStat("RivHobSeclusionWill",L);
											else
												return 0;
											end
										elseif SN == "RIVERHOBBITRDTRAITMORALE" then
											return CalcStat("RivHobHardyHolbMorale",L);
										else
											return 0;
										end
									elseif SN > "RIVHOBHARDYHOLBMORALE" then
										if SN < "RIVHOBSEENWORLDWILL" then
											if SN == "RIVHOBSECLUSIONWILL" then
												return -CalcStat("WillT",L,0.4);
											else
												return 0;
											end
										elseif SN > "RIVHOBSEENWORLDWILL" then
											if SN == "RIVHOBSLIPPERYAGILITY" then
												return CalcStat("AgilityT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									else
										return CalcStat("MoraleT",L,1.0);
									end
								elseif SN > "RIVHOBSWIMMERFROSTMITP" then
									if SN < "RUNEKEEPERCDAGILITYTOEVADE" then
										if SN < "RKFORTUNESMILESFATE" then
											if SN == "RKDETERMINATIONWILL" then
												return CalcStat("WillT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "RKFORTUNESMILESFATE" then
											if SN == "RUNEKEEPERCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									elseif SN > "RUNEKEEPERCDAGILITYTOEVADE" then
										if SN < "RUNEKEEPERCDAGILITYTOTACMAS" then
											if SN == "RUNEKEEPERCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDAGILITYTOTACMAS" then
											if SN == "RUNEKEEPERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "RUNEKEEPERCDARMOURTONONPHYMIT" then
								if SN < "RUNEKEEPERCDBASEMORALE" then
									if SN < "RUNEKEEPERCDBASEFATE" then
										if SN < "RUNEKEEPERCDARMOURTYPE" then
											if SN == "RUNEKEEPERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDARMOURTYPE" then
											if SN == "RUNEKEEPERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityL",L);
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "RUNEKEEPERCDBASEFATE" then
										if SN < "RUNEKEEPERCDBASEICPR" then
											if SN == "RUNEKEEPERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASEICPR" then
											if SN == "RUNEKEEPERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									else
										return CalcStat("ClassBaseFate",L);
									end
								elseif SN > "RUNEKEEPERCDBASEMORALE" then
									if SN < "RUNEKEEPERCDBASEVITALITY" then
										if SN < "RUNEKEEPERCDBASENCPR" then
											if SN == "RUNEKEEPERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASENCPR" then
											if SN == "RUNEKEEPERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									elseif SN > "RUNEKEEPERCDBASEVITALITY" then
										if SN < "RUNEKEEPERCDCALCTYPECOMPHYMIT" then
											if SN == "RUNEKEEPERCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDCALCTYPECOMPHYMIT" then
											if SN == "RUNEKEEPERCDCALCTYPENONPHYMIT" then
												return 12;
											else
												return 0;
											end
										else
											return 12;
										end
									else
										return CalcStat("ClassBaseVitality",L);
									end
								else
									return CalcStat("ClassBaseMorale",L);
								end
							else
								return 0.2;
							end
						elseif SN > "RUNEKEEPERCDCALCTYPETACMIT" then
							if SN < "RUNEKEEPERCDWILLTOPHYMIT" then
								if SN < "RUNEKEEPERCDPHYMITTOCOMPHYMIT" then
									if SN < "RUNEKEEPERCDMIGHTTOCRITHIT" then
										if SN < "RUNEKEEPERCDFATETOPOWER" then
											if SN == "RUNEKEEPERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDFATETOPOWER" then
											if SN == "RUNEKEEPERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "RUNEKEEPERCDMIGHTTOCRITHIT" then
										if SN < "RUNEKEEPERCDMIGHTTOTACMAS" then
											if SN == "RUNEKEEPERCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDMIGHTTOTACMAS" then
											if SN == "RUNEKEEPERCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1;
									end
								elseif SN > "RUNEKEEPERCDPHYMITTOCOMPHYMIT" then
									if SN < "RUNEKEEPERCDVITALITYTOMORALE" then
										if SN < "RUNEKEEPERCDTACMASTOOUTHEAL" then
											if SN == "RUNEKEEPERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDTACMASTOOUTHEAL" then
											if SN == "RUNEKEEPERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "RUNEKEEPERCDVITALITYTOMORALE" then
										if SN < "RUNEKEEPERCDWILLTOCRITHIT" then
											if SN == "RUNEKEEPERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDWILLTOCRITHIT" then
											if SN == "RUNEKEEPERCDWILLTOEVADE" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 4.5;
									end
								else
									return 1;
								end
							elseif SN > "RUNEKEEPERCDWILLTOPHYMIT" then
								if SN < "SKILLPOWERCOST" then
									if SN < "SHADOWMIT" then
										if SN < "RUNEKEEPERCDWILLTOTACMAS" then
											if SN == "RUNEKEEPERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDWILLTOTACMAS" then
											if SN == "RUNEKEEPERCDWILLTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "SHADOWMIT" then
										if SN < "SHIELDBLOCK" then
											if SN == "SHADOWMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										elseif SN > "SHIELDBLOCK" then
											if SN == "SHIELDBRAWLERBLOCK" then
												return CalcStat("DwarfShieldBrwlBlock",L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPShieldBlock","ItemPntS","ProgBBPE","AdjUmbarItem",L,N,0);
										end
									else
										return CalcStat("DmgTypeMit",L,N);
									end
								elseif SN > "SKILLPOWERCOST" then
									if SN < "STALKERCANBLOCK" then
										if SN < "SONGRESIST" then
											if SN == "SKILLPOWERCOSTMOUNTED" then
												return CalcStat("CombatDamageModEnergy",L,0.64*N);
											else
												return 0;
											end
										elseif SN > "SONGRESIST" then
											if SN == "SONGRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("ResistAdd",L,N);
										end
									elseif SN > "STALKERCANBLOCK" then
										if SN < "STALKERCDCALCTYPENONPHYMIT" then
											if SN == "STALKERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "STALKERCDCALCTYPENONPHYMIT" then
											if SN == "STALKERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return 1;
									end
								else
									if Lm <= 140 then
										return CalcStat("CombatDamageModEnergy",L,0.64*N);
									else
										return CalcStat("CombatDamageModEnergy",L,1.05*0.64*N);
									end
								end
							else
								return 1;
							end
						else
							return 25;
						end
					elseif SN > "STALKERCDHASPOWER" then
						if SN < "T2PENARMOUR" then
							if SN < "STOUTAXERDTRAITDISEASERESISTP" then
								if SN < "STDPOWERH" then
									if SN < "STDMORALEL" then
										if SN > "STATC" then
											if SN == "STDMORALEH" then
												if Lm <= 105 then
													return RoundDbl(2*CalcStat("StdMoraleRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleH",105));
												end
											else
												return 0;
											end
										elseif SN == "STATC" then
											return CalcStat(C,L);
										else
											return 0;
										end
									elseif SN > "STDMORALEL" then
										if SN < "STDMORALERAW" then
											if SN == "STDMORALEM" then
												if Lm <= 105 then
													return RoundDbl(1.5*CalcStat("StdMoraleRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleM",105));
												end
											else
												return 0;
											end
										elseif SN > "STDMORALERAW" then
											if SN == "STDPNTS" then
												return {{1,50,60,65,75,85,95,100,105,106,115,116,120,121,130,131,140,141,150,151,160},{1,50,60,65,75,85,95,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,7.51,227.59,1,50,L);
											else
												return LinFmod(1,227.59,1327.5,50,105,L);
											end
										end
									else
										if Lm <= 105 then
											return RoundDbl(CalcStat("StdMoraleRaw",L));
										else
											return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleL",105));
										end
									end
								elseif SN > "STDPOWERH" then
									if SN < "STOUTAXERDPSVONENAME" then
										if SN < "STDPOWERRAW" then
											if SN == "STDPOWERL" then
												if Lm <= 105 then
													return RoundDbl(CalcStat("StdPowerRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerL",105));
												end
											else
												return 0;
											end
										elseif SN > "STDPOWERRAW" then
											if SN == "STDPROG" then
												if Lm <= 50 then
													return LinFmod(N,1,10,1,50,L,"P");
												elseif Lm <= 60 then
													return LinFmod(CalcStat("StdProg",50,N),10/10,15/10,50,60,L,"P");
												elseif Lm <= 65 then
													return LinFmod(CalcStat("StdProg",60,N),15/15,20/15,60,65,L,"P");
												elseif Lm <= 75 then
													return LinFmod(CalcStat("StdProg",65,N),20/20,30/20,65,75,L,"P");
												elseif Lm <= 85 then
													return LinFmod(CalcStat("StdProg",75,N),30/30,45/30,75,85,L,"P");
												elseif Lm <= 95 then
													return LinFmod(CalcStat("StdProg",85,N),45/45,65/45,85,95,L,"P");
												elseif Lm <= 100 then
													return LinFmod(CalcStat("StdProg",95,N),65/65,90.27/65,95,100,L,"P");
												elseif Lm <= 105 then
													return LinFmod(CalcStat("StdProg",100,N),90.27/90.27,120.24/90.27,100,105,L,"P");
												elseif Lm <= 115 then
													return LinFmod(CalcStat("StdProg",105,N),1.1,1.5,106,115,L,"P");
												elseif Lm <= 120 then
													return LinFmod(CalcStat("StdProg",115,N),1.15,1.25,116,120,L,"P");
												elseif Lm <= 130 then
													return LinFmod(CalcStat("StdProg",120,N),1.15,1.5,121,130,L,"P");
												elseif Lm <= 140 then
													return LinFmod(CalcStat("StdProg",130,N),1.15,2,131,140,L,"P");
												elseif Lm <= 150 then
													return LinFmod(CalcStat("StdProg",140,N),1.3,2.205,141,150,L,"P");
												else
													return LinFmod(CalcStat("StdProg",150,N),1,2,151,160,L,"P");
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,3,91.0375,1,50,L);
											else
												return LinFmod(1,91.0375,421,50,105,L);
											end
										end
									elseif SN > "STOUTAXERDPSVONENAME" then
										if SN < "STOUTAXERDPSVTWONAME" then
											if SN == "STOUTAXERDPSVONEVITALITY" then
												return CalcStat("StoutUnwritDestVitality",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDPSVTWONAME" then
											if SN == "STOUTAXERDTRAITAGILITY" then
												return CalcStat("StoutWrBlackLAgility",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return "Unwritten Destiny";
									end
								else
									if Lm <= 105 then
										return RoundDbl(2*CalcStat("StdPowerRaw",L));
									else
										return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerH",105));
									end
								end
							elseif SN > "STOUTAXERDTRAITDISEASERESISTP" then
								if SN < "STOUTSHADOWEYEVITALITY" then
									if SN < "STOUTAXERDTRAITSHADOWMITP" then
										if SN < "STOUTAXERDTRAITMIGHT" then
											if SN == "STOUTAXERDTRAITFATE" then
												return CalcStat("StoutDoomDrasaFate",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDTRAITMIGHT" then
											if SN == "STOUTAXERDTRAITPHYMITP" then
												return CalcStat("StoutUnyieldingPhyMitP",L);
											else
												return 0;
											end
										else
											return CalcStat("StoutWrBlackLMight",L);
										end
									elseif SN > "STOUTAXERDTRAITSHADOWMITP" then
										if SN < "STOUTAXERDTRAITWILL" then
											if SN == "STOUTAXERDTRAITVITALITY" then
												return CalcStat("StoutShadowEyeVitality",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDTRAITWILL" then
											if SN == "STOUTDOOMDRASAFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("StoutUnyieldingWill",L);
										end
									else
										return CalcStat("StoutWrBlackLShadowMitP",L);
									end
								elseif SN > "STOUTSHADOWEYEVITALITY" then
									if SN < "STOUTWRBLACKLAGILITY" then
										if SN < "STOUTUNYIELDINGPHYMITP" then
											if SN == "STOUTUNWRITDESTVITALITY" then
												return CalcStat("VitalityT",L,1.0);
											else
												return 0;
											end
										elseif SN > "STOUTUNYIELDINGPHYMITP" then
											if SN == "STOUTUNYIELDINGWILL" then
												return CalcStat("WillT",L,1.0);
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "STOUTWRBLACKLAGILITY" then
										if SN < "STOUTWRBLACKLMIGHT" then
											if SN == "STOUTWRBLACKLDISEASERESISTP" then
												return 1;
											else
												return 0;
											end
										elseif SN > "STOUTWRBLACKLMIGHT" then
											if SN == "STOUTWRBLACKLSHADOWMITP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,1.0);
										end
									else
										return CalcStat("AgilityT",L,1.0);
									end
								else
									return -CalcStat("VitalityT",L,0.4);
								end
							else
								return CalcStat("StoutWrBlackLDiseaseResistP",L);
							end
						elseif SN > "T2PENARMOUR" then
							if SN < "TACMIT" then
								if SN < "TACDMGPRATPB" then
									if SN < "TACDMGPBONUS" then
										if SN < "T2PENMIT" then
											if SN == "T2PENBPE" then
												if Lm <= 115 then
													return (-40)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenBPE",115));
												end
											else
												return 0;
											end
										elseif SN > "T2PENMIT" then
											if SN == "T2PENRESIST" then
												if Lm <= 115 then
													return (-90)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenResist",115));
												end
											else
												return 0;
											end
										else
											if Lm <= 115 then
												return RoundDblDown(L*13.5)*-5;
											else
												return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenMit",115));
											end
										end
									elseif SN > "TACDMGPBONUS" then
										if SN < "TACDMGPRATP" then
											if SN == "TACDMGPPRAT" then
												return CalcStat("OutDmgPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "TACDMGPRATP" then
											if SN == "TACDMGPRATPA" then
												return CalcStat("OutDmgPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatP",L,N);
										end
									else
										return CalcStat("OutDmgPBonus",L);
									end
								elseif SN > "TACDMGPRATPB" then
									if SN < "TACMAS" then
										if SN < "TACDMGPRATPCAP" then
											if SN == "TACDMGPRATPC" then
												return CalcStat("OutDmgPRatPC",L);
											else
												return 0;
											end
										elseif SN > "TACDMGPRATPCAP" then
											if SN == "TACDMGPRATPCAPR" then
												return CalcStat("OutDmgPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatPCap",L);
										end
									elseif SN > "TACMAS" then
										if SN < "TACMASOLD" then
											if SN == "TACMASC" then
												return CalcStat("MasteryC",L,N);
											else
												return 0;
											end
										elseif SN > "TACMASOLD" then
											if SN == "TACMAST" then
												return CalcStat("MasteryT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("Mastery",L,N);
										end
									else
										return CalcStat("Mastery",L,N);
									end
								else
									return CalcStat("OutDmgPRatPB",L);
								end
							elseif SN > "TACMIT" then
								if SN < "TACMITHPRATPCAP" then
									if SN < "TACMITHPRATP" then
										if SN < "TACMITHPBONUS" then
											if SN == "TACMITC" then
												return StatLinInter("PntMPTacMitC","CreepPntS","ProgBMitigationC","",L,N,0);
											else
												return 0;
											end
										elseif SN > "TACMITHPBONUS" then
											if SN == "TACMITHPPRAT" then
												return CalcStat("MitHeavyPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPBonus",L);
										end
									elseif SN > "TACMITHPRATP" then
										if SN < "TACMITHPRATPB" then
											if SN == "TACMITHPRATPA" then
												return CalcStat("MitHeavyPRatPA",L);
											else
												return 0;
											end
										elseif SN > "TACMITHPRATPB" then
											if SN == "TACMITHPRATPC" then
												return CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPB",L);
										end
									else
										return CalcStat("MitHeavyPRatP",L,N);
									end
								elseif SN > "TACMITHPRATPCAP" then
									if SN < "TACMITLPRATP" then
										if SN < "TACMITLPBONUS" then
											if SN == "TACMITHPRATPCAPR" then
												return CalcStat("MitHeavyPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "TACMITLPBONUS" then
											if SN == "TACMITLPPRAT" then
												return CalcStat("MitLightPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPBonus",L);
										end
									elseif SN > "TACMITLPRATP" then
										if SN < "TACMITLPRATPB" then
											if SN == "TACMITLPRATPA" then
												return CalcStat("MitLightPRatPA",L);
											else
												return 0;
											end
										elseif SN > "TACMITLPRATPB" then
											if SN == "TACMITLPRATPC" then
												return CalcStat("MitLightPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatPB",L);
										end
									else
										return CalcStat("MitLightPRatP",L,N);
									end
								else
									return CalcStat("MitHeavyPRatPCap",L);
								end
							else
								return StatLinInter("PntMPTacMit","ItemPntS","ProgBMitigation","AdjUmbarItemMit",L,N,0);
							end
						else
							return CalcStat("T2penMit",L);
						end
					else
						return 1;
					end
				else
					return CalcStat("RivHobSwimmerFrostMitP",L);
				end
			else
				return 27;
			end
		elseif SN > "TACMITLPRATPCAP" then
			if SN < "WARDENCDPHYMITTOCOMPHYMIT" then
				if SN < "VIRTWISDOMVPPHYMAS" then
					if SN < "VIRTDETERMINATIONVPTACMAS" then
						if SN < "TRAIT12347CHOICE" then
							if SN < "TOMEMAIN" then
								if SN < "TACMITMPRATPC" then
									if SN < "TACMITMPPRAT" then
										if SN > "TACMITLPRATPCAPR" then
											if SN == "TACMITMPBONUS" then
												return CalcStat("MitMediumPBonus",L);
											else
												return 0;
											end
										elseif SN == "TACMITLPRATPCAPR" then
											return CalcStat("MitLightPRatPCapR",L);
										else
											return 0;
										end
									elseif SN > "TACMITMPPRAT" then
										if SN < "TACMITMPRATPA" then
											if SN == "TACMITMPRATP" then
												return CalcStat("MitMediumPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "TACMITMPRATPA" then
											if SN == "TACMITMPRATPB" then
												return CalcStat("MitMediumPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPA",L);
										end
									else
										return CalcStat("MitMediumPPRat",L,N);
									end
								elseif SN > "TACMITMPRATPC" then
									if SN < "TACRESIST" then
										if SN < "TACMITMPRATPCAPR" then
											if SN == "TACMITMPRATPCAP" then
												return CalcStat("MitMediumPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "TACMITMPRATPCAPR" then
											if SN == "TACMITT" then
												return StatLinInter("PntMPTacMit","TraitPntS","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPCapR",L);
										end
									elseif SN > "TACRESIST" then
										if SN < "TOMEFATE" then
											if SN == "TACRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										elseif SN > "TOMEFATE" then
											if SN == "TOMEFATEDEC" then
												return CalcStat("TomeTotalFateDec",L)-CalcStat("TomeTotalFateDec",L-1);
											else
												return 0;
											end
										else
											return CalcStat("TomeFateDec",RomanRankDecode(C));
										end
									else
										return CalcStat("ResistAdd",L,N);
									end
								else
									return CalcStat("MitMediumPRatPC",L);
								end
							elseif SN > "TOMEMAIN" then
								if SN < "TOMETOTALVITALITYDEC" then
									if SN < "TOMETOTALLEVEL" then
										if SN < "TOMETOTALFATE" then
											if SN == "TOMEMAINDEC" then
												return CalcStat("TomeTotalMainDec",L)-CalcStat("TomeTotalMainDec",L-1);
											else
												return 0;
											end
										elseif SN > "TOMETOTALFATE" then
											if SN == "TOMETOTALFATEDEC" then
												return CalcStat("FateT",CalcStat("TomeTotalLevel",L),2.0);
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalFateDec",RomanRankDecode(C));
										end
									elseif SN > "TOMETOTALLEVEL" then
										if SN < "TOMETOTALMAINDEC" then
											if SN == "TOMETOTALMAIN" then
												return CalcStat("TomeTotalMainDec",RomanRankDecode(C));
											else
												return 0;
											end
										elseif SN > "TOMETOTALMAINDEC" then
											if SN == "TOMETOTALVITALITY" then
												return CalcStat("TomeTotalVitalityDec",RomanRankDecode(C));
											else
												return 0;
											end
										else
											return CalcStat("MainT",CalcStat("TomeTotalLevel",L),2.0);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({4,15,27,38,48,54,60,63,67,71,75,78,85,90,95,98,101,104,106,110,114,116,121},L);
										end
									end
								elseif SN > "TOMETOTALVITALITYDEC" then
									if SN < "TPENBPE" then
										if SN < "TOMEVITALITYDEC" then
											if SN == "TOMEVITALITY" then
												return CalcStat("TomeVitalityDec",RomanRankDecode(C));
											else
												return 0;
											end
										elseif SN > "TOMEVITALITYDEC" then
											if SN == "TPENARMOUR" then
												return -CalcStat("ArmourPenT",L,CalcStat("TpenChoice",N));
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalVitalityDec",L)-CalcStat("TomeTotalVitalityDec",L-1);
										end
									elseif SN > "TPENBPE" then
										if SN < "TPENRESIST" then
											if SN == "TPENCHOICE" then
												return DataTableValue({0.5,1,2},L);
											else
												return 0;
											end
										elseif SN > "TPENRESIST" then
											if SN == "TRAIT12345CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,5},L);
												end
											else
												return 0;
											end
										else
											return -CalcStat("ResistT",L,CalcStat("TpenChoice",N)*2);
										end
									else
										return -CalcStat("BPET",L,CalcStat("TpenChoice",N));
									end
								else
									return CalcStat("VitalityT",CalcStat("TomeTotalLevel",L),2.0);
								end
							else
								return CalcStat("TomeMainDec",RomanRankDecode(C));
							end
						elseif SN > "TRAIT12347CHOICE" then
							if SN < "VIRTCHARITYRESIST" then
								if SN < "TRAIT567810CHOICE" then
									if SN < "TRAIT23456CHOICE" then
										if SN < "TRAIT123CHOICE" then
											if SN == "TRAIT1234CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT123CHOICE" then
											if SN == "TRAIT13510CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,3,5,10},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,2,3},L);
											end
										end
									elseif SN > "TRAIT23456CHOICE" then
										if SN < "TRAIT357912CHOICE" then
											if SN == "TRAIT234CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({2,3,4},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT357912CHOICE" then
											if SN == "TRAIT47101316CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({4,7,10,13,16},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({3,5,7,9,12},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({2,3,4,5,6},L);
										end
									end
								elseif SN > "TRAIT567810CHOICE" then
									if SN < "TRAITPNTSVITAL" then
										if SN < "TRAIT58121620CHOICE" then
											if SN == "TRAIT56789CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({5,6,7,8,9},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT58121620CHOICE" then
											if SN == "TRAITPNTS" then
												return {{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141,150},{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141,150}};
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({5,8,12,16,20},L);
											end
										end
									elseif SN > "TRAITPNTSVITAL" then
										if SN < "VARMOUR" then
											if SN == "U371LEGACYSTATFIX" then
												if Lm <= 2 then
													return 0;
												else
													return RoundDblUp(CalcStat(C,L-2)*0.07);
												end
											else
												return 0;
											end
										elseif SN > "VARMOUR" then
											if SN == "VIRTCHARITYPHYMIT" then
												return CalcStat("VSPhyMitM",L);
											else
												return 0;
											end
										else
											return RoundDbl(StatLinInter("PntMPArmourVirtues","ItemPntS","ProgBArmourLight","ArmourAdjTypeOther",L,N,0));
										end
									else
										return {{1,25,50,60,65,75,85,95,100,105,115,120,130,140,141,150},{1,25,50,60,65,75,85,95,100,105,115,120,130,140,141,150}};
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({5,6,7,8,10},L);
									end
								end
							elseif SN > "VIRTCHARITYRESIST" then
								if SN < "VIRTCONFIDENCEEVADE" then
									if SN < "VIRTCOMPASSIONPHYMIT" then
										if SN < "VIRTCHARITYVPMORALE" then
											if SN == "VIRTCHARITYVITALITY" then
												return CalcStat("VSVitalityL",L);
											else
												return 0;
											end
										elseif SN > "VIRTCHARITYVPMORALE" then
											if SN == "VIRTCOMPASSIONARMOUR" then
												return CalcStat("VSArmourL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTCOMPASSIONPHYMIT" then
										if SN < "VIRTCOMPASSIONVPMORALE" then
											if SN == "VIRTCOMPASSIONTACMIT" then
												return CalcStat("VSTacMitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTCOMPASSIONVPMORALE" then
											if SN == "VIRTCONFIDENCECRITHIT" then
												return CalcStat("VSCritHitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSPhyMitH",L);
									end
								elseif SN > "VIRTCONFIDENCEEVADE" then
									if SN < "VIRTDETERMINATIONAGILITY" then
										if SN < "VIRTCONFIDENCEVPPHYMAS" then
											if SN == "VIRTCONFIDENCEFINESSE" then
												return CalcStat("VSFinesseM",L);
											else
												return 0;
											end
										elseif SN > "VIRTCONFIDENCEVPPHYMAS" then
											if SN == "VIRTCONFIDENCEVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPPhyMas",L);
										end
									elseif SN > "VIRTDETERMINATIONAGILITY" then
										if SN < "VIRTDETERMINATIONPHYMAS" then
											if SN == "VIRTDETERMINATIONCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTDETERMINATIONPHYMAS" then
											if SN == "VIRTDETERMINATIONVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPhyMasM",L);
										end
									else
										return CalcStat("VSAgilityH",L);
									end
								else
									return CalcStat("VSEvadeL",L);
								end
							else
								return CalcStat("VSResistH",L);
							end
						else
							if Lm <= 0 then
								return 0;
							else
								return DataTableValue({1,2,3,4,7},L);
							end
						end
					elseif SN > "VIRTDETERMINATIONVPTACMAS" then
						if SN < "VIRTINNOCENCERESIST" then
							if SN < "VIRTFORTITUDERESIST" then
								if SN < "VIRTEMPATHYRESIST" then
									if SN < "VIRTDISCIPLINERESIST" then
										if SN > "VIRTDISCIPLINEINHEAL" then
											if SN == "VIRTDISCIPLINEPHYMIT" then
												return CalcStat("VSPhyMitL",L);
											else
												return 0;
											end
										elseif SN == "VIRTDISCIPLINEINHEAL" then
											return CalcStat("VSInHealM",L);
										else
											return 0;
										end
									elseif SN > "VIRTDISCIPLINERESIST" then
										if SN < "VIRTEMPATHYARMOUR" then
											if SN == "VIRTDISCIPLINEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTEMPATHYARMOUR" then
											if SN == "VIRTEMPATHYCRITDEF" then
												return CalcStat("VSCritDefM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSArmourH",L);
										end
									else
										return CalcStat("VSResistH",L);
									end
								elseif SN > "VIRTEMPATHYRESIST" then
									if SN < "VIRTFIDELITYVITALITY" then
										if SN < "VIRTFIDELITYPHYMIT" then
											if SN == "VIRTEMPATHYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTFIDELITYPHYMIT" then
											if SN == "VIRTFIDELITYTACMIT" then
												return CalcStat("VSTacMitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPhyMitL",L);
										end
									elseif SN > "VIRTFIDELITYVITALITY" then
										if SN < "VIRTFORTITUDECRITDEF" then
											if SN == "VIRTFIDELITYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTFORTITUDECRITDEF" then
											if SN == "VIRTFORTITUDEMORALE" then
												return CalcStat("VSMoraleH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritDefM",L);
										end
									else
										return CalcStat("VSVitalityM",L);
									end
								else
									return CalcStat("VSResistL",L);
								end
							elseif SN > "VIRTFORTITUDERESIST" then
								if SN < "VIRTHONOURMORALE" then
									if SN < "VIRTHONESTYVPPHYMAS" then
										if SN < "VIRTHONESTYCRITHIT" then
											if SN == "VIRTFORTITUDEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONESTYCRITHIT" then
											if SN == "VIRTHONESTYTACMAS" then
												return CalcStat("VSTacMasH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									elseif SN > "VIRTHONESTYVPPHYMAS" then
										if SN < "VIRTHONESTYWILL" then
											if SN == "VIRTHONESTYVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONESTYWILL" then
											if SN == "VIRTHONOURCRITDEF" then
												return CalcStat("VSCritDefL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSWillM",L);
										end
									else
										return CalcStat("VSVPPhyMas",L);
									end
								elseif SN > "VIRTHONOURMORALE" then
									if SN < "VIRTIDEALISMINHEAL" then
										if SN < "VIRTHONOURVPMORALE" then
											if SN == "VIRTHONOURTACMIT" then
												return CalcStat("VSTacMitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONOURVPMORALE" then
											if SN == "VIRTIDEALISMFATE" then
												return CalcStat("VSFateH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTIDEALISMINHEAL" then
										if SN < "VIRTIDEALISMVPMORALE" then
											if SN == "VIRTIDEALISMMORALE" then
												return CalcStat("VSMoraleL",L);
											else
												return 0;
											end
										elseif SN > "VIRTIDEALISMVPMORALE" then
											if SN == "VIRTINNOCENCEPHYMIT" then
												return CalcStat("VSPhyMitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSInHealM",L);
									end
								else
									return CalcStat("VSMoraleH",L);
								end
							else
								return CalcStat("VSResistL",L);
							end
						elseif SN > "VIRTINNOCENCERESIST" then
							if SN < "VIRTPATIENCEEVADE" then
								if SN < "VIRTLOYALTYINHEAL" then
									if SN < "VIRTJUSTICEMORALE" then
										if SN < "VIRTINNOCENCEVPMORALE" then
											if SN == "VIRTINNOCENCETACMIT" then
												return CalcStat("VSTacMitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTINNOCENCEVPMORALE" then
											if SN == "VIRTJUSTICEICMR" then
												return CalcStat("VSICMRH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTJUSTICEMORALE" then
										if SN < "VIRTJUSTICEVPMORALE" then
											if SN == "VIRTJUSTICETACMIT" then
												return CalcStat("VSTacMitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTJUSTICEVPMORALE" then
											if SN == "VIRTLOYALTYARMOUR" then
												return CalcStat("VSArmourM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSMoraleM",L);
									end
								elseif SN > "VIRTLOYALTYINHEAL" then
									if SN < "VIRTMERCYFATE" then
										if SN < "VIRTLOYALTYVPMORALE" then
											if SN == "VIRTLOYALTYVITALITY" then
												return CalcStat("VSVitalityH",L);
											else
												return 0;
											end
										elseif SN > "VIRTLOYALTYVPMORALE" then
											if SN == "VIRTMERCYEVADE" then
												return CalcStat("VSEvadeH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTMERCYFATE" then
										if SN < "VIRTMERCYVPMORALE" then
											if SN == "VIRTMERCYVITALITY" then
												return CalcStat("VSVitalityL",L);
											else
												return 0;
											end
										elseif SN > "VIRTMERCYVPMORALE" then
											if SN == "VIRTPATIENCECRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSFateM",L);
									end
								else
									return CalcStat("VSInHealL",L);
								end
							elseif SN > "VIRTPATIENCEEVADE" then
								if SN < "VIRTTOLERANCEVPMORALE" then
									if SN < "VIRTRNKCOSTTOT" then
										if SN < "VIRTPATIENCEVPMORALE" then
											if SN == "VIRTPATIENCEPOWER" then
												return CalcStat("VSPowerH",L);
											else
												return 0;
											end
										elseif SN > "VIRTPATIENCEVPMORALE" then
											if SN == "VIRTRNKCOST" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 10 then
													return 1000;
												elseif Lm <= 60 then
													return RoundDbl(18*L+878,-2);
												elseif Lm <= 73 then
													return RoundDbl(18.75*L+878,-2);
												elseif Lm <= 90 then
													return RoundDbl(17.45*L+878,-2);
												else
													return 2500;
												end
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTRNKCOSTTOT" then
										if SN < "VIRTTOLERANCERESIST" then
											if SN == "VIRTTOLERANCEPHYMIT" then
												return CalcStat("VSPhyMitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTTOLERANCERESIST" then
											if SN == "VIRTTOLERANCETACMIT" then
												return CalcStat("VSTacMitH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSResistM",L);
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return CalcStat("VirtRnkCostTot",L-1)+CalcStat("VirtRnkCost",L);
										end
									end
								elseif SN > "VIRTTOLERANCEVPMORALE" then
									if SN < "VIRTVALOURVPPHYMAS" then
										if SN < "VIRTVALOURFINESSE" then
											if SN == "VIRTVALOURCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										elseif SN > "VIRTVALOURFINESSE" then
											if SN == "VIRTVALOURPHYMAS" then
												return CalcStat("VSPhyMasH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSFinesseM",L);
										end
									elseif SN > "VIRTVALOURVPPHYMAS" then
										if SN < "VIRTWISDOMFINESSE" then
											if SN == "VIRTVALOURVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTWISDOMFINESSE" then
											if SN == "VIRTWISDOMTACMAS" then
												return CalcStat("VSTacMasM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSFinesseL",L);
										end
									else
										return CalcStat("VSVPPhyMas",L);
									end
								else
									return CalcStat("VSVPMorale",L);
								end
							else
								return CalcStat("VSEvadeM",L);
							end
						else
							return CalcStat("VSResistM",L);
						end
					else
						return CalcStat("VSVPTacMas",L);
					end
				elseif SN > "VIRTWISDOMVPPHYMAS" then
					if SN < "VSMORALEL" then
						if SN < "VRNKTOILVL" then
							if SN < "VITALITYADJ" then
								if SN < "VIRTWITVPPHYMAS" then
									if SN < "VIRTWITCRITHIT" then
										if SN > "VIRTWISDOMVPTACMAS" then
											if SN == "VIRTWISDOMWILL" then
												return CalcStat("VSWillH",L);
											else
												return 0;
											end
										elseif SN == "VIRTWISDOMVPTACMAS" then
											return CalcStat("VSVPTacMas",L);
										else
											return 0;
										end
									elseif SN > "VIRTWITCRITHIT" then
										if SN < "VIRTWITPHYMAS" then
											if SN == "VIRTWITFINESSE" then
												return CalcStat("VSFinesseH",L);
											else
												return 0;
											end
										elseif SN > "VIRTWITPHYMAS" then
											if SN == "VIRTWITTACMAS" then
												return CalcStat("VSTacMasL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPhyMasL",L);
										end
									else
										return CalcStat("VSCritHitM",L);
									end
								elseif SN > "VIRTWITVPPHYMAS" then
									if SN < "VIRTZEALPHYMAS" then
										if SN < "VIRTZEALCRITHIT" then
											if SN == "VIRTWITVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTZEALCRITHIT" then
											if SN == "VIRTZEALMIGHT" then
												return CalcStat("VSMightH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									elseif SN > "VIRTZEALPHYMAS" then
										if SN < "VIRTZEALVPTACMAS" then
											if SN == "VIRTZEALVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTZEALVPTACMAS" then
											if SN == "VITALITY" then
												return RoundDblDown(StatLinInter("PntMPVitality","ItemPntSVital","ProgBVitality","VitalityAdj",L,N,0));
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									else
										return CalcStat("VSPhyMasM",L);
									end
								else
									return CalcStat("VSVPPhyMas",L);
								end
							elseif SN > "VITALITYADJ" then
								if SN < "VMLOW" then
									if SN < "VMASTERY" then
										if SN < "VITALITYT" then
											if SN == "VITALITYC" then
												return RoundDblDown(StatLinInter("PntMPVitalityC","CreepPntS","ProgBVitalityC","",L,N,0));
											else
												return 0;
											end
										elseif SN > "VITALITYT" then
											if SN == "VITALITYTADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 65 then
													return 0.8;
												elseif Lm <= 75 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPVitalityT","TraitPntSVital","ProgBVitality","VitalityTAdj",L,N,0));
										end
									elseif SN > "VMASTERY" then
										if SN < "VMASTERYOLD" then
											if SN == "VMASTERYADJ" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 350 then
													return 1;
												elseif Lm <= 399 then
													return 988/900;
												elseif Lm <= 450 then
													return 1;
												elseif Lm <= 499 then
													return 0.9;
												elseif Lm <= 550 then
													return 741/900;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "VMASTERYOLD" then
											if SN == "VMHIGH" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat(C,CalcStat("VRnkToILvl",L),2.0);
												end
											else
												return 0;
											end
										else
											return CalcStat("VMastery",L,N);
										end
									else
										return StatLinInter("PntMPMastery","ItemPntSVirtueMastery","ProgBMastery","VMasteryAdj",L,N,0);
									end
								elseif SN > "VMLOW" then
									if SN < "VMORALE" then
										if SN < "VMMEDIUM" then
											if SN == "VMMASTERYPSV" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat("VMastery",CalcStat("VRnkToILvl",L),0.2);
												end
											else
												return 0;
											end
										elseif SN > "VMMEDIUM" then
											if SN == "VMMORALEPSV" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat("VMorale",CalcStat("VRnkToILvl",L),0.3);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return CalcStat(C,CalcStat("VRnkToILvl",L),1.0);
											end
										end
									elseif SN > "VMORALE" then
										if SN < "VRNKCAP" then
											if SN == "VMORALEADJ" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 2 then
													return 1;
												elseif Lm <= 50 then
													return 2;
												elseif Lm <= 80 then
													return 1.5;
												elseif Lm <= 450 then
													return 1;
												elseif Lm <= 499 then
													return 0.9475;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "VRNKCAP" then
											if SN == "VRNKLVLCAP" then
												if Lm <= 4 then
													return 2;
												elseif Lm <= 110 then
													return RoundDblDown(L/2);
												elseif Lm <= 139 then
													return L-55;
												elseif Lm <= 140 then
													return L-54;
												elseif Lm <= 149 then
													return L-53;
												else
													return 98;
												end
											else
												return 0;
											end
										else
											return 92;
										end
									else
										return StatLinInter("PntMPMoraleVirtues","ItemPntSVirtueMorale","ProgBMorale","VMoraleAdj",L,N,0);
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return CalcStat(C,CalcStat("VRnkToILvl",L),0.6);
									end
								end
							else
								if Lm <= 25 then
									return 0.5;
								elseif Lm <= 50 then
									return 0.6;
								elseif Lm <= 60 then
									return 0.7;
								elseif Lm <= 79 then
									return 0.8;
								elseif Lm <= 80 then
									return 0.9;
								else
									return 1;
								end
							end
						elseif SN > "VRNKTOILVL" then
							if SN < "VSFATEH" then
								if SN < "VSCRITDEFL" then
									if SN < "VSARMOURH" then
										if SN < "VSAGILITYL" then
											if SN == "VSAGILITYH" then
												return CalcStat("VMHigh",L,"Agility");
											else
												return 0;
											end
										elseif SN > "VSAGILITYL" then
											if SN == "VSAGILITYM" then
												return CalcStat("VMMedium",L,"Agility");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Agility");
										end
									elseif SN > "VSARMOURH" then
										if SN < "VSARMOURM" then
											if SN == "VSARMOURL" then
												return CalcStat("VMLow",L,"Varmour");
											else
												return 0;
											end
										elseif SN > "VSARMOURM" then
											if SN == "VSCRITDEFH" then
												return CalcStat("VMHigh",L,"CritDef");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Varmour");
										end
									else
										return CalcStat("VMHigh",L,"Varmour");
									end
								elseif SN > "VSCRITDEFL" then
									if SN < "VSCRITHITM" then
										if SN < "VSCRITHITH" then
											if SN == "VSCRITDEFM" then
												return CalcStat("VMMedium",L,"CritDef");
											else
												return 0;
											end
										elseif SN > "VSCRITHITH" then
											if SN == "VSCRITHITL" then
												return CalcStat("VMLow",L,"CritHit");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"CritHit");
										end
									elseif SN > "VSCRITHITM" then
										if SN < "VSEVADEL" then
											if SN == "VSEVADEH" then
												return CalcStat("VMHigh",L,"Evade");
											else
												return 0;
											end
										elseif SN > "VSEVADEL" then
											if SN == "VSEVADEM" then
												return CalcStat("VMMedium",L,"Evade");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Evade");
										end
									else
										return CalcStat("VMMedium",L,"CritHit");
									end
								else
									return CalcStat("VMLow",L,"CritDef");
								end
							elseif SN > "VSFATEH" then
								if SN < "VSICMRM" then
									if SN < "VSFINESSEL" then
										if SN < "VSFATEM" then
											if SN == "VSFATEL" then
												return CalcStat("VMLow",L,"Fate");
											else
												return 0;
											end
										elseif SN > "VSFATEM" then
											if SN == "VSFINESSEH" then
												return CalcStat("VMHigh",L,"Finesse");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Fate");
										end
									elseif SN > "VSFINESSEL" then
										if SN < "VSICMRH" then
											if SN == "VSFINESSEM" then
												return CalcStat("VMMedium",L,"Finesse");
											else
												return 0;
											end
										elseif SN > "VSICMRH" then
											if SN == "VSICMRL" then
												return CalcStat("VMLow",L,"ICMR");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"ICMR");
										end
									else
										return CalcStat("VMLow",L,"Finesse");
									end
								elseif SN > "VSICMRM" then
									if SN < "VSMIGHTH" then
										if SN < "VSINHEALL" then
											if SN == "VSINHEALH" then
												return CalcStat("VMHigh",L,"InHeal");
											else
												return 0;
											end
										elseif SN > "VSINHEALL" then
											if SN == "VSINHEALM" then
												return CalcStat("VMMedium",L,"InHeal");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"InHeal");
										end
									elseif SN > "VSMIGHTH" then
										if SN < "VSMIGHTM" then
											if SN == "VSMIGHTL" then
												return CalcStat("VMLow",L,"Might");
											else
												return 0;
											end
										elseif SN > "VSMIGHTM" then
											if SN == "VSMORALEH" then
												return CalcStat("VMHigh",L,"VMorale");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Might");
										end
									else
										return CalcStat("VMHigh",L,"Might");
									end
								else
									return CalcStat("VMMedium",L,"ICMR");
								end
							else
								return CalcStat("VMHigh",L,"Fate");
							end
						else
							if Lm <= 0 then
								return 0;
							elseif Lm <= 38 then
								return LinFmod(1,4,78,1,38,L);
							elseif Lm <= 48 then
								return LinFmod(1,78,178,38,48,L);
							elseif Lm <= 49 then
								return LinFmod(1,178,190,48,49,L);
							elseif Lm <= 50 then
								return LinFmod(1,190,210,49,50,L);
							elseif Lm <= 51 then
								return LinFmod(1,210,222,50,51,L);
							elseif Lm <= 52 then
								return LinFmod(1,222,236,51,52,L);
							elseif Lm <= 53 then
								return LinFmod(1,236,260,52,53,L);
							elseif Lm <= 55 then
								return LinFmod(1,260,292,53,55,L);
							elseif Lm <= 68 then
								return LinFmod(1,292,396,55,68,L);
							else
								return LinFmod(1,396,706,68,130,L);
							end
						end
					elseif SN > "VSMORALEL" then
						if SN < "WARDENCDAGILITYTOOUTHEAL" then
							if SN < "VSTACMASL" then
								if SN < "VSPHYMITM" then
									if SN < "VSPHYMASL" then
										if SN > "VSMORALEM" then
											if SN == "VSPHYMASH" then
												return CalcStat("VMHigh",L,"PhyMas");
											else
												return 0;
											end
										elseif SN == "VSMORALEM" then
											return CalcStat("VMMedium",L,"VMorale");
										else
											return 0;
										end
									elseif SN > "VSPHYMASL" then
										if SN < "VSPHYMITH" then
											if SN == "VSPHYMASM" then
												return CalcStat("VMMedium",L,"PhyMas");
											else
												return 0;
											end
										elseif SN > "VSPHYMITH" then
											if SN == "VSPHYMITL" then
												return CalcStat("VMLow",L,"PhyMit");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"PhyMit");
										end
									else
										return CalcStat("VMLow",L,"PhyMas");
									end
								elseif SN > "VSPHYMITM" then
									if SN < "VSRESISTH" then
										if SN < "VSPOWERL" then
											if SN == "VSPOWERH" then
												return CalcStat("VMHigh",L,"Power");
											else
												return 0;
											end
										elseif SN > "VSPOWERL" then
											if SN == "VSPOWERM" then
												return CalcStat("VMMedium",L,"Power");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Power");
										end
									elseif SN > "VSRESISTH" then
										if SN < "VSRESISTM" then
											if SN == "VSRESISTL" then
												return CalcStat("VMLow",L,"Resist");
											else
												return 0;
											end
										elseif SN > "VSRESISTM" then
											if SN == "VSTACMASH" then
												return CalcStat("VMHigh",L,"TacMas");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Resist");
										end
									else
										return CalcStat("VMHigh",L,"Resist");
									end
								else
									return CalcStat("VMMedium",L,"PhyMit");
								end
							elseif SN > "VSTACMASL" then
								if SN < "VSVPMORALE" then
									if SN < "VSTACMITM" then
										if SN < "VSTACMITH" then
											if SN == "VSTACMASM" then
												return CalcStat("VMMedium",L,"TacMas");
											else
												return 0;
											end
										elseif SN > "VSTACMITH" then
											if SN == "VSTACMITL" then
												return CalcStat("VMLow",L,"TacMit");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"TacMit");
										end
									elseif SN > "VSTACMITM" then
										if SN < "VSVITALITYL" then
											if SN == "VSVITALITYH" then
												return CalcStat("VMHigh",L,"Vitality");
											else
												return 0;
											end
										elseif SN > "VSVITALITYL" then
											if SN == "VSVITALITYM" then
												return CalcStat("VMMedium",L,"Vitality");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Vitality");
										end
									else
										return CalcStat("VMMedium",L,"TacMit");
									end
								elseif SN > "VSVPMORALE" then
									if SN < "VSWILLL" then
										if SN < "VSVPTACMAS" then
											if SN == "VSVPPHYMAS" then
												return CalcStat("VMMasteryPsv",L);
											else
												return 0;
											end
										elseif SN > "VSVPTACMAS" then
											if SN == "VSWILLH" then
												return CalcStat("VMHigh",L,"Will");
											else
												return 0;
											end
										else
											return CalcStat("VMMasteryPsv",L);
										end
									elseif SN > "VSWILLL" then
										if SN < "WARDENCDAGILITYTOBLOCK" then
											if SN == "VSWILLM" then
												return CalcStat("VMMedium",L,"Will");
											else
												return 0;
											end
										elseif SN > "WARDENCDAGILITYTOBLOCK" then
											if SN == "WARDENCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return CalcStat("VMLow",L,"Will");
									end
								else
									return CalcStat("VMMoralePsv",L);
								end
							else
								return CalcStat("VMLow",L,"TacMas");
							end
						elseif SN > "WARDENCDAGILITYTOOUTHEAL" then
							if SN < "WARDENCDBASENCPR" then
								if SN < "WARDENCDARMOURTYPE" then
									if SN < "WARDENCDAGILITYTOTACMIT" then
										if SN < "WARDENCDAGILITYTOPHYMAS" then
											if SN == "WARDENCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDAGILITYTOPHYMAS" then
											if SN == "WARDENCDAGILITYTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "WARDENCDAGILITYTOTACMIT" then
										if SN < "WARDENCDARMOURTONONPHYMIT" then
											if SN == "WARDENCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDARMOURTONONPHYMIT" then
											if SN == "WARDENCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "WARDENCDARMOURTYPE" then
									if SN < "WARDENCDBASEICPR" then
										if SN < "WARDENCDBASEFATE" then
											if SN == "WARDENCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityH",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEFATE" then
											if SN == "WARDENCDBASEICMR" then
												return CalcStat("ClassBaseICMRH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									elseif SN > "WARDENCDBASEICPR" then
										if SN < "WARDENCDBASEMORALE" then
											if SN == "WARDENCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEMORALE" then
											if SN == "WARDENCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								else
									return 2;
								end
							elseif SN > "WARDENCDBASENCPR" then
								if SN < "WARDENCDFATETONCPR" then
									if SN < "WARDENCDCALCTYPECOMPHYMIT" then
										if SN < "WARDENCDBASEVITALITY" then
											if SN == "WARDENCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEVITALITY" then
											if SN == "WARDENCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									elseif SN > "WARDENCDCALCTYPECOMPHYMIT" then
										if SN < "WARDENCDCALCTYPETACMIT" then
											if SN == "WARDENCDCALCTYPENONPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "WARDENCDCALCTYPETACMIT" then
											if SN == "WARDENCDCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return 26;
										end
									else
										return 13;
									end
								elseif SN > "WARDENCDFATETONCPR" then
									if SN < "WARDENCDMIGHTTOCRITHIT" then
										if SN < "WARDENCDHASPOWER" then
											if SN == "WARDENCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDHASPOWER" then
											if SN == "WARDENCDMIGHTTOBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "WARDENCDMIGHTTOCRITHIT" then
										if SN < "WARDENCDMIGHTTOOUTHEAL" then
											if SN == "WARDENCDMIGHTTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "WARDENCDMIGHTTOOUTHEAL" then
											if SN == "WARDENCDMIGHTTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1.5;
									end
								else
									return 0.07;
								end
							else
								return CalcStat("ClassBaseNCPR",L);
							end
						else
							return 3;
						end
					else
						return CalcStat("VMLow",L,"VMorale");
					end
				else
					return CalcStat("VSVPPhyMas",L);
				end
			elseif SN > "WARDENCDPHYMITTOCOMPHYMIT" then
				if SN < "WORTHTABN" then
					if SN < "WORTHTABAU" then
						if SN < "WORTHMPB" then
							if SN < "WARLEADERCDCALCTYPECOMPHYMIT" then
								if SN < "WARDENCDWILLTOOUTHEAL" then
									if SN < "WARDENCDVITALITYTOICMR" then
										if SN > "WARDENCDPHYMITTONONPHYMIT" then
											if SN == "WARDENCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN == "WARDENCDPHYMITTONONPHYMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "WARDENCDVITALITYTOICMR" then
										if SN < "WARDENCDVITALITYTONCMR" then
											if SN == "WARDENCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "WARDENCDVITALITYTONCMR" then
											if SN == "WARDENCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 0.012;
									end
								elseif SN > "WARDENCDWILLTOOUTHEAL" then
									if SN < "WARDENCDWILLTOTACMIT" then
										if SN < "WARDENCDWILLTOPHYMIT" then
											if SN == "WARDENCDWILLTOPHYMAS" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDWILLTOPHYMIT" then
											if SN == "WARDENCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1.5;
										end
									elseif SN > "WARDENCDWILLTOTACMIT" then
										if SN < "WARDINGLORETACMIT" then
											if SN == "WARDINGLOREPHYMIT" then
												if Lm <= 105 then
													return CalcStat("Mitigation",L,1.6);
												elseif Lm <= 119 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 120 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 129 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 130 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 139 then
													return CalcStat("PhyMitT",L,1.2);
												else
													return CalcStat("PhyMitT",L,1.6);
												end
											else
												return 0;
											end
										elseif SN > "WARDINGLORETACMIT" then
											if SN == "WARLEADERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return CalcStat("Mitigation",L,1.6);
											elseif Lm <= 119 then
												return CalcStat("TacMitT",L,1.2);
											elseif Lm <= 120 then
												return CalcStat("TacMitT",L,1.6);
											elseif Lm <= 129 then
												return CalcStat("TacMitT",L,1.2);
											elseif Lm <= 130 then
												return CalcStat("TacMitT",L,1.6);
											elseif Lm <= 139 then
												return CalcStat("TacMitT",L,1.2);
											else
												return CalcStat("TacMitT",L,1.6);
											end
										end
									else
										return 1.5;
									end
								else
									return 1;
								end
							elseif SN > "WARLEADERCDCALCTYPECOMPHYMIT" then
								if SN < "WEAVERCDHASPOWER" then
									if SN < "WEAVERCANBLOCK" then
										if SN < "WARLEADERCDCALCTYPETACMIT" then
											if SN == "WARLEADERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "WARLEADERCDCALCTYPETACMIT" then
											if SN == "WARLEADERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "WEAVERCANBLOCK" then
										if SN < "WEAVERCDCALCTYPENONPHYMIT" then
											if SN == "WEAVERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "WEAVERCDCALCTYPENONPHYMIT" then
											if SN == "WEAVERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return 1;
									end
								elseif SN > "WEAVERCDHASPOWER" then
									if SN < "WORTHEXT" then
										if SN < "WILLC" then
											if SN == "WILL" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "WILLC" then
											if SN == "WILLT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MainC",L,N);
										end
									elseif SN > "WORTHEXT" then
										if SN < "WORTHEXT8LIN" then
											if SN == "WORTHEXT4LIN" then
												if Lm <= 501 then
													return CalcStat("WorthExt",L,C);
												elseif Lm <= 601 then
													return CalcStat("WorthExt",501,C)+(L-501)*4;
												else
													return 0;
												end
											else
												return 0;
											end
										elseif SN > "WORTHEXT8LIN" then
											if SN == "WORTHMPA" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.1,1.15,1.2,1.3},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 501 then
												return CalcStat("WorthExt",L,C);
											elseif Lm <= 601 then
												return CalcStat("WorthExt",501,C)+(L-501)*8;
											else
												return 0;
											end
										end
									else
										if Lm <= 360 then
											return RoundDbl(CalcStat("StatC",L,C));
										elseif Lm <= 601 then
											return RoundDbl(CalcStat("StatC",L-1,C));
										else
											return 0;
										end
									end
								else
									return 1;
								end
							else
								return 14;
							end
						elseif SN > "WORTHMPB" then
							if SN < "WORTHTABAF" then
								if SN < "WORTHMPJ" then
									if SN < "WORTHMPF" then
										if SN < "WORTHMPD" then
											if SN == "WORTHMPC" then
												if Lm <= 0 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "WORTHMPD" then
											if SN == "WORTHMPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.1,1.15,1.2,1.25},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,1,1,2,3},L);
											end
										end
									elseif SN > "WORTHMPF" then
										if SN < "WORTHMPH" then
											if SN == "WORTHMPG" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,5},L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHMPH" then
											if SN == "WORTHMPI" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,2.5,3,10},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,1.2,1.8,3.2,5},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({0.5,1,1.25,1.5,2},L);
										end
									end
								elseif SN > "WORTHMPJ" then
									if SN < "WORTHTABAB" then
										if SN < "WORTHTABA" then
											if SN == "WORTHMPK" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.2,1.3,1.35,1.4},L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABA" then
											if SN == "WORTHTABAA" then
												return CalcStat("WorthTabD",L)+20;
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 1;
											else
												return CalcStat("WorthTabAF",L);
											end
										end
									elseif SN > "WORTHTABAB" then
										if SN < "WORTHTABAD" then
											if SN == "WORTHTABAC" then
												return CalcStat("WorthTabE",L)-30;
											else
												return 0;
											end
										elseif SN > "WORTHTABAD" then
											if SN == "WORTHTABAE" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										else
											return 7*L+100;
										end
									else
										return 7*L+50;
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({1,1,1,1,5},L);
									end
								end
							elseif SN > "WORTHTABAF" then
								if SN < "WORTHTABAM" then
									if SN < "WORTHTABAJ" then
										if SN < "WORTHTABAH" then
											if SN == "WORTHTABAG" then
												return CalcStat("WorthTabAF",L)+25;
											else
												return 0;
											end
										elseif SN > "WORTHTABAH" then
											if SN == "WORTHTABAI" then
												if Lm <= 16 then
													return 10.73*L+54.3;
												elseif Lm <= 34 then
													return 10.735*L+54;
												elseif Lm <= 35 then
													return 429;
												elseif Lm <= 41 then
													return 10.65*L+57.1;
												elseif Lm <= 49 then
													return 10.7*L+55.3;
												else
													return 9*L+141;
												end
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L+7)-66;
											else
												return 9*L+72;
											end
										end
									elseif SN > "WORTHTABAJ" then
										if SN < "WORTHTABAL" then
											if SN == "WORTHTABAK" then
												if Lm <= 49 then
													return CalcStat("WorthTabALBase",L+1);
												else
													return 9*L+78;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAL" then
											if SN == "WORTHTABALBASE" then
												return 9.86*L+23.51+RoundDbl(L*0.1+0.3)*0.4;
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L);
											else
												return 9*L+69;
											end
										end
									else
										return CalcStat("WorthTabAH",L);
									end
								elseif SN > "WORTHTABAM" then
									if SN < "WORTHTABAQ" then
										if SN < "WORTHTABAO" then
											if SN == "WORTHTABAN" then
												if Lm <= 50 then
													return 8.25*L+27.75;
												else
													return 8*L+40;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAO" then
											if SN == "WORTHTABAP" then
												return CalcStat("WorthTabAQ",L);
											else
												return 0;
											end
										else
											return CalcStat("WorthTabE",L);
										end
									elseif SN > "WORTHTABAQ" then
										if SN < "WORTHTABAS" then
											if SN == "WORTHTABAR" then
												return CalcStat("WorthTabD",L)+37.5;
											else
												return 0;
											end
										elseif SN > "WORTHTABAS" then
											if SN == "WORTHTABAT" then
												if Lm <= 4 then
													return RoundDbl(3*L+20,-1);
												elseif Lm <= 7 then
													return 60;
												elseif Lm <= 10 then
													return 100;
												elseif Lm <= 13 then
													return 180;
												elseif Lm <= 16 then
													return 270;
												elseif Lm <= 19 then
													return 310;
												elseif Lm <= 40 then
													return RoundDbl(3.2*L+274,-1);
												elseif Lm <= 43 then
													return 400;
												elseif Lm <= 53 then
													return RoundDbl(1.9*L+334,-1);
												elseif Lm <= 56 then
													return 460;
												elseif Lm <= 59 then
													return 480;
												else
													return 20*L-680;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabU",L);
										end
									else
										if Lm <= 10 then
											return 7.5*L+60;
										elseif Lm <= 49 then
											return 5.5*L+80;
										else
											return 6*L+55;
										end
									end
								else
									return CalcStat("WorthTabB",L);
								end
							else
								if Lm <= 49 then
									return 2.5*L;
								else
									return 3*L-25;
								end
							end
						else
							if Lm <= 0 then
								return 0;
							else
								return DataTableValue({1,1.2,2,3,4},L);
							end
						end
					elseif SN > "WORTHTABAU" then
						if SN < "WORTHTABBX" then
							if SN < "WORTHTABBH" then
								if SN < "WORTHTABB" then
									if SN < "WORTHTABAX" then
										if SN > "WORTHTABAV" then
											if SN == "WORTHTABAW" then
												if Lm <= 1 then
													return 50;
												elseif Lm <= 49 then
													return 2.5*L+102.5;
												elseif Lm <= 80 then
													return 3*L+78;
												elseif Lm <= 120 then
													return 2.97*L+79.5;
												else
													return 3*L+75;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABAV" then
											if Lm <= 4 then
												return RoundDbl(3*L+20,-1);
											elseif Lm <= 7 then
												return 70;
											elseif Lm <= 10 then
												return 140;
											elseif Lm <= 13 then
												return 270;
											elseif Lm <= 16 then
												return 400;
											elseif Lm <= 19 then
												return 460;
											elseif Lm <= 25 then
												return RoundDbl(3*L+447.5,-1);
											elseif Lm <= 46 then
												return RoundDbl(3.2*L+455,-1);
											elseif Lm <= 50 then
												return 620;
											else
												return 20*L-360;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABAX" then
										if SN < "WORTHTABAZ" then
											if SN == "WORTHTABAY" then
												return 5*L;
											else
												return 0;
											end
										elseif SN > "WORTHTABAZ" then
											if SN == "WORTHTABAZBASE" then
												if Lm <= 44 then
													return RoundDbl(0.0525*L+0.7);
												elseif Lm <= 65 then
													return RoundDbl(0.19*L-4.85);
												elseif Lm <= 81 then
													return RoundDbl(0.19*L+28.15);
												else
													return RoundDbl(0.19*L+28.15+RoundDbl(L*0.05-4.55)*0.2);
												end
											else
												return 0;
											end
										else
											if Lm <= 10 then
												return CalcStat("WorthTabAZBase",L)*62.5;
											elseif Lm <= 65 then
												return CalcStat("WorthTabAZBase",L)*125;
											else
												return CalcStat("WorthTabAZBase",L)*25;
											end
										end
									else
										if Lm <= 16 then
											return 10.75*L+54;
										elseif Lm <= 34 then
											return 10.75*L+53.7;
										elseif Lm <= 35 then
											return 429;
										elseif Lm <= 41 then
											return 10.65*L+57.1;
										elseif Lm <= 49 then
											return 10.68*L+56.25;
										else
											return 12*L-9;
										end
									end
								elseif SN > "WORTHTABB" then
									if SN < "WORTHTABBD" then
										if SN < "WORTHTABBB" then
											if SN == "WORTHTABBA" then
												return 5*L+385;
											else
												return 0;
											end
										elseif SN > "WORTHTABBB" then
											if SN == "WORTHTABBC" then
												if Lm <= 34 then
													return RoundDbl(L/15+0.8)*3750+1250;
												elseif Lm <= 35 then
													return 100000;
												else
													return 125500;
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 40;
											elseif Lm <= 10 then
												return RoundDbl(0.35*L-0.1)*20;
											elseif Lm <= 13 then
												return 100;
											elseif Lm <= 19 then
												return RoundDbl(0.35*L+1.6)*20;
											elseif Lm <= 25 then
												return RoundDbl(2*L+150,-1);
											elseif Lm <= 40 then
												return RoundDbl(2*L+164,-1);
											elseif Lm <= 49 then
												return RoundDbl(L+201,-1);
											else
												return 20*L-740;
											end
										end
									elseif SN > "WORTHTABBD" then
										if SN < "WORTHTABBF" then
											if SN == "WORTHTABBE" then
												if Lm <= 1 then
													return 60;
												elseif Lm <= 7 then
													return RoundDbl(L/3)*40+30;
												elseif Lm <= 13 then
													return RoundDbl(L/3)*130-210;
												elseif Lm <= 19 then
													return RoundDbl(L/3)*60+140;
												elseif Lm <= 25 then
													return RoundDbl(L/3)*10+480;
												elseif Lm <= 46 then
													return RoundDbl(L/3)*10+490;
												elseif Lm <= 50 then
													return 660;
												else
													return 20*L-320;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBF" then
											if SN == "WORTHTABBG" then
												if Lm <= 7 then
													return RoundDbl(0.2*L-0.4)*2+1;
												elseif Lm <= 16 then
													return RoundDbl(L/3-2)*6;
												elseif Lm <= 22 then
													return RoundDbl(L/3+1)*3;
												elseif Lm <= 25 then
													return RoundDbl(L/3+1)*3-2;
												elseif Lm <= 80 then
													return RoundDbl(L/3+18)-2;
												else
													return L-37;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabAV",L)+20;
										end
									else
										return CalcStat("WorthTabK",L)+1;
									end
								else
									if Lm <= 50 then
										return 8.25*L+22.25;
									else
										return 8*L+35;
									end
								end
							elseif SN > "WORTHTABBH" then
								if SN < "WORTHTABBP" then
									if SN < "WORTHTABBL" then
										if SN < "WORTHTABBJ" then
											if SN == "WORTHTABBI" then
												return CalcStat("WorthTabG",L)-25;
											else
												return 0;
											end
										elseif SN > "WORTHTABBJ" then
											if SN == "WORTHTABBK" then
												if Lm <= 10 then
													return RoundDbl(0.1*L+0.45);
												else
													return RoundDbl(0.1*L-0.6)*2;
												end
											else
												return 0;
											end
										else
											return RoundDbl(0.1*L+0.45)*3;
										end
									elseif SN > "WORTHTABBL" then
										if SN < "WORTHTABBN" then
											if SN == "WORTHTABBM" then
												if Lm <= 1 then
													return 10;
												else
													return 20*L-20;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBN" then
											if SN == "WORTHTABBO" then
												if Lm <= 10 then
													return 10;
												elseif Lm <= 40 then
													return RoundDbl(0.1*L-0.55)*50-25;
												elseif Lm <= 80 then
													return RoundDbl(0.1*L-0.55)*50;
												else
													return RoundDbl(0.1*L-0.55)*25+175;
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 400;
											else
												return 20*L+800;
											end
										end
									else
										if Lm <= 35 then
											return 126600;
										elseif Lm <= 46 then
											return 150600;
										elseif Lm <= 47 then
											return 180720;
										else
											return 150600;
										end
									end
								elseif SN > "WORTHTABBP" then
									if SN < "WORTHTABBT" then
										if SN < "WORTHTABBR" then
											if SN == "WORTHTABBQ" then
												if Lm <= 1 then
													return 50;
												elseif Lm <= 49 then
													return RoundDbl(2.5*L+100);
												else
													return 3*L+75;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBR" then
											if SN == "WORTHTABBS" then
												return CalcStat("WorthTabBR",L)*2;
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabBQ",L)*2;
											else
												return 4*L+250;
											end
										end
									elseif SN > "WORTHTABBT" then
										if SN < "WORTHTABBV" then
											if SN == "WORTHTABBU" then
												if Lm <= 80 then
													return 20*L+300;
												else
													return 10*L+1100;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBV" then
											if SN == "WORTHTABBW" then
												if Lm <= 1 then
													return 645;
												elseif Lm <= 9 then
													return 20*L+980;
												else
													return 50*L+920;
												end
											else
												return 0;
											end
										else
											if Lm <= 29 then
												return 1250;
											else
												return RoundDbl(0.05*L-1)*2500;
											end
										end
									else
										return 62500;
									end
								else
									if Lm <= 10 then
										return 156.25*L+1562.5;
									elseif Lm <= 20 then
										return 312.5*L;
									elseif Lm <= 30 then
										return 625*L-6250;
									elseif Lm <= 40 then
										return 1250*L-25000;
									else
										return 2500*L-75000;
									end
								end
							else
								if Lm <= 80 then
									return RoundDbl(0.05*L-0.05)+RoundDbl(0.05*L+0.45);
								else
									return RoundDbl(0.05*L-0.1)+RoundDbl(0.05*L+0.45);
								end
							end
						elseif SN > "WORTHTABBX" then
							if SN < "WORTHTABCM" then
								if SN < "WORTHTABCE" then
									if SN < "WORTHTABCA" then
										if SN < "WORTHTABBZ" then
											if SN == "WORTHTABBY" then
												return 90000;
											else
												return 0;
											end
										elseif SN > "WORTHTABBZ" then
											if SN == "WORTHTABC" then
												return CalcStat("WorthTabD",L)+20;
											else
												return 0;
											end
										else
											if Lm <= 10 then
												return 27.42*L+273.1;
											elseif Lm <= 20 then
												return 54.69*L;
											elseif Lm <= 30 then
												return 109.35*L-1093;
											elseif Lm <= 40 then
												return 218.75*L-4375;
											elseif Lm <= 80 then
												return 437.5*L-13125;
											else
												return 437*L-13085;
											end
										end
									elseif SN > "WORTHTABCA" then
										if SN < "WORTHTABCC" then
											if SN == "WORTHTABCB" then
												return 0.1;
											else
												return 0;
											end
										elseif SN > "WORTHTABCC" then
											if SN == "WORTHTABCD" then
												return CalcStat("WorthTabBK",L)*25;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabBK",L)*5;
										end
									else
										if Lm <= 10 then
											return 7.25*L+72.25;
										elseif Lm <= 20 then
											return 14.49*L+0.11;
										elseif Lm <= 30 then
											return 29*L-290;
										elseif Lm <= 35 then
											return 57.99*L-1160.16;
										elseif Lm <= 40 then
											return 57.99*L-1160.12;
										elseif Lm <= 80 then
											return 115.94*L-3478.2;
										else
											return 116*L-3483;
										end
									end
								elseif SN > "WORTHTABCE" then
									if SN < "WORTHTABCI" then
										if SN < "WORTHTABCG" then
											if SN == "WORTHTABCF" then
												if Lm <= 20 then
													return RoundDbl(0.1*L-0.55)*600+200;
												elseif Lm <= 40 then
													return RoundDbl(0.1*L-0.55)*1000-500;
												elseif Lm <= 60 then
													return RoundDbl(0.1*L-0.55)*2000-4000;
												elseif Lm <= 70 then
													return 8500;
												elseif Lm <= 140 then
													return RoundDbl(0.1*L-0.55)*2500-7500;
												elseif Lm <= 200 then
													return RoundDbl(0.1*L-0.55)*3000-14000;
												else
													return RoundDbl(0.1*L-0.55)*2500-3500;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABCG" then
											if SN == "WORTHTABCH" then
												return 50000;
											else
												return 0;
											end
										else
											return 20;
										end
									elseif SN > "WORTHTABCI" then
										if SN < "WORTHTABCK" then
											if SN == "WORTHTABCJ" then
												return 20000;
											else
												return 0;
											end
										elseif SN > "WORTHTABCK" then
											if SN == "WORTHTABCL" then
												return 12500;
											else
												return 0;
											end
										else
											return 15000;
										end
									else
										return 40000;
									end
								else
									return CalcStat("WorthTabBN",L)*10;
								end
							elseif SN > "WORTHTABCM" then
								if SN < "WORTHTABF" then
									if SN < "WORTHTABCQ" then
										if SN < "WORTHTABCO" then
											if SN == "WORTHTABCN" then
												return RoundDbl(0.1*L+0.5)*10000;
											else
												return 0;
											end
										elseif SN > "WORTHTABCO" then
											if SN == "WORTHTABCP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabAB",L);
										end
									elseif SN > "WORTHTABCQ" then
										if SN < "WORTHTABD" then
											if SN == "WORTHTABCR" then
												return CalcStat("WorthTabM",L)-15;
											else
												return 0;
											end
										elseif SN > "WORTHTABD" then
											if SN == "WORTHTABE" then
												if Lm <= 9 then
													return 30*L+50;
												else
													return 7*L+280;
												end
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return 7.5*L;
											else
												return 8*L-25;
											end
										end
									else
										if Lm <= 10 then
											return RoundDbl(0.3*L+0.2)*2;
										elseif Lm <= 16 then
											return RoundDbl(0.25*L-1)*6;
										elseif Lm <= 22 then
											return RoundDbl(0.3*L+5.5)*2;
										elseif Lm <= 50 then
											return RoundDbl((1/6)*L+8.75)*2;
										elseif Lm <= 80 then
											return RoundDbl((1/3)*L-11.5)*6;
										else
											return RoundDbl(0.25*L-4.6)*6;
										end
									end
								elseif SN > "WORTHTABF" then
									if SN < "WORTHTABJ" then
										if SN < "WORTHTABH" then
											if SN == "WORTHTABG" then
												if Lm <= 1 then
													return 50;
												else
													return CalcStat("WorthTabAF",L)+100;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABH" then
											if SN == "WORTHTABI" then
												if Lm <= 4 then
													return 1;
												elseif Lm <= 10 then
													return 0.7*L-1;
												elseif Lm <= 15 then
													return 1.4*L-7;
												elseif Lm <= 24 then
													return 1.25*L-5;
												elseif Lm <= 49 then
													return 2.5*L-35;
												else
													return 3*L-60;
												end
											else
												return 0;
											end
										else
											if Lm <= 2 then
												return L;
											elseif Lm <= 9 then
												return 1.48*L-2.85;
											elseif Lm <= 11 then
												return 2.5*L-13;
											elseif Lm <= 24 then
												return 2.5*L-10;
											else
												return 5*L-70;
											end
										end
									elseif SN > "WORTHTABJ" then
										if SN < "WORTHTABL" then
											if SN == "WORTHTABK" then
												if Lm <= 9 then
													return 12*L+24;
												else
													return 7*L+74;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABL" then
											if SN == "WORTHTABM" then
												if Lm <= 9 then
													return 10*L+11;
												elseif Lm <= 49 then
													return 4*L+65;
												else
													return 5*L+15;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabB",L)+19.25;
										end
									else
										if Lm <= 4 then
											return 2.5*L+7.5;
										elseif Lm <= 5 then
											return 23;
										else
											return 4*L;
										end
									end
								else
									return 0.1;
								end
							else
								return 7500;
							end
						else
							if Lm <= 10 then
								return RoundDbl(0.1*L+0.45)*3;
							elseif Lm <= 140 then
								return RoundDbl(0.1*L-0.6)*6;
							else
								return RoundDbl(0.1*L+0.4)*4+22;
							end
						end
					else
						return CalcStat("WorthTabBN",L)*1.25;
					end
				elseif SN > "WORTHTABN" then
					if SN < "WORTHVALBW" then
						if SN < "WORTHVALAR" then
							if SN < "WORTHVALAB" then
								if SN < "WORTHTABU" then
									if SN < "WORTHTABQ" then
										if SN > "WORTHTABO" then
											if SN == "WORTHTABP" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										elseif SN == "WORTHTABO" then
											return CalcStat("WorthTabB",L)+11;
										else
											return 0;
										end
									elseif SN > "WORTHTABQ" then
										if SN < "WORTHTABS" then
											if SN == "WORTHTABR" then
												if Lm <= 9 then
													return 12*L+30;
												else
													return 7*L+75;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABS" then
											if SN == "WORTHTABT" then
												if Lm <= 1 then
													return 54;
												elseif Lm <= 17 then
													return 10.73*L+43.57;
												elseif Lm <= 35 then
													return 10.735*L+43.265;
												elseif Lm <= 36 then
													return 429;
												elseif Lm <= 42 then
													return 10.65*L+46.45;
												elseif Lm <= 49 then
													return 10.7*L+44.6;
												else
													return 9*L+130;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+17.5;
										end
									else
										return 7*L+25;
									end
								elseif SN > "WORTHTABU" then
									if SN < "WORTHTABY" then
										if SN < "WORTHTABW" then
											if SN == "WORTHTABV" then
												return CalcStat("WorthTabB",L)-2.75;
											else
												return 0;
											end
										elseif SN > "WORTHTABW" then
											if SN == "WORTHTABX" then
												return CalcStat("WorthTabAF",L)+75;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+25;
										end
									elseif SN > "WORTHTABY" then
										if SN < "WORTHVALA" then
											if SN == "WORTHTABZ" then
												return CalcStat("WorthTabR",L)-15;
											else
												return 0;
											end
										elseif SN > "WORTHVALA" then
											if SN == "WORTHVALAA" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAA"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabA"));
										end
									else
										return CalcStat("WorthTabD",L)+30;
									end
								else
									if Lm <= 10 then
										return 9*L+20;
									elseif Lm <= 49 then
										return 5.5*L+55;
									else
										return 6*L+30;
									end
								end
							elseif SN > "WORTHVALAB" then
								if SN < "WORTHVALAJ" then
									if SN < "WORTHVALAF" then
										if SN < "WORTHVALAD" then
											if SN == "WORTHVALAC" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAC"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAD" then
											if SN == "WORTHVALAE" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAE"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAD"));
										end
									elseif SN > "WORTHVALAF" then
										if SN < "WORTHVALAH" then
											if SN == "WORTHVALAG" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAG"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAH" then
											if SN == "WORTHVALAI" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAI"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAH"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAF"));
									end
								elseif SN > "WORTHVALAJ" then
									if SN < "WORTHVALAN" then
										if SN < "WORTHVALAL" then
											if SN == "WORTHVALAK" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAK"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAL" then
											if SN == "WORTHVALAM" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAM"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAL"));
										end
									elseif SN > "WORTHVALAN" then
										if SN < "WORTHVALAP" then
											if SN == "WORTHVALAO" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAO"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAP" then
											if SN == "WORTHVALAQ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAQ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAP"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAN"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAJ"));
								end
							else
								return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAB"));
							end
						elseif SN > "WORTHVALAR" then
							if SN < "WORTHVALBG" then
								if SN < "WORTHVALAZ" then
									if SN < "WORTHVALAV" then
										if SN < "WORTHVALAT" then
											if SN == "WORTHVALAS" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAS"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAT" then
											if SN == "WORTHVALAU" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAU"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAT"));
										end
									elseif SN > "WORTHVALAV" then
										if SN < "WORTHVALAX" then
											if SN == "WORTHVALAW" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAW"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAX" then
											if SN == "WORTHVALAY" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabAY"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAX"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAV"));
									end
								elseif SN > "WORTHVALAZ" then
									if SN < "WORTHVALBC" then
										if SN < "WORTHVALBA" then
											if SN == "WORTHVALB" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabB"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBA" then
											if SN == "WORTHVALBB" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBB"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBA"));
										end
									elseif SN > "WORTHVALBC" then
										if SN < "WORTHVALBE" then
											if SN == "WORTHVALBD" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBD"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBE" then
											if SN == "WORTHVALBF" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBF"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpJ",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBE"));
										end
									else
										return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBC"));
									end
								else
									return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabAZ"));
								end
							elseif SN > "WORTHVALBG" then
								if SN < "WORTHVALBO" then
									if SN < "WORTHVALBK" then
										if SN < "WORTHVALBI" then
											if SN == "WORTHVALBH" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBH"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBI" then
											if SN == "WORTHVALBJ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBJ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBI"));
										end
									elseif SN > "WORTHVALBK" then
										if SN < "WORTHVALBM" then
											if SN == "WORTHVALBL" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBL"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBM" then
											if SN == "WORTHVALBN" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBN"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBM"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBK"));
									end
								elseif SN > "WORTHVALBO" then
									if SN < "WORTHVALBS" then
										if SN < "WORTHVALBQ" then
											if SN == "WORTHVALBP" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBP"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBQ" then
											if SN == "WORTHVALBR" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBR"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBQ"));
										end
									elseif SN > "WORTHVALBS" then
										if SN < "WORTHVALBU" then
											if SN == "WORTHVALBT" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBT"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBU" then
											if SN == "WORTHVALBV" then
												return RoundDbl(CalcStat("WorthMpD",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBV"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpF",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBU"));
										end
									else
										return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBS"));
									end
								else
									return RoundDbl(CalcStat("WorthMpG",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBO"));
								end
							else
								return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBG"));
							end
						else
							return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAR"));
						end
					elseif SN > "WORTHVALBW" then
						if SN < "WORTHVALM" then
							if SN < "WORTHVALCL" then
								if SN < "WORTHVALCD" then
									if SN < "WORTHVALC" then
										if SN < "WORTHVALBY" then
											if SN == "WORTHVALBX" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBX"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBY" then
											if SN == "WORTHVALBZ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBZ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBY"));
										end
									elseif SN > "WORTHVALC" then
										if SN < "WORTHVALCB" then
											if SN == "WORTHVALCA" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCA"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCB" then
											if SN == "WORTHVALCC" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCC"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCB"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabC"));
									end
								elseif SN > "WORTHVALCD" then
									if SN < "WORTHVALCH" then
										if SN < "WORTHVALCF" then
											if SN == "WORTHVALCE" then
												return RoundDbl(CalcStat("WorthMpH",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCE"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCF" then
											if SN == "WORTHVALCG" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCG"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCF"));
										end
									elseif SN > "WORTHVALCH" then
										if SN < "WORTHVALCJ" then
											if SN == "WORTHVALCI" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCI"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCJ" then
											if SN == "WORTHVALCK" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCK"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCJ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCH"));
									end
								else
									return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCD"));
								end
							elseif SN > "WORTHVALCL" then
								if SN < "WORTHVALE" then
									if SN < "WORTHVALCP" then
										if SN < "WORTHVALCN" then
											if SN == "WORTHVALCM" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCM"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCN" then
											if SN == "WORTHVALCO" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCO"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpI",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCN"));
										end
									elseif SN > "WORTHVALCP" then
										if SN < "WORTHVALCR" then
											if SN == "WORTHVALCQ" then
												return RoundDbl(CalcStat("WorthMpK",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCQ"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCR" then
											if SN == "WORTHVALD" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabD"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCR"));
										end
									else
										return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCP"));
									end
								elseif SN > "WORTHVALE" then
									if SN < "WORTHVALI" then
										if SN < "WORTHVALG" then
											if SN == "WORTHVALF" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabF"));
											else
												return 0;
											end
										elseif SN > "WORTHVALG" then
											if SN == "WORTHVALH" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabH"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabG"));
										end
									elseif SN > "WORTHVALI" then
										if SN < "WORTHVALK" then
											if SN == "WORTHVALJ" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabJ"));
											else
												return 0;
											end
										elseif SN > "WORTHVALK" then
											if SN == "WORTHVALL" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabL"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabK"));
										end
									else
										return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabI"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabE"));
								end
							else
								return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCL"));
							end
						elseif SN > "WORTHVALM" then
							if SN < "WPNDMGMAX" then
								if SN < "WORTHVALU" then
									if SN < "WORTHVALQ" then
										if SN < "WORTHVALO" then
											if SN == "WORTHVALN" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabN"));
											else
												return 0;
											end
										elseif SN > "WORTHVALO" then
											if SN == "WORTHVALP" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabP"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabO"));
										end
									elseif SN > "WORTHVALQ" then
										if SN < "WORTHVALS" then
											if SN == "WORTHVALR" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabR"));
											else
												return 0;
											end
										elseif SN > "WORTHVALS" then
											if SN == "WORTHVALT" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabT"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabS"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabQ"));
									end
								elseif SN > "WORTHVALU" then
									if SN < "WORTHVALY" then
										if SN < "WORTHVALW" then
											if SN == "WORTHVALV" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabV"));
											else
												return 0;
											end
										elseif SN > "WORTHVALW" then
											if SN == "WORTHVALX" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabX"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabW"));
										end
									elseif SN > "WORTHVALY" then
										if SN < "WOUNDRESIST" then
											if SN == "WORTHVALZ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabZ"));
											else
												return 0;
											end
										elseif SN > "WOUNDRESIST" then
											if SN == "WOUNDRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("ResistAdd",L,N);
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabY"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabU"));
								end
							elseif SN > "WPNDMGMAX" then
								if SN < "WRDPHYMAS" then
									if SN < "WRDBATSTRIKESCRITDEF" then
										if SN < "WPNDPS" then
											if SN == "WPNDMGMIN" then
												return ((2-2*CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2)))/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
											else
												return 0;
											end
										elseif SN > "WPNDPS" then
											if SN == "WPNDPSVARIANCETYPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.25,0.25,0.25},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("CombatBase",L,C);
										end
									elseif SN > "WRDBATSTRIKESCRITDEF" then
										if SN < "WRDFINESSE" then
											if SN == "WRDCRITDEF" then
												return CalcStat("CritDefT",L,1.0);
											else
												return 0;
											end
										elseif SN > "WRDFINESSE" then
											if SN == "WRDIMPRBLADESPARRY" then
												return CalcStat("ParryT",L,2.8);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
										end
									else
										return CalcStat("CritDefT",L,CalcStat("Trait123Choice",N)*0.4);
									end
								elseif SN > "WRDPHYMAS" then
									if SN < "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSHIELDMASBLOCK" then
											if SN == "WRDRECKLESSNCRITHIT" then
												return CalcStat("CritHitT",L,2.4);
											else
												return 0;
											end
										elseif SN > "WRDSHIELDMASBLOCK" then
											if SN == "WRDSHIELDTACTCRITDEF" then
												return CalcStat("CritDefT",L,2);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,2.8);
										end
									elseif SN > "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRBLOCK" then
												return CalcStat("BlockT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRPARRY" then
												return CalcStat("ParryT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										else
											return -CalcStat("EvadeT",L,CalcStat("Trait1234Choice",N)*0.4);
										end
									else
										return -CalcStat("BlockT",L,4);
									end
								else
									return CalcStat("PhyMasT",L,CalcStat("Trait12345Choice",N)*0.4);
								end
							else
								return (2/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
							end
						else
							return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabM"));
						end
					else
						return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBW"));
					end
				else
					return CalcStat("WorthTabAF",L)+25;
				end
			else
				return 1;
			end
		else
			return CalcStat("MitLightPRatPCap",L);
		end
	else
		return 2;
	end

end

-- Support functions for CalcStat. These consist of implementations of more complex calculation types, decode methods for parameter "C" and rounding/min/max/compare functions for floating point numbers.

-- ****************** Calculation Type support functions ******************

-- DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
-- DataTableValue: Takes a value from an array table.

function DataTableValue(vDataArray, dIndex)

	local lIndex = RoundDbl(dIndex);
	if lIndex <= 1 then
		return vDataArray[1];
	elseif lIndex > #vDataArray then
		return vDataArray[#vDataArray];
	else
		return vDataArray[lIndex];
	end

end

-- EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
-- ExpFmod: Exponential function based on percentage.
-- Common percentage values are around ~5.5% for between levels and ~20% jumps between level segments.

function ExpFmod(dVal, dLstart, dPlvl, dLvl, vDec, vAdd)

	local dRng = dLvl-dLstart+1;
	if dRng <= DblCalcDev then
		return dVal;
	else
		local dFac = 1+dPlvl/100;
		local dAdd;
		if vAdd == nil then
			dAdd = 0;
		else
			dAdd = vAdd;
		end
		if vDec == nil then
			local dFacExp = dFac^dRng;
			return dVal*dFacExp+dAdd*((dFacExp-1)/(dFac-1));
		else
			local dResult = dVal;
			local dLm = dLstart-DblCalcDev;
			while dLm <= dLvl do
				dResult = RoundDbl(dResult*dFac+dAdd,vDec);
				dLm = dLm+1;
			end
			return dResult;
		end
	end

end

-- PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
-- CalcPercAB: Calculates the percentage out of a rating based on the AB formula.

function CalcPercAB(dA, dB, dPCap, dR)

	if dR <= DblCalcDev then
		return 0.0;
	else
		local dResult = dA/(1+dB/dR);
		if dResult >= dPCap-DblCalcDev then
			return dPCap;
		else
			return dResult;
		end
	end

end

-- RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
-- CalcRatAB: Calculates the rating out of a percentage based on the AB formula.

function CalcRatAB(dA, dB, dCapR, dP)

	if dP <= DblCalcDev then
		return 0.0;
	else
		local dResult = dB/(dA/dP-1);
		if dResult >= dCapR-DblCalcDev then
			return dCapR;
		else
			return dResult;
		end
	end

end

-- SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS
-- StatLinInter: (Normalized) Stat Linear Interpolating

function StatLinInter(sPntMP, sProgScheme, sProgBase, sAdj, dLvl, vNorC, vRoundType)

	-- parameter processing
	local dN = 1;
	local sC = "";
	if type(vNorC) == "number" then
		dN = vNorC;
	elseif type(vNorC) == "string" then
		sC = vNorC;
	end

	local dRoundType = 0;
	if type(vRoundType) == "number" then
		dRoundType = vRoundType;
	end
	
	local dProgScheme = CalcStat(sProgScheme,dLvl);
	if type(dProgScheme) ~= "table" then
		return 0.0;
	end

	-- find level interval
	local dLvlMinus = dLvl-DblCalcDev;
	local iPointIndexHigh = 2;
	local iPointIndexMax = #dProgScheme[1];
	while iPointIndexHigh < iPointIndexMax do
		if dLvlMinus <= dProgScheme[1][iPointIndexHigh] then
			break;
		end
		iPointIndexHigh = iPointIndexHigh+1;
	end
	local iPointIndexLow = iPointIndexHigh-1;
		
	local dAccessLvlLow = dProgScheme[1][iPointIndexLow];
	local dAccessLvlHigh = dProgScheme[1][iPointIndexHigh];
	local dBaseLvlLow = dProgScheme[2][iPointIndexLow];
	local dBaseLvlHigh = dProgScheme[2][iPointIndexHigh];
	
	-- get values from base progression
	local dValLow = CalcStat(sProgBase,dBaseLvlLow,sC);
	local dValHigh = CalcStat(sProgBase,dBaseLvlHigh,sC);

	-- graph point multiplications
	if type(sPntMP) == "string" and sPntMP ~= "" then
		dValLow = dValLow*CalcStat(sPntMP,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sPntMP,dAccessLvlHigh,sC);
	end
	if type(sAdj) == "string" and sAdj ~= "" then
		dValLow = dValLow*CalcStat(sAdj,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sAdj,dAccessLvlHigh,sC);
	end
	dValLow = dValLow*dN;
	dValHigh = dValHigh*dN;

	-- graph point roundings
	if dRoundType == 0 then
		dValLow = RoundDblLotro(dValLow);
		dValHigh = RoundDblLotro(dValHigh);
	elseif dRoundType == 1 then
		if -100.0 <= dValLow and dValLow <= 100.0 then
			dValLow = RoundDblUp(dValLow,2);
		elseif -1000.0 <= dValLow and dValLow <= 1000.0 then
			dValLow = RoundDblUp(dValLow,1);
		else
			dValLow = RoundDblLotro(dValLow);
		end
		if -100.0 <= dValHigh and dValHigh <= 100.0 then
			dValHigh = RoundDblUp(dValHigh,2);
		elseif -1000.0 <= dValHigh and dValHigh <= 1000.0 then
			dValHigh = RoundDblUp(dValHigh,1);
		else
			dValHigh = RoundDblLotro(dValHigh);
		end
	elseif dRoundType == 2 then
		dValLow = RoundDblLotro(dValLow);
		if dValLow == -1 then
			dValLow = -2;
		end
		dValHigh = RoundDblLotro(dValHigh);
		if dValHigh == -1 then
			dValHigh = -2;
		end
	elseif dRoundType == 3 then
		dValLow = RoundDblProg(dValLow);
		dValHigh = RoundDblProg(dValHigh);
	end

	-- return interpolated value from the calculated graph points
	return LinFmod(1,dValLow,dValHigh,dAccessLvlLow,dAccessLvlHigh,dLvl);

end

-- TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
-- LinFmod: Linear line function between 2 points with some optional modifications.
-- Connects point (dLstart,dVal*dFstart) with (dLend,dVal*dFend).
-- Usually used with dVal=1 and dFstart/dFend containing unrelated points or dVal=# and dFstart/dFend containing multiplier factors.
-- Modification for in-between points on the line: rounding.

function LinFmod(dVal, dFstart, dFend, dLstart, dLend, dLvl, vDec)

	if type(vDec) == "string" then
		local sRoundType = string.upper(vDec);
		if sRoundType == "P" then
			return LinFmod(1,RoundDblProg(dVal*dFstart),RoundDblProg(dVal*dFend),dLstart,dLend,dLvl);
		elseif sRoundType == "L" then
			return LinFmod(1,RoundDblLotro(dVal*dFstart),RoundDblLotro(dVal*dFend),dLstart,dLend,dLvl);
		else
			return LinFmod(1,dVal*dFstart,dVal*dFend,dLstart,dLend,dLvl);
		end
	end
	if dLstart-DblCalcDev <= dLvl and dLvl <= dLstart+DblCalcDev then
		return dVal*dFstart;
	elseif dLend-DblCalcDev <= dLvl and dLvl <= dLend+DblCalcDev then
		return dVal*dFend;
	elseif dLstart == dLend then
		return 0.0;
	elseif vDec == nil then
		return dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart);
	else
		return RoundDbl(dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart),vDec);
	end

end

-- ****************** Parameter "C" decode support functions ******************

-- ArmCodeIndex: returns a specified index from an Armour Code.
-- sACode string:
-- 1st position: H=heavy, M=medium, L=light
-- 2nd position: H=head, S=shoulders, CL=cloak/back, C=chest, G=gloves, L=leggings, B=boots, Sh=shield
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic
-- Note: no such thing exists as a heavy, medium or light cloak, so no H/M/L in cloak codes (cloaks go automatically in the M class since U23, although historically this was L)

function ArmCodeIndex(sACode, iI)

	if sACode == nil or iI == nil then
		return 0;
	end
	if type(sACode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sArmCode = string.match(sACode,"(%a+)");
	if not sArmCode then
		return 0;
	end
	sArmCode = string.upper(sArmCode).."    ";
	
	local sArmCat = string.sub(sArmCode,1,1);
	local sArmType = string.sub(sArmCode,2,2);
	local sArmCol = string.sub(sArmCode,3,3);

	if sArmType == "S" and sArmCol == "H" then
		sArmType = "SH";
		sArmCol = string.sub(sArmCode,4,4);
	elseif sArmCat == "C" and sArmType == "L" then
		sArmCat = "M";
		sArmType = "CL";
	elseif sArmType then
		sArmType = " "..sArmType;
	end
	
	local result = 0;
	if iI == 1 then
		if sArmCat then
			result = string.find("HML",sArmCat);
		end
	elseif iI == 2 then
		if sArmType then
			result = string.find(" H SCL C G L BSH",sArmType);
			if result then
				result = (result+1)/2;
			end
		end
	elseif iI == 3 then
		if sArmCol then
			result = string.find("WYPTG",sArmCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- QualityCodeIndex: returns a quality index from a Quality Code.
-- sQCode string: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function QualityCodeIndex(sQCode)

	if sQCode == nil then
		return 0;
	end
	if type(sQCode) ~= "string" then
		return 0;
	end

	local sQtyCode = string.match(sQCode,"(%a+)");
	if not sQtyCode then
		return 0;
	end
	sQtyCode = string.upper(sQtyCode).." ";

	local sQtyCol = string.sub(sQtyCode,1,1);
	
	local result = 0;
	if sQtyCol then
		result = string.find("WYPTG",sQtyCol);
	end
	
	if result then
		return result;
	else
		return 0;
	end
	
end

-- WpnCodeIndex: returns a specified index from a Weapon Code.
-- sWCode string:
-- 1st position: H=heavy, L=light
-- 2nd position: O=one-handed, T=two-handed, B=bow
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function WpnCodeIndex(sWCode, iI)

	if sWCode == nil or iI == nil then
		return 0;
	end
	if type(sWCode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sWpnCode = string.match(sWCode,"(%a+)");
	if not sWpnCode then
		return 0;
	end
	sWpnCode = string.upper(sWpnCode).."   ";
	
	local sWpnCat = string.sub(sWpnCode,1,1);
	local sWpnType = string.sub(sWpnCode,2,2);
	local sWpnCol = string.sub(sWpnCode,3,3);

	local result = 0;
	if iI == 1 then
		if sWpnCat then
			result = string.find("HL",sWpnCat);
		end
	elseif iI == 2 then
		if sWpnType then
			result = string.find("OTB",sWpnType);
		end
	elseif iI == 3 then
		if sWpnCol then
			result = string.find("WYPTG",sWpnCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- RomanRankDecode: converts a string with a Roman number in characters, to an integer number.
-- used for Legendary Item Title calculation.

local RomanCharsToValues = {["M"]=1000,["CM"]=900,["D"]=500,["CD"]=400,["C"]=100,["XC"]=90,["L"]=50,["XL"]=40,["X"]=10,["IX"]=9,["V"]=5,["IV"]=4,["I"]=1};

function RomanRankDecode(sNumber)

	if sNumber ~= nil and type(sNumber) == "string" and sNumber ~= "" then
		for sRomanRankChar, iRomanRankValue in pairs(RomanCharsToValues) do
			if string.sub(string.upper(sNumber),1,string.len(sRomanRankChar)) == sRomanRankChar then
				return iRomanRankValue+RomanRankDecode(string.sub(sNumber,string.len(sRomanRankChar)+1));
			end
		end
	end
	
	return 0;
	
end

-- ReverseCalc: tries to calculate back a calculation result to the original (integer) level.
-- Does not support N.

function ReverseCalc(sStat, dNum)

	if string.len(string.match(sStat,"([-%w]+)")) > 0 then
		local dNumMinus = dNum-DblCalcDev;
		local dNumPlus = dNum+DblCalcDev;
		local minlvl = 1;
		local maxlvl = 549;
		local devlvl = 3;
	
		local left = minlvl-1;
		local right = maxlvl;
		local middle;
		
		local count = minlvl;

		while right > left+1 and count <= maxlvl do
			count = count+1;
			middle = math.floor((left+right)/2);
			if CalcStat(sStat,middle) >= dNumMinus then
				right = middle;
			else
				left = middle;
			end
		end

		local mintest = right-devlvl;
		if mintest < minlvl then
			mintest = minlvl;
		end
		local maxtest = right+devlvl;
		if maxtest > maxlvl then
			maxtest = maxlvl;
		end

		local dFound;
	
		-- we check nearby in case the progression is not completely ascending/sorted.
		for test = mintest, maxtest do
			dFound = CalcStat(sStat,test);
			if dNumMinus <= dFound and dFound <= dNumPlus then
				return test;
			end
		end
	end

	return 0;

end

-- ****************** Misc. floating point support functions ******************

-- Misc. functions for floating point rounding.
-- 2nd parameter is number of decimals.

function RoundDbl(dNum, vDec)

	local dCorrection = 0.5+DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblDown(dNum, vDec)

	local dCorrection = DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblUp(dNum, vDec)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblLotro(dNum)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end

	local iNumCeiled = math.floor(iSign*dNum+dCorrection);
	if iNumCeiled <= 1000 then
		return iSign*iNumCeiled;
	end

	local iFactor = 1;
	local iTestNum = math.floor(iNumCeiled/1000);
	while iTestNum > 0 do
		iTestNum = math.floor(iTestNum/10);
		iFactor = iFactor*10;
	end

	return iSign*math.floor(iNumCeiled/iFactor+dCorrection)*iFactor;

end

function RoundDblProg(dNum)

	local dCorrection = 0.5+DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	
	local dTestNum = dNum/(0.5*(iSign*63));
	local dDec = -math.floor(math.log10(dTestNum));

	if -DblCalcDev <= dDec and dDec <= DblCalcDev then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^dDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end

end

-- to be used by other modules
p.DblCalcDev = DblCalcDev;
p.CalcStat = CalcStat;

-- ******************************* End CalcStat *******************************
